#!/bin/bash

# 获取当前路径
basedir=$(dirname "$(realpath "${BASH_SOURCE[0]}")")

# 读取配置文件
if [ -f "${basedir}/function.sh" ]; then
    . ${basedir}/function.sh
fi

# 读取环境变量
if [ -f "$MANGO_ENV" ]; then
   . $MANGO_ENV
fi

# 安装包名称，安装目录定义
PACKAGE=kafka_${KAFKA_VER}.tgz
INS_DIR=${PACKAGE%.tgz}

# 检测安装状态
KAFKA_HOME=${SOFT_INSTALL_DIR}/${INS_DIR}
if [ -d "${KAFKA_HOME}" ]; then
    _info "本机已经安装 ${INS_DIR} ."
    exit 0
fi

# 检测/安装 JDK 环境
if ! command -v java &>/dev/null; then
    ${basedir}/jdk.sh
fi

# 配置 KAFKA 环境变量
update_env_marked KAFKA_ENV "$(cat <<EOF
KAFKA_HOME=${SOFT_INSTALL_DIR}/${INS_DIR}
pathmunge \$KAFKA_HOME/bin
EOF
)"

# 准备软件安装包
if [ -f "${PACKAGE_DIR}/${PACKAGE}" ]; then
    _info "本机已存在软件包 ${PACKAGE} ."
else
    retry_until_success curl -sL -o ${PACKAGE_DIR}/${PACKAGE} ${SERVER_URL}/${PACKAGE}
    _log "下载 >>> ${PACKAGE} ."
fi

mkdir -p ${KAFKA_HOME}/{data,logs}
tar xf ${PACKAGE_DIR}/${PACKAGE} --strip-components=1 -C ${KAFKA_HOME}
_log "解压 >>> ${PACKAGE}"

# 获取 broker.id
for NODE in ${KAFKA[@]}; do
    let i+=1
    [ "$NODE" == "$LAN_IP_ADDRESS" ] && BROKER_ID=$i
done

if [ -n "$BROKER_ID" ]; then
    _log "本节点 broker.id=$BROKER_ID ."
else
    _err "获取节点 broker.id 失败 ."
fi

# 获取 zk 节点信息
zk_list=$(lookup_hostname_with_port 2181 ${ZOOKEEPER[@]})
if [ -n "$zk_list" ]; then
    _log "获取 zk 节点列表: $zk_list ."
else
    _err "获取 zk 节点失败 ."
fi

# 创建配置文件
cat > ${KAFKA_HOME}/config/server.properties <<EOF
#auto.create.topics.enable                 = true
broker.id                                  = ${BROKER_ID}
group.initial.rebalance.delay.ms           = 0
listeners                                  = PLAINTEXT://${LAN_IP_ADDRESS}:9092
log.dirs                                   = ${KAFKA_HOME}/data
log.retention.check.interval.ms            = 300000
log.retention.hours                        = 168
log.segment.bytes                          = 1073741824
log.cleaner.enable                         = true
log.cleanup.policy                         = delete
num.io.threads                             = 8
num.network.threads                        = 6
num.partitions                             = 1
num.recovery.threads.per.data.dir          = 1
offsets.topic.replication.factor           = 1
socket.receive.buffer.bytes                = 102400
socket.request.max.bytes                   = 104857600
socket.send.buffer.bytes                   = 102400
transaction.state.log.min.isr              = 1
transaction.state.log.replication.factor   = 1
zookeeper.connect                          = ${zk_list}
zookeeper.connection.timeout.ms            = 6000
EOF
_log "创建配置文件"

. /etc/profile &>/dev/null
cat > /usr/lib/systemd/system/kafka.service <<EOF
[Unit]
Description=kafka
After=network.target
Wants=zookeeper.service

[Service]
Type=simple
LimitCORE=infinity
LimitNPROC=1000000
LimitNOFILE=1000000
Restart=always
RestartSec=3
TimeoutSec=10
Environment="JAVA_HOME=${JAVA_HOME}"
WorkingDirectory=${KAFKA_HOME}
ExecStart=/bin/bash bin/kafka-server-start.sh config/server.properties
ExecStop=/bin/bash bin/kafka-server-stop.sh
SuccessExitStatus=143
StandardOutput=null

[Install]
WantedBy=multi-user.target
EOF
_log "配置系统服务 ."
systemctl daemon-reload

add_service_to_rc_local kafka
_log "添加服务启动命令到 /etc/rc.d/rc.local ."