#!/bin/bash

# 获取当前路径
basedir=$(dirname "$(realpath "${BASH_SOURCE[0]}")")

# 读取配置文件
if [ -f "${basedir}/function.sh" ]; then
    . ${basedir}/function.sh
fi

# 读取环境变量
if [ -f "$MANGO_ENV" ]; then
   . $MANGO_ENV
fi

# 安装包名称
PACKAGE=flink-${FLINK_VER}-bin-scala_2.12.tgz
INS_DIR=${PACKAGE%.tgz}

# 检测安装状态
FLINK_HOME=${SOFT_INSTALL_DIR}/${INS_DIR}
if [ -d "${FLINK_HOME}" ]; then
    _info "本机已经安装 ${INS_DIR} ."
    exit 0
fi

# 检测/安装 JDK 环境
if ! command -v java &>/dev/null; then
    ${basedir}/jdk.sh
fi

# 配置 FLINK 环境变量
update_env_marked FLINK_ENV "$(cat <<EOF
FLINK_HOME=${SOFT_INSTALL_DIR}/${INS_DIR}
pathmunge \$FLINK_HOME/bin
EOF
)"

# 准备软件安装包
if [ -f "${PACKAGE_DIR}/${PACKAGE}" ]; then
    _info "本机已存在软件包 ${PACKAGE} ."
else
    retry_until_success curl -sL -o ${PACKAGE_DIR}/${PACKAGE} ${SERVER_URL}/${PACKAGE}
    _log "下载 >>> ${PACKAGE} ."
fi

mkdir -p ${FLINK_HOME}
tar xf ${PACKAGE_DIR}/${PACKAGE} --strip-components=1 -C ${FLINK_HOME}
_log "解压 >>> ${PACKAGE} ."


# 计算任务驱动
if [ ! -f "${PACKAGE_DIR}/flink-opertor-1.4-SNAPSHOT.jar" ]; then
    retry_until_success curl -sL -o ${PACKAGE_DIR}/flink-opertor-1.6.2.jar ${SERVER_URL}/flink-opertor-1.6.2.jar
    _log "下载 >>> flink-opertor-1.6.2.jar ."
fi
#cp ${PACKAGE_DIR}/flink-opertor-1.6.2.jar ${FLINK_HOME}/lib/flink-opertor-1.6.2.jar

# 角色分配
# 节点数量大于或等于5个, 则前3个为 jobmanager, 其他全部为 taskmanager
if (( ${#FLINK[@]} >= 5 )); then
    jobmanager=${FLINK[@]:0:3}
    taskmanager=${FLINK[@]:3}
    deploy_type=cluster
else
# 节点数量少于5个, 则第1个为 jobmanager, 其他全部为 taskmanager
    jobmanager=${FLINK[0]}
    taskmanager=${FLINK[@]:1}
    deploy_type=standalone
fi

# 如果当前角色为 jobmanager 填充变量 master_node(当前节点IP)
for jobmanager_ip in $jobmanager; do
    if [ "$jobmanager_ip" == "${LAN_IP_ADDRESS}" ]; then
        master_ip="${LAN_IP_ADDRESS}"
    fi
done

# 如果当前角色为 jobmanager 则 jobmanager 节点IP为自己, 否则为第一个节点
jobmanager_node=$(lookup_ip_hostname ${master_ip:-${FLINK[0]}})
sed -i "s|#rest.port: 8081|rest.port: 8087|" ${FLINK_HOME}/conf/flink-conf.yaml
sed -i "s|^rest.address:.*|rest.address: ${jobmanager_node}|" ${FLINK_HOME}/conf/flink-conf.yaml
sed -i "s|^rest.bind-address:.*|rest.bind-address: 0.0.0.0|" ${FLINK_HOME}/conf/flink-conf.yaml
sed -i "s|^jobmanager.bind-host:.*|jobmanager.bind-host: 0.0.0.0|" ${FLINK_HOME}/conf/flink-conf.yaml
sed -i "s|^jobmanager.rpc.address:.*|jobmanager.rpc.address: ${jobmanager_node}|" ${FLINK_HOME}/conf/flink-conf.yaml
sed -i "s|^taskmanager.host:.*|taskmanager.host: $(hostname -f)|" ${FLINK_HOME}/conf/flink-conf.yaml
sed -i "s|^taskmanager.bind-host:.*|taskmanager.bind-host: 0.0.0.0|" ${FLINK_HOME}/conf/flink-conf.yaml
sed -i "s|^taskmanager.numberOfTaskSlots:.*|taskmanager.numberOfTaskSlots: 5|" ${FLINK_HOME}/conf/flink-conf.yaml
sed -i "s|# io.tmp.dirs:.*|io.tmp.dirs: ${FLINK_HOME}/tmpdir|"  ${FLINK_HOME}/conf/flink-conf.yaml

sed -i "/rest.flamegraph.enabled/d" ${FLINK_HOME}/conf/flink-conf.yaml
sed -i "/rest.bind-address/a\rest.flamegraph.enabled: true" ${FLINK_HOME}/conf/flink-conf.yaml

## task 资源配置(使用二分之一CPU，三分之一内存)
CPU2Z1=$(($(nproc)/2))
MEM3Z1=$(($(awk '/MemTotal/{print $2}' /proc/meminfo) / 1024 / 1024 / 3 * 1024))
sed -i "/taskmanager.cpu.cores/d" ${FLINK_HOME}/conf/flink-conf.yaml
sed -i "/taskmanager.numberOfTaskSlots/a\taskmanager.cpu.cores: ${CPU2Z1}" ${FLINK_HOME}/conf/flink-conf.yaml
sed -i "s|taskmanager.memory.process.size:.*|taskmanager.memory.process.size: ${MEM3Z1}m|g" ${FLINK_HOME}/conf/flink-conf.yaml

## 集群配置
grep -q '## HC' ${FLINK_HOME}/conf/flink-conf.yaml || cat <<EOF >> ${FLINK_HOME}/conf/flink-conf.yaml
## HC
state.backend: filesystem
state.savepoints.dir: file://${FLINK_HOME}/data/savepoints
state.checkpoints.dir: file://${FLINK_HOME}/data/checkpoints
historyserver.archive.fs.dir: file://${FLINK_HOME}/data/history
jobmanager.archive.fs.dir: file://${FLINK_HOME}/data/jobmanager

# 保留最近3个成功的checkpoint
execution.checkpointing.max-retained-checkpoints: 3

# 作业停止时自动删除checkpoint（默认RETAIN_ON_CANCELLATION）
execution.checkpointing.externalized-checkpoint-retention: DELETE_ON_CANCELLATION

## HA
high-availability.type: zookeeper
high-availability.storageDir: file://${FLINK_HOME}/data/recovery
high-availability.zookeeper.quorum: $(lookup_hostname_with_port 2181 $jobmanager)
high-availability.zookeeper.path.root: /flink-ha
high-availability.cluster-id: /default_ns


env.pid.dir: ${FLINK_HOME}/pid
#env.java.home: /usr/java/jdk17.0.12
#env.java.opts: --add-exports java.base/sun.net.util=ALL-UNNAMED --add-opens java.base/java.lang=ALL-UNNAMED --add-opens java.base/java.util=ALL-UNNAMED --add-opens java.base/java.text=ALL-UNNAMED
EOF

# 非高可用集群
if [ "$deploy_type" == 'standalone' ]; then
    sed -i "s@^env.java.home@#env.java.home@" ${FLINK_HOME}/conf/flink-conf.yaml
    sed -i "s@^env.java.opts@#env.java.opts@" ${FLINK_HOME}/conf/flink-conf.yaml
    sed -i "s@^high-availability@#high-availability@" ${FLINK_HOME}/conf/flink-conf.yaml
fi

# 配置预处理日志级别为 ERROR
grep -q com.hc.preprocess ${FLINK_HOME}/conf/logback.xml || sed -i '/Suppress the irrelevant/i\    <logger name="com.hc.preprocess" level="ERROR" />\n' ${FLINK_HOME}/conf/logback.xml

# 配置服务
. /etc/profile &>/dev/null
function CFG_SYSTEMCTL() {
    local NAME="$1" START_CMD="$2" STOP_CMD="$3"
    cat > /usr/lib/systemd/system/${NAME}.service <<-EOF
	[Unit]
	Description=Flink ${NAME}
	After=syslog.target network.target

	[Service]
	LimitCORE=infinity
	LimitNPROC=1000000
	LimitNOFILE=1000000
	Restart=always
	RestartSec=10
	TimeoutSec=10
	Type=forking
	User=root
	Group=root
	Environment="JAVA_HOME=${JAVA_HOME}"
	WorkingDirectory=${FLINK_HOME}
	ExecStart=$START_CMD
	ExecStop=$STOP_CMD
	SuccessExitStatus=143

	[Install]
	WantedBy=multi-user.target
	EOF

    _log "配置系统服务 ${NAME} ."
    systemctl daemon-reload

    add_service_to_rc_local ${NAME}
    _log "添加服务启动命令到 /etc/rc.d/rc.local ."

    systemctl restart ${NAME}
    _log "启动服务 ${NAME} ."
}

# 配置服务
for role in jobmanager historyserver taskmanager; do
    # 将变量转换为数组
    role_array=(${!role})

    #echo "$role  --->  ${role_array[@]}"

    # 判断当前IP是否存在于数组中
    if contains "$LAN_IP_ADDRESS" role_array[@]; then
        case $role in
            jobmanager)
                JOBMANAGER=($jobmanager)
                if [ "$deploy_type" == 'cluster' ]; then
                    sed -i '/server/d' ${FLINK_HOME}/conf/zoo.cfg
                    sed -i '/dataDir/d' ${FLINK_HOME}/conf/zoo.cfg
                    sed -i "/# The directory/a\dataDir=${FLINK_HOME}/zookeeper" ${FLINK_HOME}/conf/zoo.cfg
                    for IP in ${JOBMANAGER[@]}; do
                        let i+=1
                        NODE=$(lookup_ip_hostname $IP)
                        sed -i "\$a server.$i=${NODE}:2888:3888" ${FLINK_HOME}/conf/zoo.cfg
                        [ "$IP" == "$LAN_IP_ADDRESS" ] && ID=$i
                    done
                    CFG_SYSTEMCTL "flink-zookeeper" "bash ./bin/zookeeper.sh start $ID" "bash ./bin/zookeeper.sh stop"
                fi
                CFG_SYSTEMCTL "flink-jobmanager" "bash ./bin/jobmanager.sh start" "bash ./bin/jobmanager.sh stop"
                ;;
            historyserver)
                CFG_SYSTEMCTL "flink-historyserver" "bash ./bin/historyserver.sh start" "bash ./bin/historyserver.sh stop"
				mkdir -p ${FLINK_HOME}/data/history
                ;;
            taskmanager)
                CFG_SYSTEMCTL "flink-taskmanager" "bash ./bin/taskmanager.sh start" "bash ./bin/taskmanager.sh stop"
				mkdir -p ${FLINK_HOME}/data/{savepoints,checkpoints}
                ;;
        esac
    fi
done


# 配置预处理
# flink run-application -t yarn-application -p 8 -yjm 1024M -Dtaskmanager.memory.process.size=2048mb -c com.hc.preprocess.app.ProcessAPP /home/mango/process/process-2.0.1.0013.jar /home/mango/process/application.properties
kafka_list=$(lookup_ips_with_port 9092 ${KAFKA[@]})
mkdir -p /home/mango/process
cat > /home/mango/process/application.properties <<EOF
## kafka
kafka_brokers=${kafka_list}

## topic
kafka_topic=DCTopic

## topic
alarm_topic=alarmTopic

## topic
alarm_data_point=alarm_data_point
mango_process_bak=mango_process_bak

## boxId
cdc_host=${MYSQL[0]}
cdc_port=3306
cdc_username=root
cdc_password=${MYSQL_PASS}
cdc_database=loong
cdc_table=loong.box_terminal,loong.t_mpoint,loong.t_alarm_condition

## checkpoint
file=file:///home/mango/process/.checkpoint
flink_port=8087
testResultTopic=testResultTopic

## MangoDB
mango_url=${MONGODB[0]}:27017
mango_user=admin
mango_password=${MONGODB_PASS}

## TDEngine
td_url=jdbc:TAOS://${TDENGINE[0]}:6030/mango
td_user=root
td_password=taosdata

## redis
redis_url=redis://${REDIS[0]}:6379
redis_database=1
redis_password=${REDIS_PASS}

timestamp=$(date +%s%3N)
EOF
_log "创建预处理配置 application.properties ."

# 配置计算任务
# flink run-application -t yarn-application -p 3 -yjm 1024M -Dtaskmanager.memory.process.size=2048mb -c com.hc.task.app.CalcTaskAPP /home/mango/calc-task/calc_task_flink-1.1.0.0013.jar /home/mango/calc-task/task.properties
mkdir -p /home/mango/calc-task
cat > /home/mango/calc-task/task.properties <<EOF
flink_port=8117
check_point=file:///home/mango/calc-task/.checkpoint

## kafka
topic=calc
kafka_group=mango_calc_bak
recalcDataTopic=recalcDataTopic
kafka_brokers=${kafka_list}

## TDEngine
td_url=jdbc:TAOS://${TDENGINE[0]}:6030/mango
td_user=root
td_password=taosdata
td_count=10

## Mysql
mysql_url=jdbc:mysql://${MYSQL[0]}:3306/loong?useUnicode=true&characterEncoding=utf-8&zeroDateTimeBehavior=convertToNull&useSSL=false
mysql_user=root
mysql_password=${MYSQL_PASS}
mysql_driver=com.mysql.cj.jdbc.Driver
mysql_count=100

## log
kafka_log=calc_log

## cdc
cdc_host=${MYSQL[0]}
cdc_port=3306
cdc_username=root
cdc_password=${MYSQL_PASS}
cdc_database=loong
cdc_table=loong.t_calc_opertor
EOF

_log "创建计算任务配置 application.properties ."