#!/bin/bash

# 获取当前路径
basedir=$(dirname "$(realpath "${BASH_SOURCE[0]}")")

# 读取配置文件
if [ -f "${basedir}/function.sh" ]; then
    . ${basedir}/function.sh
fi

# 读取环境变量
if [ -f "$MANGO_ENV" ]; then
   . $MANGO_ENV
fi

# 安装包名称，安装目录定义
PACKAGE=hbase-${HBASE_VER}-bin.tar.gz
INS_DIR=${PACKAGE%.tar.gz}

# 检测安装状态
HBASE_HOME=${SOFT_INSTALL_DIR}/${INS_DIR}
if [ -d "${HBASE_HOME}" ]; then
    _info "本机已经安装 ${INS_DIR} ."
    exit 0
fi

# 检测/安装 JDK 环境
if ! command -v java &>/dev/null; then
    ${basedir}/jdk.sh
fi

# 配置 HBASE 环境变量
update_env_marked HBASE_ENV "$(cat <<EOF
HBASE_HOME=${SOFT_INSTALL_DIR}/${INS_DIR}
pathmunge \$HBASE_HOME/bin
EOF
)"

# 准备软件安装包
if [ -f "${PACKAGE_DIR}/${PACKAGE}" ]; then
    _info "本机已存在软件包 ${PACKAGE} ."
else
    retry_until_success curl -sL -o ${PACKAGE_DIR}/${PACKAGE} ${SERVER_URL}/${PACKAGE}
    _log "下载 >>> ${PACKAGE} ."
fi

mkdir -p ${HBASE_HOME}/logs
tar xf ${PACKAGE_DIR}/${PACKAGE} --strip-components=1 -C ${HBASE_HOME}
_log "解压 >>> ${PACKAGE}"


# 角色分配
# 节点数量大于等于5个, 则前2个为 namenode, 其他全部为 datanode
if (( ${#HBASE[@]} >= 5 )); then
    master=${HBASE[@]:0:2}
    regionserver=${HBASE[@]:2}
    HBASE_ROOTDIR=hdfs://mghdpha/hbase
else
    master=${HBASE[0]}
    regionserver=${HBASE[@]:1}
	HBASE_ROOTDIR=hdfs://$(lookup_hostname_with_port 9000 ${HADOOP[0]})/hbase
fi

# hbase 集群配置
cat > /tmp/hbase.env <<EOF
HBASE_CONF_hbase_rootdir=${HBASE_ROOTDIR}
HBASE_CONF_hbase_cluster_distributed=true
HBASE_CONF_hbase_zookeeper_quorum=$(lookup_ip_hostname ${ZOOKEEPER[@]})
HBASE_CONF_hbase_zookeeper_property_clientPort=2181
HBASE_CONF_hbase_zookeeper_property_dataDir=${HBASE_HOME}/zookeeper
HBASE_CONF_hbase_tmp_dir=${HBASE_HOME}/tmpdir
HBASE_CONF_hbase_client_scanner_timeout_period=180000
HBASE_CONF_zookeeper_session_timeout=120000
HBASE_CONF_zookeeper_rpc_timeout=300000
HBASE_CONF_hbase_hregion_majorcompaction=0
HBASE_CONF_hbase_regionserver_thread_compaction_large=5
HBASE_CONF_hbase_regionserver_thread_compaction_small=5
HBASE_CONF_hbase_regionserver_thread_compaction_throttle=10737418240
HBASE_CONF_hbase_regionserver_regionSplitLimit=150
HBASE_CONF_hfile_block_cache_size=0
EOF
_log "创建 hbase 配置文件"

# yarn资源不足造
# https://www.cnblogs.com/jizhong/p/16828300.html

green_echo ">>> 集群角色信息 ."
printf "%-20s %s\n" " master:" "${master}"
printf "%-20s %s\n" " regionserver:" "${regionserver}"


# 设置模板内容为变量
while read line; do
    eval export $line
done < <(egrep -v '^$|^#' /tmp/hbase.env)
_log "读取模板变量设置为全局变量 ."

# 拼接 hbase 配置
function addProperty() {
    local path=$1
    local name=$2
    local value=$3
    local entry="  <property>\n    <name>${name}</name>\n    <value>${value}</value>\n  </property>"
    local escapedEntry=$(printf '%s' "$entry" | sed 's/\//\\\//g')
    sed -i "/<\/configuration>/ s/.*/${escapedEntry}\n&/" $path
}

# 读取环境变量解析参数&& 参数值
function configure() {
    local path=$1
    local envPrefix=$2
    local var value
	green_echo ">>> Configuring $(basename ${path})"

    while IFS='=' read -r line; do
        if [[ $line =~ ^${envPrefix}_([^=]+) ]]; then
            c=${BASH_REMATCH[1]}
            name=${c//___/-}
            name=${name//__/@}
            name=${name//_/.}
            name=${name//@/_}
            value=${!line}
            echo " - Setting $name=$value"
            addProperty $path $name $value
            eval unset "$line"
        fi
    done < <(sort -r < <(grep "^$envPrefix" < <(compgen -v)))
}

# 备份默认配置
[ -f ${HBASE_HOME}/conf/hbase-site.xml.example ] || cat ${HBASE_HOME}/conf/hbase-site.xml > ${HBASE_HOME}/conf/hbase-site.xml.example

# 生成 hbase 配置
if [ ! -f ${HBASE_HOME}/conf/.init ]; then
    configure ${HBASE_HOME}/conf/hbase-site.xml HBASE_CONF
    _log "生成配置文件 hbase-site.xml ."
    echo "# $(date +'%F %T') hbase-site.xml" > ${HBASE_HOME}/conf/.init
fi

# 配置 hbase
rm -f ${HBASE_HOME}/conf/backup-masters ${HBASE_HOME}/conf/regionservers
echo "${HBASE[1]}" >> ${HBASE_HOME}/conf/backup-masters
for node in $regionserver; do echo "$node" >> ${HBASE_HOME}/conf/regionservers; done

# 配置 ssh 端口
echo "export HBASE_SSH_OPTS='-p $SSH_PORT'" >> ${HBASE_HOME}/conf/hbase-env.sh

# 禁用内置 zk
sed -i 's/# export HBASE_MANAGES_ZK=true/export HBASE_MANAGES_ZK=false/' ${HBASE_HOME}/conf/hbase-env.sh

# systemctl 启动失败处理
grep -qr '/etc/profile' ${HBASE_HOME}/bin/hbase-daemon.sh || sed -ie '2a\. /etc/profile\n' ${HBASE_HOME}/bin/hbase-daemon.sh

# 配置服务
. /etc/profile &>/dev/null
function CFG_SYSTEMCTL() {
    local NAME="$1" START_CMD="$2" STOP_CMD="$3" WANTS="$4"
    cat > /usr/lib/systemd/system/${NAME}.service <<-EOF
	[Unit]
	Description=hbase ${NAME}
	After=syslog.target network.target

	[Service]
	LimitCORE=infinity
	LimitNPROC=1000000
	LimitNOFILE=1000000
	Restart=always
	RestartSec=3
	TimeoutSec=10
	Type=forking
	User=root
	Group=root
	Environment="JAVA_HOME=${JAVA_HOME}"
	WorkingDirectory=${HBASE_HOME}
	ExecStart=$START_CMD
	ExecStop=$STOP_CMD
	SuccessExitStatus=143

	[Install]
	WantedBy=multi-user.target
	EOF

    # 添加依赖服务
	[ -n "${WANTS}" ] && sed -i "/After=/a\Wants=${WANTS}/" /usr/lib/systemd/system/${NAME}.service

    true &&_log "配置系统服务 ${NAME} ."
    systemctl daemon-reload

    add_service_to_rc_local ${NAME}
    _log "添加服务启动命令到 /etc/rc.d/rc.local ."
}

# 配置服务
for role in master regionserver; do
    # 将变量转换为数组
    role_array=(${!role})

    #echo "$role  --->  ${role_array[@]}"

    # 判断当前IP是否存在于数组中
    if contains "$LAN_IP_ADDRESS" role_array[@]; then
        case $role in
            master)
                CFG_SYSTEMCTL "hbase-master" "/bin/bash bin/hbase-daemon.sh --config conf start master" "/bin/bash bin/hbase-daemon.sh --config conf stop master" \
                              "hdp-namenode.service zookeeper.service"
                ;;
            regionserver)
                CFG_SYSTEMCTL "hbase-regionserver" "/bin/bash bin/hbase-daemon.sh --config conf start regionserver" "/bin/bash bin/hbase-daemon.sh --config conf stop regionserver" \
				              "hdp-namenode.service zookeeper.service"
                ;;
        esac
    fi
done

# HBASE_PID_DIR