#!/bin/bash

# 获取当前路径
basedir=$(dirname "$(realpath "${BASH_SOURCE[0]}")")

# 读取配置文件
if [ -f "${basedir}/function.sh" ]; then
    . ${basedir}/function.sh
fi

# 读取环境变量
if [ -f "$MANGO_ENV" ]; then
   . $MANGO_ENV
fi

# 安装包名称，安装目录定义  opentsdb-${OPENTSDB_VER}.noarch.rpm
PACKAGE=opentsdb-${OPENTSDB_VER}.tgz
INS_DIR=${PACKAGE%-*}

# 检测 opentsdb 环境
if command -v tsdb &>/dev/null; then
    _info "本机已经安装 ${PACKAGE} ."
    exit 0
fi

# 检测/安装 JDK 环境
if ! command -v java &>/dev/null; then
    ${basedir}/jdk.sh
fi

# 准备软件安装包
OPENTSDB_HOME=${SOFT_INSTALL_DIR}/${INS_DIR}
if [ -f "${PACKAGE_DIR}/${PACKAGE}" ]; then
    _info "本机已存在软件包 ${PACKAGE} ."
else
    retry_until_success curl -sL -o ${PACKAGE_DIR}/${PACKAGE} ${SERVER_URL}/${PACKAGE}
    _log "下载 >>> ${PACKAGE} ."
fi

mkdir -p ${OPENTSDB_HOME}/pkgs
tar xf ${PACKAGE_DIR}/${PACKAGE} --strip-components=1 -C ${OPENTSDB_HOME}/pkgs
_log "解压 >>> ${PACKAGE}"

# 安裝 opentsdb
dnf install -y gnuplot-minimal &> /tmp/opentsdb.log
dnf install -y ${OPENTSDB_HOME}/pkgs/opentsdb*.rpm &>> /tmp/opentsdb.log && rm -rf ${OPENTSDB_HOME}/pkgs
_log "安装 opentsdb ."

# 配置 opentsdb
[ -f /etc/opentsdb/opentsdb.conf.example ] || cat /etc/opentsdb/opentsdb.conf > /etc/opentsdb/opentsdb.conf.example
cat <<EOF   > /etc/opentsdb/opentsdb.conf
# 启动时预加载所有 UID 缓存到内存，加快查询速度（增加启动时间和内存占用）
tsd.core.preload_uid_cache       = true

# 自动为新指标创建 UID，简化新指标接入流程
tsd.core.auto_create_metrics     = true

# 启用 OpenTSDB 自带的 Web 管理界面
tsd.core.enable_ui               = true

# 启用 REST API 接口，允许通过 HTTP 操作数据
tsd.core.enable_api              = true

# 服务监听端口（需确保防火墙放行此端口）
tsd.network.port                 = 14242

# Web UI 静态资源路径（HTML/CSS/JS 文件目录）
tsd.http.staticroot              = /usr/share/opentsdb/static

# # HTTP 请求临时缓存目录
tsd.http.cachedir                = ${OPENTSDB_HOME}/tmp

# # 启用 HTTP 分块传输编码（支持流式传输大请求）
tsd.http.request.enable_chunked  = true

# # 单个 HTTP 分块的最大字节数（单位：字节）
tsd.http.request.max_chunk       = 65535

# # HBase 的 ZooKeeper 集群地址
tsd.storage.hbase.zk_quorum      = $(lookup_ip_hostname ${ZOOKEEPER[@]})

# # 查询超时时间（0 表示无超时限制，生产环境建议设置合理值）
tsd.query.timeout                = 0

# 过滤条件（如通配符）最大返回条目数（防内存溢出）
tsd.query.filter.expansion_limit = 65535

# 启用 TCP Keep-Alive 机制提升长连接性能
tsd.network.keep_alive           = true

# 等待处理的最大连接队列长度（高并发需调大）
tsd.network.backlog              = 3072

# 自动修复重复数据点（根据时间戳去重，牺牲部分写入性能）
tsd.storage.fix_duplicates       = true

# 启用 HBase 的 append 操作（需 HBase 版本支持）
# - 作用：允许通过 HBase 的 append 接口追加数据，减少写入延迟
# - 优点：提升高并发写入性能，适合频繁更新的场景
# - 风险：若 HBase 版本不支持或集群未配置追加功能，可能导致写入失败
tsd.storage.enable_appends       = false

# 启用存储自动合并（Compaction）功能
# - 作用：自动合并 HBase 中的小文件（HFile），优化存储结构
# - 优点：提升查询性能和磁盘空间利用率
# - 缺点：合并过程会占用大量 I/O 和 CPU 资源，可能影响实时写入性能
tsd.storage.enable_compaction    = false
EOF
_log "配置 opentsdb."

# 创建数据目录
mkdir -p ${OPENTSDB_HOME}/{tmp,logs}
chown -Rf opentsdb:opentsdb ${OPENTSDB_HOME}

# 配置服务
. /etc/profile &>/dev/null
cat <<EOF   > /usr/lib/systemd/system/opentsdb.service
[Unit]
Description=OpenTSDB on port
After=network-online.target
Before=shutdown.target
Wants=zookeeper.service hbase-regionserver.service

[Service]
LimitCORE=infinity
LimitNPROC=1000000
LimitNOFILE=1000000
Restart=always
RestartSec=3
TimeoutSec=10
Type=simple
User=opentsdb
Group=opentsdb
Environment="JAVA_HOME=${JAVA_HOME}" "JAVA=${JAVA_HOME}/bin/java"
Environment='JVMARGS=-Xmx4096m -DLOG_FILE=${OPENTSDB_HOME}/log/opentsdb.log -DQUERY_LOG=${OPENTSDB_HOME}/log/queries.log -XX:+ExitOnOutOfMemoryError -enableassertions -enablesystemassertions'
ExecStart=/usr/bin/tsdb tsd --config /etc/opentsdb/opentsdb.conf
StandardOutput=journal
EOF

_log "配置系统服务 opentsdb ."
systemctl daemon-reload

add_service_to_rc_local opentsdb
_log "添加服务启动命令到 /etc/rc.d/rc.local ."