#!/bin/bash

# 获取当前路径
basedir=$(dirname "$(realpath "${BASH_SOURCE[0]}")")

# 读取配置文件
if [ -f "${basedir}/function.sh" ]; then
    . ${basedir}/function.sh
fi

# 读取环境变量
if [ -f "$MANGO_ENV" ]; then
   . $MANGO_ENV
fi

# FE/BE 角色分配
if [ "${#DORIS[@]}" -ge 5 ]; then
    FE_NODE="${DORIS[@]:0:3}"
    BE_NODE="${DORIS[@]:3}"
else
    FE_NODE="${DORIS[0]}"
    BE_NODE="${DORIS[@]:1}"
fi

if [[ "${FE_NODE}" =~ "$LAN_IP_ADDRESS" ]]; then
    # 安装包名称
    ROLE=fe
    PACKAGE=apache-doris-${ROLE}-${DORIS_VER}-bin-x86_64.tar.xz
    INS_DIR=${PACKAGE%.tar.xz}
else
    # CPU 是否支持 avx2 指令集
    if grep -q avx2 /proc/cpuinfo; then
        avx2=
    else
        avx2='-noavx2'
    fi

    ROLE=be
    PACKAGE=apache-doris-${ROLE}-${DORIS_VER}-bin-x86_64${avx2}.tar.xz
    INS_DIR=${PACKAGE%.tar.xz}
fi

# 检测安装状态
DORIS_HOME=${SOFT_INSTALL_DIR}/${INS_DIR}
if [ -d "${DORIS_HOME}" ]; then
    _info "本机已经安装 ${INS_DIR} ."
    exit 0
fi

# 检测/安装 JDK 环境
if ! command -v java &>/dev/null; then
    ${basedir}/jdk.sh
fi

# 配置 DORIS 环境变量
update_env_marked DORIS_ENV "$(cat <<EOF
DORIS_HOME=${SOFT_INSTALL_DIR}/${INS_DIR}
pathmunge \$DORIS_HOME/bin
EOF
)"

# 准备软件安装包
if [ -f "${PACKAGE_DIR}/${PACKAGE}" ]; then
    _info "本机已存在软件包 ${PACKAGE} ."
else
    retry_until_success curl -sL -o ${PACKAGE_DIR}/${PACKAGE} ${SERVER_URL}/${PACKAGE}
    _log "下载 >>> ${PACKAGE} ."
fi

mkdir -p ${DORIS_HOME}
tar xf ${PACKAGE_DIR}/${PACKAGE} --strip-components=1 -C ${DORIS_HOME}
_log "解压 >>> ${PACKAGE} ."

# MySQL-Client
if [ "$ROLE" == 'fe' ]; then
    # 安装依赖
    dnf install -y ncurses-compat-libs &>/dev/null
    _log "安装 ncurses-compat-libs ."

    PACKAGE=mysql-client-${MYSQL_VER}.tar.gz
    if [ -f "${PACKAGE_DIR}/${PACKAGE}" ]; then
        _info "本机已存在软件包 ${PACKAGE} ."
    else
        retry_until_success curl -sL -o ${PACKAGE_DIR}/${PACKAGE} ${SERVER_URL}/${PACKAGE}
        _log "下载 >>> ${PACKAGE} ."
    fi

    tar xf ${PACKAGE_DIR}/${PACKAGE} -C ${PACKAGE_DIR}
    _log "解压 >>> ${PACKAGE} ."
    fetch_and_install_rpm ${PACKAGE_DIR}/mysql-client-${MYSQL_VER}/mysql-community-common-${MYSQL_VER}-1.el7.x86_64.rpm
    fetch_and_install_rpm ${PACKAGE_DIR}/mysql-client-${MYSQL_VER}/mysql-community-libs-${MYSQL_VER}-1.el7.x86_64.rpm
    fetch_and_install_rpm ${PACKAGE_DIR}/mysql-client-${MYSQL_VER}/mysql-community-client-${MYSQL_VER}-1.el7.x86_64.rpm

    # 创建配置
    cat > ${DORIS_HOME}/conf/${ROLE}.conf <<-EOF
	priority_networks                = ${LAN_IP_ADDRESS%.*}.0/24
	qe_max_connection                = 65535
	max_conn_per_user                = 1024
	sys_log_roll_num                 = 10
	audit_log_roll_num               = 10

	LOG_DIR                          = ${DORIS_HOME}/log
	DATE                             = \`date +%Y%m%d-%H%M%S\`
	JAVA_OPTS                        = "-Xmx8192m -XX:+UseMembar -XX:SurvivorRatio=8 -XX:MaxTenuringThreshold=7 -XX:+PrintGCDateStamps -XX:+PrintGCDetails -XX:+UseConcMarkSweepGC -XX:+UseParNewGC -XX:+CMSClassUnloadingEnabled -XX:-CMSParallelRemarkEnabled -XX:CMSInitiatingOccupancyFraction=80 -XX:SoftRefLRUPolicyMSPerMB=0 -Xloggc:$DORIS_HOME/log/fe.gc.log.$DATE"
	JAVA_OPTS_FOR_JDK_9              = "-Xmx8192m -XX:SurvivorRatio=8 -XX:MaxTenuringThreshold=7 -XX:+CMSClassUnloadingEnabled -XX:-CMSParallelRemarkEnabled -XX:CMSInitiatingOccupancyFraction=80 -XX:SoftRefLRUPolicyMSPerMB=0 -Xlog:gc*:$DORIS_HOME/log/fe.gc.log.$DATE:time"
	sys_log_level                    = INFO
	http_port                        = 8130
	rpc_port                         = 9020
	query_port                       = 9030
	edit_log_port                    = 9010
	mysql_service_nio_enabled        = true
	EOF
    _log "创建 doris-${ROLE} 配置文件 ."
fi

# JniUtil
if [ "$ROLE" == 'be' ]; then
    # apache-doris-dependencies-${DORIS_VER}-bin-x86_64.tar.xz
    PACKAGE=java-udf-jar-with-dependencies.jar
    if [ ! -f "${PACKAGE_DIR}/${PACKAGE}" ]; then
        retry_until_success curl -sL -o ${PACKAGE_DIR}/${PACKAGE} ${SERVER_URL}/${PACKAGE}
        _log "下载 >>> ${PACKAGE} ."
    else
        true && _info "本机已存在软件包 ${PACKAGE} ."
    fi
    cp ${PACKAGE_DIR}/${PACKAGE} ${DORIS_HOME}/lib/

    # 创建配置
    cat > ${DORIS_HOME}/conf/${ROLE}.conf <<-EOF
	priority_networks                = ${LAN_IP_ADDRESS%.*}.0/24
	PPROF_TMPDIR                     = ${DORIS_HOME}/log
	sys_log_level                    = INFO
	webserver_port                   = 8140
	brpc_port                        = 8060
	heartbeat_service_port           = 9050
	be_port                          = 9060
	EOF

    _log "创建 doris-${ROLE} 配置文件 ."
fi


# 配置 systemctl 启动
. /etc/profile &>/dev/null
cat > /usr/lib/systemd/system/doris-${ROLE}.service <<EOF
[Unit]
Description=cosmo-bdp doris-${ROLE}
After=network.target

[Service]
LimitCORE=infinity
LimitNPROC=1000000
LimitNOFILE=1000000
Restart=always
RestartSec=15
TimeoutSec=10
Type=forking
User=root
Group=root
Environment="JAVA_HOME=${JAVA_HOME}"
WorkingDirectory=${DORIS_HOME}
ExecStartPre=/usr/sbin/sysctl -w vm.max_map_count=2000000
ExecStartPre=/bin/bash -c "/usr/bin/echo never > /sys/kernel/mm/transparent_hugepage/enabled"
ExecStartPre=/bin/bash -c "/usr/bin/echo never > /sys/kernel/mm/transparent_hugepage/defrag"
ExecStart=/bin/bash bin/start_${ROLE}.sh --daemon
ExecStop=/bin/bash bin/stop_${ROLE}.sh --daemon

[Install]
WantedBy=multi-user.target
EOF

_log "配置系统服务 ."
systemctl daemon-reload

add_service_to_rc_local doris-${ROLE}
_log "添加服务启动命令到 /etc/rc.d/rc.local ."


### 初始化
# # 如果是高可用集群，且为第二/三个 FE 节点(关闭主节点服务[重复安装异常]，其他节点启动服务)
# if [ "${#DORIS[@]}" -ge 5 ] && [[ "${DORIS[1]} ${DORIS[2]}" =~ "$LAN_IP_ADDRESS" ]]; then
#     systemctl stop doris-${ROLE}
# else
#     systemctl restart doris-${ROLE}
#     _log "启动 doris 服务 doris-${ROLE} ."
# fi

# # 如果是集群，第一个 FE 节点
# if [ "$LAN_IP_ADDRESS" == "${DORIS[0]}" ]; then
#     until mysql -h127.0.0.1 -P9030 -e "SHOW PROC '/frontends'" &>/dev/null; do
#         _info "等待 doris-${ROLE} 服务启动就绪 ."
#         sleep 2
#     done
#     _log "doris-${ROLE} 服务启动就绪 ."

#     for be in $BE_NODE; do
#         mysql -h127.0.0.1 -P9030 -e "ALTER SYSTEM ADD BACKEND '${be}:9050'"
#         _log "节点 ${be}:9050 加入 backends ."
#     done

#     # 如果是高可用集群，注册其他 FE
#     if [ "${#DORIS[@]}" -ge 5 ]; then
#         mysql -h127.0.0.1 -P9030 -e "ALTER SYSTEM ADD FOLLOWER '${DORIS[1]}:9010'"
#         mysql -h127.0.0.1 -P9030 -e "ALTER SYSTEM ADD FOLLOWER '${DORIS[2]}:9010'"
#         _log "节点 ${DORIS[1]}:9010  ${DORIS[2]}:9010 加入 frontends ."
#     fi

#     #mysql -h127.0.0.1 -P9030 -e "SHOW PROC '/backends'\G" 
#     #mysql -h127.0.0.1 -P9030 -e "SHOW PROC '/frontends'\G"

#     _info "frontends"
#     awk '
#         BEGIN { FS=": "; OFS="  " }
#         NR%4==1 { 
#             if (NR > 1) print line
#             line = "\t" $0
#         }
#         NR%4==2 { line = line OFS $0 }
#         NR%4==3 { line = line OFS $0 }
#         NR%4==0 { line = line OFS $0 }
#         END {print line}
#         ' <<<$(sed 's/   //g' < <(egrep 'IsMaster|IP|HostName|Alive' < <(mysql -h127.0.0.1 -P9030 -e "SHOW PROC '/frontends'\G")))

#     _info "backends"
#     awk '
#         BEGIN { FS=": "; OFS="  " }
#         NR%4==1 { 
#             if (NR > 1) print line
#             line = "\t" $0
#         }
#         NR%4==2 { line = line OFS $0 }
#         NR%4==3 { line = line OFS $0 }
#         NR%4==0 { line = line OFS $0 }
#         END {print line}
#         ' <<<$(sed 's/   //g' < <(egrep 'BackendId|IP|HostName|Alive' < <(mysql -h127.0.0.1 -P9030 -e "SHOW PROC '/backends'\G")))

#     # 修改登录密码  https://www.cnblogs.com/whiteY/p/18375883
#     mysql -h127.0.0.1 -P9030 -e "SET PASSWORD FOR 'root' = PASSWORD('doris123');"
# fi

# # 如果是高可用集群，且为第二/三个 FE 节点
# if [ "${#DORIS[@]}" -ge 5 ] && [[ "${DORIS[1]} ${DORIS[2]}" =~ "$LAN_IP_ADDRESS" ]]; then
#     nohup ${DORIS_HOME}/bin/start_fe.sh --helper "${DORIS[0]}:9010" &>/dev/null &
#     HELPER_PID=$!

#     while : ; do
#         eval $(mysql -h127.0.0.1 -P9030 -e "SHOW PROC '/frontends'\G" 2>/dev/null | sed -n "/$LAN_IP_ADDRESS/,/CurrentConnected/p" | egrep "Join|Alive" | sed 's@: @=@')
#         if [[ "$Join" == "true" && "$Alive" == "true" ]]; then
#             _info "同步 ${DORIS[0]}:9010 FE 节点数据同步完成 ."
#             break
#         else
#             _info "等待同步 ${DORIS[0]}:9010 FE 节点数据 ."
#         fi
#         sleep 5
#     done

#     kill $HELPER_PID
#     sleep 3
#     while ps -aux | grep -v grep | grep -q 'org.apache.doris.PaloFe -helper'; do
#         ps -aux | grep -v grep | grep 'org.apache.doris.PaloFe -helper' | awk '{print $2}' | xargs kill -9
#         sleep 3
#     done
#     systemctl start doris-${ROLE}
#     _log "启动 doris 服务 doris-${ROLE} ."

#     until mysql -h127.0.0.1 -P9030 -e "SHOW PROC '/frontends'" &>/dev/null; do
#         _log "等待 doris-${ROLE} 服务启动就绪 ."
#         sleep 2
#     done
#     _log "doris-${ROLE} 服务启动就绪 ."

#     green_echo ">>> frontends: "
#     awk '
#         BEGIN { FS=": "; OFS="  " }
#         NR%4==1 { 
#             if (NR > 1) print line
#             line = "\t" $0
#         }
#         NR%4==2 { line = line OFS $0 }
#         NR%4==3 { line = line OFS $0 }
#         NR%4==0 { line = line OFS $0 }
#         END {print line}
#         ' <<<$(sed 's/   //g' < <(egrep 'IsMaster|IP|HostName|Alive' < <(mysql -h127.0.0.1 -P9030 -e "SHOW PROC '/frontends'\G")))

#     green_echo ">>> backends: "
#     awk '
#         BEGIN { FS=": "; OFS="  " }
#         NR%4==1 { 
#             if (NR > 1) print line
#             line = "\t" $0
#         }
#         NR%4==2 { line = line OFS $0 }
#         NR%4==3 { line = line OFS $0 }
#         NR%4==0 { line = line OFS $0 }
#         END {print line}
#         ' <<<$(sed 's/   //g' < <(egrep 'BackendId|IP|HostName|Alive' < <(mysql -h127.0.0.1 -P9030 -e "SHOW PROC '/backends'\G")))
# fi

## 创建库
# mysql -h127.0.0.1 -P9030 -e 'CREATE DATABASE blueberry_data;'

## 备份
#  mysqldump -h127.0.0.1 -P9030 --no-tablespaces --databases blueberry_data > blueberry_data.sql

## 还原
# mysql -h127.0.0.1 -P9030 < blueberry_data.sql

## 查看
# mysql -h127.0.0.1 -P9030 -e 'use blueberry_data; show tables;'