#!/bin/bash

# 获取当前路径
basedir=$(dirname "$(realpath "${BASH_SOURCE[0]}")")

# 读取配置文件
if [ -f "${basedir}/function.sh" ]; then
    . ${basedir}/function.sh
fi

# 读取环境变量
if [ -f "$MANGO_ENV" ]; then
   . $MANGO_ENV
fi

# 安装包名称
PACKAGE=mysql-${MYSQL_VER}-linux-glibc2.12-x86_64.tar.gz
INS_DIR=${PACKAGE%.tar.gz}

# 检测安装状态
MYSQL_HOME=${SOFT_INSTALL_DIR}/${INS_DIR}
if [ -d "${MYSQL_HOME}" ]; then
    _info "本机已经安装 ${INS_DIR} ."
    exit 0
fi

# 创建服务运行用户
if ! id mysql &>/dev/null; then
    groupadd -g 27 -o -r mysql
    useradd -N -g mysql -o -r -M -d /var/log/mysql -s /sbin/nologin -c "MySQL Server" -u 27 mysql
    _log "创建服务运行用户 ."
fi

# 安装依赖
dnf install -y ncurses-compat-libs &>/dev/null
_log "安装 ncurses-compat-libs ."

# 配置 MYSQL 环境变量
update_env_marked MYSQL_ENV "$(cat <<EOF
MYSQL_HOME=${SOFT_INSTALL_DIR}/${INS_DIR}
pathmunge \$MYSQL_HOME/bin
EOF
)"

# 准备软件安装包
if [ -f "${PACKAGE_DIR}/${PACKAGE}" ]; then
    _info "本机已存在软件包 ${PACKAGE} ."
else
    retry_until_success curl -sL -o ${PACKAGE_DIR}/${PACKAGE} ${SERVER_URL}/${PACKAGE}
    _log "下载 >>> ${PACKAGE} ."
fi

mkdir -p ${MYSQL_HOME}
tar xf ${PACKAGE_DIR}/${PACKAGE} --strip-components=1 -C ${MYSQL_HOME}
_log "解压 >>> ${PACKAGE} ."

# 获取 server_id
for NODE in ${MYSQL[@]}; do
    let i+=1
    [ "$NODE" == "$LAN_IP_ADDRESS" ] && { server_id=$i; break; }
done

# 创建配置文件
cat > ${MYSQL_HOME}/my.cnf <<EOF
[mysqld]
skip-ssl
skip-host-cache
skip-name-resolve
disable-partition-engine-check
server-id                       =  ${server_id:-${LAN_IP_ADDRESS##*.}}
log-bin                         =  binlog-${server_id}
gtid_mode                       =  ON
enforce-gtid-consistency        =  ON
binlog_format                   =  ROW
expire_logs_days                =  7
# slave-skip-errors             =  all
log_bin_trust_function_creators =  1
default-time_zone               =  '+8:00'
basedir                         =  ${MYSQL_HOME}
datadir                         =  ${MYSQL_HOME}/data
socket                          =  /var/log/mysql/mysqld.sock
pid-file                        =  /var/log/mysql/mysqld.pid
log-error                       =  /var/log/mysql/mysqld.log
max_binlog_size                 =  1024M
character-set-server            =  utf8
symbolic-links                  =  0
max_connections                 =  50000
max_connect_errors              =  3000
wait_timeout                    =  30000
interactive_timeout             =  30000
lower_case_table_names          =  1
sql_mode                        =  STRICT_TRANS_TABLES,NO_ZERO_IN_DATE,NO_ZERO_DATE,ERROR_FOR_DIVISION_BY_ZERO,NO_AUTO_CREATE_USER,NO_ENGINE_SUBSTITUTION
default-storage-engine          =  INNODB
explicit_defaults_for_timestamp =  true
slow_query_log                  =  on
long_query_time                 =  5
slow_query_log_file             =  /var/log/mysql/slow_query.log

[mysql]
default-character-set           =  utf8
socket                          =  /var/log/mysql/mysqld.sock

[client]
default-character-set           =  utf8
socket                          =  /var/log/mysql/mysqld.sock
EOF
_log "创建 MySQL 配置 my.cnf ."
install -m 644 ${MYSQL_HOME}/my.cnf /etc/my.cnf

# 创建数据目录 && 初始化
if [ ! -d "${MYSQL_HOME}/data/mysql" ]; then
    install -o mysql -g mysql -d /var/log/mysql
    install -o mysql -g mysql -d /var/log/mysql
    install -o mysql -g mysql -d ${MYSQL_HOME}/data
    ${MYSQL_HOME}/bin/mysqld --defaults-file=/etc/my.cnf --initialize-insecure --user=mysql --ssl
    _log "初始化 MySQL ."
fi

# 配置服务
cat <<-EOF   > /usr/lib/systemd/system/mysqld.service
[Unit]
Description=MySQL Server
Documentation=man:mysqld(7)
Documentation=http://dev.mysql.com/doc/refman/en/using-systemd.html
After=network.target

[Install]
WantedBy=multi-user.target

[Service]
LimitCORE=infinity
LimitNPROC=1000000
LimitNOFILE=1000000
User=mysql
Group=mysql
PIDFile=/var/log/mysql/mysqld.pid
ExecStart=${MYSQL_HOME}/bin/mysqld_safe --defaults-file=/etc/my.cnf
ExecStop=/bin/sh -c "/bin/kill -s TERM \$(/bin/cat /var/log/mysql/mysqld.pid)"
Restart=always
RestartSec=10
TimeoutSec=0
PrivateTmp=false
EOF

_log "配置系统服务 ."
systemctl daemon-reload

add_service_to_rc_local mysqld
_log "添加服务启动命令到 /etc/rc.d/rc.local ."

systemctl start mysqld
_log "启动 MySQL 服务 ."


# 安装备份
XTRABACKUP_HOME=/usr/local/xtrabackup
if [ ! -d "${XTRABACKUP_HOME}" ]; then
    green_echo ">>> 安装 XtraBackup ."

    # 安装包名称
    PACKAGE=percona-xtrabackup-2.4.27.tgz

    # 配置 XTRABACKUP 环境变量
    update_env_marked XTRABACKUP_ENV "$(cat <<-EOF
	XTRABACKUP_HOME=${XTRABACKUP_HOME}
	pathmunge \$XTRABACKUP_HOME/bin
	EOF
	)"

    # 准备软件安装包
    if [ -f "${PACKAGE_DIR}/${PACKAGE}" ]; then
        _info "本机已存在软件包 ${PACKAGE} ."
    else
        retry_until_success curl -sL -o ${PACKAGE_DIR}/${PACKAGE} ${SERVER_URL}/${PACKAGE}
        _log "下载 >>> ${PACKAGE} ."
    fi

     mkdir -p "${XTRABACKUP_HOME}"
    _log "创建 XtraBackup 目录."

    tar xf ${PACKAGE_DIR}/${PACKAGE} --strip-components=1 -C ${XTRABACKUP_HOME}
    _log "解压 >>> ${PACKAGE} ."

fi

# 第一个节点为主,其他所有节点为从
MYSQL_MASTER=${MYSQL[0]}
MYSQL_SLAVE=${MYSQL[@]:1}

# 配置主从复制
[ -f ~/.my.cnf ] && rm -f ~/.my.cnf
if [ "$LAN_IP_ADDRESS" == "${MYSQL[0]}" ]; then
    green_echo ">>> 配置主从复制 ."

    # 检查第一个节点是否准备就绪
    until nc -vw 1 ${MYSQL[0]} -z 3306 &>/dev/null; do
        sleep 2
        echo "等待节点 ${MYSQL[0]} 准备就绪 ."
    done

    # 创建管理员账号 && 允许远程连接 && 修改 root 密码
    ${MYSQL_HOME}/bin/mysql -e "CREATE USER 'root'@'%' IDENTIFIED BY '${MYSQL_PASS}'"
    ${MYSQL_HOME}/bin/mysql -e "GRANT ALL PRIVILEGES ON *.* TO 'root'@'%' WITH GRANT OPTION"
    ${MYSQL_HOME}/bin/mysql -e "CREATE USER '${MYSQL_USER}'@'%' IDENTIFIED BY '${MYSQL_PASS}'"
    ${MYSQL_HOME}/bin/mysql -e "GRANT ALL PRIVILEGES ON *.* TO '${MYSQL_USER}'@'%' WITH GRANT OPTION"
    ${MYSQL_HOME}/bin/mysql -e "FLUSH PRIVILEGES"
    # ${MYSQL_HOME}/bin/mysql -e "SHOW GRANTS FOR '${MYSQL_USER}'@'%'"
    # ${MYSQL_HOME}/bin/mysql -e "GRANT ALL ON *.* TO 'root'@'%' IDENTIFIED BY '${MYSQL_PASS}'"
    ${MYSQL_HOME}/bin/mysql -e "SELECT host,user FROM mysql.user"
    ${MYSQL_HOME}/bin/mysql -e "ALTER USER 'root'@'%' IDENTIFIED BY '${MYSQL_PASS}'"
    ${MYSQL_HOME}/bin/mysql -e "ALTER USER 'root'@'localhost' IDENTIFIED BY '${MYSQL_PASS}'"
    # 列出用户 select user,host,password_last_changed from mysql.user;

    # 配置免密码登录 MySQL
    cat > ~/.my.cnf <<-EOF
	[client]
	user                            = '${MYSQL_USER}'
	password                        = '${MYSQL_PASS}'
	EOF
    chmod 600 ~/.my.cnf

    # 创建复制用户
    # for NODE in ${MYSQL[@]:1}; do
    #     ${MYSQL_HOME}/bin/mysql -e "CREATE USER 'repl2024'@'${NODE}' IDENTIFIED BY '${MYSQL_PASS}';"
    #     ${MYSQL_HOME}/bin/mysql -e "GRANT REPLICATION SLAVE ON *.* TO 'repl2024'@'${NODE}';"
    # done

    # 创建复制用户
    ${MYSQL_HOME}/bin/mysql -e "CREATE USER 'repl2024'@'%' IDENTIFIED BY '${MYSQL_PASS}';"
    ${MYSQL_HOME}/bin/mysql -e "GRANT REPLICATION SLAVE ON *.* TO 'repl2024'@'%';"

    ${MYSQL_HOME}/bin/mysql -e "FLUSH PRIVILEGES;"
    ${MYSQL_HOME}/bin/mysql -e "SHOW MASTER STATUS;"
else
    until nc -vw 1 ${MYSQL[0]} -z 3306 &>/dev/null; do
        sleep 2
        echo "等待节点 ${MYSQL[0]} 准备就绪 ."
    done

    sleep 3
    # MySQL Slave 参数
    grep -q 'relay-log' /etc/my.cnf || sed -i '/binlog_format/a\relay-log                       =  relay-bin' /etc/my.cnf
    grep -q 'read_only' /etc/my.cnf || sed -i '/relay-log/a\read_only                       =  1' /etc/my.cnf
    systemctl restart mysqld && sleep 5

    ${MYSQL_HOME}/bin/mysql -e "CHANGE MASTER TO MASTER_HOST='${MYSQL[0]}', MASTER_USER='repl2024', MASTER_PASSWORD='${MYSQL_PASS}', MASTER_AUTO_POSITION=1;"
    ${MYSQL_HOME}/bin/mysql -e "START SLAVE;"
    egrep '_Running:|Master_Log_' < <(${MYSQL_HOME}/bin/mysql -e "SHOW SLAVE STATUS\G;")

    (
        sleep 10;  
        cat > /root/.my.cnf <<-EOF
		[client]
		user                            = '${MYSQL_USER}'
		password                        = '${MYSQL_PASS}'
		EOF
        chmod 600 ~/.my.cnf
    ) &
fi

if [ "$LAN_IP_ADDRESS" == "${MYSQL[0]}" ]; then
    # 主节点配置数据库备份
    green_echo ">>> 主节点配置数据库备份."

    # 配置备份脚本
    mkdir -p /script
    install -m 755 ${basedir}/innobackup.sh /script/innobackup.sh
    sed -i "s|^MYSQL_PASS=.*|MYSQL_PASS='${MYSQL_PASS}'|" ${basedir}/innobackup.sh /script/innobackup.sh

    # 添加定时任务
    crontab -l 2>/dev/null | { cat; echo "10 1 * * * /script/innobackup.sh -e /var/lib/xtrabackup -R root@${MYSQL[1]} -D $SOFT_INSTALL_DIR/backupmysql -b /home/.backup"; } | uniq | crontab -
    crontab -l
fi

## mysqladmin flush-hosts





# 主服务器配置
# mysql> grant replication client,replication slave,super,reload  on *.* to 'root'@'fueltank-2' identified by 'BBDERS1@bbdops.com';
# mysql> flush privileges;

# 从服务器配置
# mysql> change master to master_host='fueltank-1', master_port=3306, master_user='root', master_password='BBDERS1@bbdops.com', master_auto_position=1;
# mysql> start slave;
# mysql> show slave status\G;

# mysql -e "show slave status\G" | grep -i "running"