#!/bin/bash

# 获取当前路径
basedir=$(dirname "$(realpath "${BASH_SOURCE[0]}")")

# 读取配置文件
if [ -f "${basedir}/function.sh" ]; then
    . ${basedir}/function.sh
fi

# 读取环境变量
if [ -f "$MANGO_ENV" ]; then
   . $MANGO_ENV
fi

# 安装包名称
PACKAGE=apache-dolphinscheduler-${DOLPHINSCHEDULER_VER}-bin.tar.gz
INS_DIR=${PACKAGE%.tar.gz}

# 检测安装状态
DOLPHINSCHEDULER_HOME=${SOFT_INSTALL_DIR}/${INS_DIR}
if [ -d "${DOLPHINSCHEDULER_HOME}" ]; then
    _info "本机已经安装 ${INS_DIR} ."
    exit 0
fi

# 检测/安装 JDK 环境
if ! command -v java &>/dev/null; then
    ${basedir}/jdk.sh
fi

# 配置 DOLPHINSCHEDULER 环境变量
update_env_marked DOLPHINSCHEDULER_ENV "$(cat <<EOF
DOLPHINSCHEDULER_HOME=${SOFT_INSTALL_DIR}/${INS_DIR}
pathmunge \$DOLPHINSCHEDULER_HOME/bin
EOF
)"

# 准备软件安装包
if [ -f "${PACKAGE_DIR}/${PACKAGE}" ]; then
    _info "本机已存在软件包 >>> ${PACKAGE} ."
else
    retry_until_success curl -sL -o ${PACKAGE_DIR}/${PACKAGE} ${SERVER_URL}/${PACKAGE}
    _log "下载 >>> ${PACKAGE} ."
fi

mkdir -p ${DOLPHINSCHEDULER_HOME}
tar xf ${PACKAGE_DIR}/${PACKAGE} --strip-components=1 -C ${DOLPHINSCHEDULER_HOME}
_log "解压 >>> ${PACKAGE} ."

# 数据库/zookeeper 驱动
if [ ! -f "${PACKAGE_DIR}/apache-dolphinscheduler-${DOLPHINSCHEDULER_VER}-libs.tgz" ]; then
    retry_until_success curl -sL -o ${PACKAGE_DIR}/apache-dolphinscheduler-${DOLPHINSCHEDULER_VER}-libs.tgz ${SERVER_URL}/apache-dolphinscheduler-${DOLPHINSCHEDULER_VER}-libs.tgz
    _log "下载驱动 >>> apache-dolphinscheduler-${DOLPHINSCHEDULER_VER}-libs.tgz ."
fi

# 配置数据库/zookeeper 驱动
for LIBDIR in master-server worker-server api-server alert-server standalone-server tools; do
    rm -f ${DOLPHINSCHEDULER_HOME}/${LIBDIR}/libs/{curator-recipes-5.3.0.jar,curator-framework-5.3.0.jar,curator-client-5.3.0.jar,zookeeper-3.8.0.jar}
    tar xf ${PACKAGE_DIR}/apache-dolphinscheduler-${DOLPHINSCHEDULER_VER}-libs.tgz -C ${DOLPHINSCHEDULER_HOME}/${LIBDIR}/libs/
    _log "解压驱动 >>> apache-dolphinscheduler-${DOLPHINSCHEDULER_VER}-libs.tgz --> $LIBDIR/libs ."
done

# 定义变量(用于生成配置参数)
DATABASE=mysql
REGISTRY_ZOOKEEPER_CONNECT_STRING=$(lookup_ips_with_port 2181 ${ZOOKEEPER[@]})

# 定义软件安装路径(用于生成配置)
. /etc/profile &>/dev/null
HADOOP_HOME=${HADOOP_HOME:-${SOFT_INSTALL_DIR}/hadoop-${HADOOP_VER}}
FLINK_HOME=${FLINK_HOME:-${SOFT_INSTALL_DIR}/flink-${FLINK_VER}-bin-scala_2.12}
SEATUNNEL_HOME=${SEATUNNEL_HOME:-${SOFT_INSTALL_DIR}/apache-seatunnel-${SEATUNNEL_VER}-bin}

# 创建配置文件
ips=$(lookup_ip_addresses ${DOLPHINSCHEDULER[@]})

# 角色分配
if (( ${#DOLPHINSCHEDULER[@]} >= 5 )); then
    masters=${DOLPHINSCHEDULER[@]:0:2}
    apiServers=${DOLPHINSCHEDULER[@]:0:2}
    alertServer=${DOLPHINSCHEDULER[-1]}
    workers=${DOLPHINSCHEDULER[@]:2}
    workers_list=$(lookup_ips_with_port default ${workers})
else
    masters=${DOLPHINSCHEDULER[0]}
    apiServers=${DOLPHINSCHEDULER[0]}
    alertServer=${DOLPHINSCHEDULER[-1]}
    workers=${DOLPHINSCHEDULER[@]:1}
    workers_list=$(lookup_ips_with_port default ${workers})
fi

# 安装变量
cat > ${DOLPHINSCHEDULER_HOME}/bin/env/install_env.sh <<EOF
sshPort=22
ips="$ips"
masters="$masters"
workers="$workers_list"
alertServer="$alertServer"
apiServers="$apiServers"
installPath=${DOLPHINSCHEDULER_HOME}
deployUser=root
# deployUser=dolphinscheduler
EOF

# 安装配置
cat > ${DOLPHINSCHEDULER_HOME}/bin/env/dolphinscheduler_env.sh <<EOF
# Java
export JAVA_HOME=${JAVA_HOME}

# MySQL
export DATABASE=\${DATABASE:-mysql}
export SPRING_PROFILES_ACTIVE=\${DATABASE}
export SPRING_DATASOURCE_URL="jdbc:mysql://${NACOS[1]}:3306/dolphinscheduler?useUnicode=true&characterEncoding=UTF-8&useSSL=false"
export SPRING_DATASOURCE_USERNAME=root
export SPRING_DATASOURCE_PASSWORD=${MYSQL_PASS}

# Dolphinscheduler
export SPRING_CACHE_TYPE=\${SPRING_CACHE_TYPE:-none}
export MASTER_FETCH_COMMAND_NUM=\${MASTER_FETCH_COMMAND_NUM:-10}
export SPRING_JACKSON_TIME_ZONE=\${SPRING_JACKSON_TIME_ZONE:-UTC}

# Zookeeper
export REGISTRY_TYPE=\${REGISTRY_TYPE:-zookeeper}
export REGISTRY_ZOOKEEPER_CONNECT_STRING=${REGISTRY_ZOOKEEPER_CONNECT_STRING}

# Hadoop
export HADOOP_HOME=\${HADOOP_HOME:-${SOFT_INSTALL_DIR}/hadoop-${HADOOP_VER}}
export HADOOP_CONF_DIR=${HADOOP_HOME}/etc/hadoop

# Seatunnel
export SEATUNNEL_HOME=\${SEATUNNEL_HOME:-${SOFT_INSTALL_DIR}/apache-seatunnel-${SEATUNNEL_VER}-bin}

# PATH
export PATH=\$JAVA_HOME/bin:\$HADOOP_HOME/bin:\$FLINK_HOME/bin:\$SEATUNNEL_HOME/bin:\$PATH
EOF

function CFG_SYSTEMCTL() {
    local NAME="$1" START_CMD="$2" STOP_CMD="$3"
    cat > /usr/lib/systemd/system/${NAME}.service <<-EOF
	[Unit]
	Description=Dolphinscheduler ${NAME}
	After=syslog.target network.target

	[Service]
	LimitCORE=infinity
	LimitNPROC=1000000
	LimitNOFILE=1000000
	Restart=always
	RestartSec=10
	TimeoutSec=10
	Type=forking
	User=root
	Group=root
	Environment="JAVA_HOME=${JAVA_HOME}"
	WorkingDirectory=${DOLPHINSCHEDULER_HOME}
	ExecStart=$START_CMD
	ExecStop=$STOP_CMD
	SuccessExitStatus=143

	[Install]
	WantedBy=multi-user.target
	EOF

    # 启动执行初始化 SQL
    if [ "${LAN_IP_ADDRESS}" == "${DOLPHINSCHEDULER[0]}" ] && [ "${NAME}" == "dolphinscheduler-master" ]; then
        #sed -i "/^ExecStart=/i \ExecStartPre=/bin/sh -c '[ ! -f \"tools/bin/upgrade-schema.sh\" ] || { sh tools/bin/upgrade-schema.sh && mv tools/bin/upgrade-schema.sh tools/bin/upgrade-schema.sh.bak; }; exit 0'" /usr/lib/systemd/system/${NAME}.service
        sed -i "/^ExecStart=/i \ExecStartPre=/bin/sh -c '[ -f \"tools/bin/upgrade-schema.sh\" ] && { sh tools/bin/upgrade-schema.sh && mv tools/bin/upgrade-schema.sh tools/bin/upgrade-schema.sh.bak; }; exit 0'" /usr/lib/systemd/system/${NAME}.service
    fi

    _log "配置系统服务 ${NAME} ."
    systemctl daemon-reload

    add_service_to_rc_local ${NAME}
    _log "添加服务启动命令到 /etc/rc.d/rc.local ."

    systemctl restart ${NAME}
    _log "启动服务 ${NAME} ."
}

# for role in masters apiServers alertServer workers; do
#     role_array=(${!role})
#     # 如果当前IP在某个角色则配置服务
#     echo $role  --  ${!role}  -- ${role_array[@]}
# done

# 如果当前IP在某个角色则配置服务
for role in masters workers apiServers alertServer; do
    # 将变量转换为数组
    role_array=(${!role})

    #echo "$role  --->  ${role_array[@]}"

    # 判断当前IP是否存在于数组中
    if contains "$LAN_IP_ADDRESS" role_array[@]; then
        case $role in
            masters)
                CFG_SYSTEMCTL "dolphinscheduler-master" "bash ./bin/dolphinscheduler-daemon.sh start master-server" "bash ./bin/dolphinscheduler-daemon.sh stop master-server"
                ;;
            workers)
                CFG_SYSTEMCTL "dolphinscheduler-worker" "bash ./bin/dolphinscheduler-daemon.sh start worker-server" "bash ./bin/dolphinscheduler-daemon.sh stop worker-server"
                ;;
            apiServers)
                CFG_SYSTEMCTL "dolphinscheduler-api" "bash ./bin/dolphinscheduler-daemon.sh start api-server" "bash ./bin/dolphinscheduler-daemon.sh stop api-server"
                ;;
            alertServer)
                CFG_SYSTEMCTL "dolphinscheduler-alert" "bash ./bin/dolphinscheduler-daemon.sh start alert-server" "bash ./bin/dolphinscheduler-daemon.sh stop alert-server"
                ;;
        esac
    fi
done

### 配置初始化 SQL
if [ "$LAN_IP_ADDRESS" == "${DOLPHINSCHEDULER[0]}" ]; then
    LM_NODE=${WFW[0]}
    WORK_LIST=$(lookup_ips_with_port 1234 ${workers})
    INIT_DML_SQL_FILE=$DOLPHINSCHEDULER_HOME/tools/sql/sql/dolphinscheduler_mysql.sql
    grep -q '\-- #######' $INIT_DML_SQL_FILE && sed -i '/-- #######/,$d' $INIT_DML_SQL_FILE
    echo -e "\n-- #######" >> $INIT_DML_SQL_FILE
    echo -e "\n-- 创建令牌 Tokeh\nINSERT INTO dolphinscheduler.t_ds_access_token VALUES (0,1,'43d17c8d3687a9b983c88d6ef18d84de','2100-01-01 08:00:00','$(date "+%F %T")','$(date "+%F %T")');" >> $INIT_DML_SQL_FILE
    echo -e "\n-- 创建租户\nINSERT INTO dolphinscheduler.t_ds_tenant VALUES (0,'root','',1,'$(date "+%F %T")','$(date "+%F %T")');" >> $INIT_DML_SQL_FILE
    echo -e "\n-- 创建工作组\nINSERT INTO dolphinscheduler.t_ds_worker_group VALUES (0,'数据集成','${WORK_LIST}','$(date "+%F %T")','$(date "+%F %T")','',NULL);" >> $INIT_DML_SQL_FILE
    echo -e "\n-- 创建环境 SEATUNNEL\nINSERT INTO dolphinscheduler.t_ds_environment VALUES (1,16151081128704,'数据集成','export SEATUNNEL_HOME=$SEATUNNEL_HOME\\\nexport PATH=\$HADOOP_HOME/bin:\$SPARK_HOME1/bin:\$SPARK_HOME2/bin:\$PYTHON_HOME/bin:$JAVA_HOME/bin:\$HIVE_HOME/bin:\$FLINK_HOME/bin:$DATAX_HOME/bin:\$SEATUNNEL_HOME/bin:$CHUNJUN_HOME/bin:\$PATH','数据集成环境',1,'$(date "+%F %T")','$(date "+%F %T")');" >> $INIT_DML_SQL_FILE
    echo -e "INSERT INTO dolphinscheduler.t_ds_environment_worker_group_relation VALUES (1,16151081128704,'数据集成',1,'$(date "+%F %T")','$(date "+%F %T")');" >> $INIT_DML_SQL_FILE
    echo -e "\n-- 创建告警实例\nINSERT INTO dolphinscheduler.t_ds_alert_plugin_instance  VALUES (0,8,'{\\\"headerParams\\\":\\\"{\\\\\\\\\\\"Content-Type\\\\\\\\\\\":\\\\\\\\\\\"application/json\\\\\\\\\\\"}\\\",\\\"requestType\\\":\\\"POST\\\",\\\"WarningType\\\":\\\"all\\\",\\\"bodyParams\\\":\\\"{}\\\",\\\"url\\\":\\\"http://${WFW[0]}:48089/datasync/dolphinscheduler/result\\\",\\\"contentField\\\":\\\"text/plain\\\"}','$(date +'%F %T')','$(date +'%F %T')','任务完成告警');" >> $INIT_DML_SQL_FILE
    echo -e "\n-- 创建告警组关联实例\nINSERT INTO dolphinscheduler.t_ds_alertgroup VALUES (2,'1',1,'任务执行结果返回','','$(date "+%F %T")','$(date "+%F %T")');" >> $INIT_DML_SQL_FILE
    echo -e "\n-- 创建项目\nINSERT INTO dolphinscheduler.t_ds_project VALUES (1,'蓝莓数据集成平台',16151203461760,'',1,1,'$(date "+%F %T")','$(date "+%F %T")');" >> $INIT_DML_SQL_FILE
fi

### 初始化
# # 验证 数据库已经启动
# until nc -vw 1 ${NACOS[1]} -z 3306 &>/dev/null; do
#     sleep 2
#     echo "等待数据库 ${NACOS[1]} 准备就绪 ."
# done

# # 第一个节点初始化
# if [ "$LAN_IP_ADDRESS" == "${DOLPHINSCHEDULER[0]}" ]; then
#     LM_NODE=${WFW[0]}
#     WORK_LIST=$(lookup_ips_with_port 1234 ${workers})
#     INIT_DML_SQL_FILE=$DOLPHINSCHEDULER_HOME/tools/sql/sql/dolphinscheduler_mysql.sql
#     grep -q '\-- #######' $INIT_DML_SQL_FILE && sed -i '/-- #######/,$d' $INIT_DML_SQL_FILE
#     echo -e "-- #######" >> $INIT_DML_SQL_FILE
#     echo -e "\n-- 创建令牌 Tokeh\nINSERT INTO dolphinscheduler.t_ds_access_token VALUES (0,1,'43d17c8d3687a9b983c88d6ef18d84de','2100-01-01 08:00:00','$(date "+%F %T")','$(date "+%F %T")');" >> $INIT_DML_SQL_FILE
#     echo -e "\n-- 创建租户\nINSERT INTO dolphinscheduler.t_ds_tenant VALUES (0,'root','',1,'$(date "+%F %T")','$(date "+%F %T")');" >> $INIT_DML_SQL_FILE
#     echo -e "\n-- 创建工作组\nINSERT INTO dolphinscheduler.t_ds_worker_group VALUES (0,'数据集成','${WORK_LIST}','$(date "+%F %T")','$(date "+%F %T")','',NULL);" >> $INIT_DML_SQL_FILE
#     echo -e "\n-- 创建环境 SEATUNNEL\nINSERT INTO dolphinscheduler.t_ds_environment VALUES (1,16151081128704,'数据集成','export SEATUNNEL_HOME=$SEATUNNEL_HOME\\\nexport PATH=\$HADOOP_HOME/bin:\$SPARK_HOME1/bin:\$SPARK_HOME2/bin:\$PYTHON_HOME/bin:$JAVA_HOME/bin:\$HIVE_HOME/bin:\$FLINK_HOME/bin:$DATAX_HOME/bin:\$SEATUNNEL_HOME/bin:$CHUNJUN_HOME/bin:\$PATH','数据集成环境',1,'$(date "+%F %T")','$(date "+%F %T")');" >> $INIT_DML_SQL_FILE
#     echo -e "INSERT INTO dolphinscheduler.t_ds_environment_worker_group_relation VALUES (1,16151081128704,'数据集成',1,'$(date "+%F %T")','$(date "+%F %T")');" >> $INIT_DML_SQL_FILE
#     #echo -e "\n-- 创建告警实例\nSELECT id INTO @id FROM dolphinscheduler.t_ds_plugin_define WHERE plugin_params!='null' AND plugin_name='Http';" >> $INIT_DML_SQL_FILE
#     #echo -e "INSERT INTO dolphinscheduler.t_ds_alert_plugin_instance  VALUES (0,@id,'{\\\"headerParams\\\":\\\"{\\\\\\\\\\\"Content-Type\\\\\\\\\\\":\\\\\\\\\\\"application/json\\\\\\\\\\\"}\\\",\\\"requestType\\\":\\\"POST\\\",\\\"WarningType\\\":\\\"all\\\",\\\"bodyParams\\\":\\\"{}\\\",\\\"url\\\":\\\"http://${WFW[0]}:48089/datasync/dolphinscheduler/result\\\",\\\"contentField\\\":\\\"text/plain\\\"}','$(date +'%F %T')','$(date +'%F %T')','任务完成告警');" >> $INIT_DML_SQL_FILE
#     echo -e "\n-- 创建告警实例\nINSERT INTO dolphinscheduler.t_ds_alert_plugin_instance  VALUES (0,8,'{\\\"headerParams\\\":\\\"{\\\\\\\\\\\"Content-Type\\\\\\\\\\\":\\\\\\\\\\\"application/json\\\\\\\\\\\"}\\\",\\\"requestType\\\":\\\"POST\\\",\\\"WarningType\\\":\\\"all\\\",\\\"bodyParams\\\":\\\"{}\\\",\\\"url\\\":\\\"http://${WFW[0]}:48089/datasync/dolphinscheduler/result\\\",\\\"contentField\\\":\\\"text/plain\\\"}','$(date +'%F %T')','$(date +'%F %T')','任务完成告警');" >> $INIT_DML_SQL_FILE
#     echo -e "\n-- 创建告警组关联实例\nINSERT INTO dolphinscheduler.t_ds_alertgroup VALUES (2,'1',1,'任务执行结果返回','','$(date "+%F %T")','$(date "+%F %T")');" >> $INIT_DML_SQL_FILE
#     echo -e "\n-- 创建项目\nINSERT INTO dolphinscheduler.t_ds_project VALUES (1,'蓝莓数据集成平台',16151203461760,'',1,1,'$(date "+%F %T")','$(date "+%F %T")');" >> $INIT_DML_SQL_FILE

#     # 查询创建结果(datasync-server-prod.yaml dolphin配置项)
#     # mysql -e "select code from dolphinscheduler.t_ds_project;"
#     # mysql -e "select code from dolphinscheduler.t_ds_environment;"
#     # mysql -e "select token from dolphinscheduler.t_ds_access_token;"
#     # mysql -e "select worker_group from dolphinscheduler.t_ds_environment_worker_group_relation;"
#     # mysql -e "select id from dolphinscheduler.t_ds_alertgroup where group_name='任务执行结果返回';"
#     # mysql -e "select id from dolphinscheduler.t_ds_project"

#     green_echo ">>> 初始化 dolphinscheduler 数据库 ."
#     bash ${DOLPHINSCHEDULER_HOME}/tools/bin/upgrade-schema.sh &>/dev/null
#     if [ $? -eq 0 ]; then
#         _log "数据库初始化成功 ."

#         systemctl restart dolphinscheduler-master dolphinscheduler-api
#         _log "启动 Dolphinscheduler 服务 dolphinscheduler-master / dolphinscheduler-api."
#     else
#         _warn "数据库初始化失败 ."
#     fi
# else
#     until nc -vw 1 ${DOLPHINSCHEDULER[0]} -z 5678 &>/dev/null; do
#         sleep 2
#         _info "等待 Dolphinscheduler master ${DOLPHINSCHEDULER[0]} 准备就绪 ."
#     done

#     # 如果当前IP在某个角色则配置服务
#     for role in masters workers apiServers alertServer; do
#         # 将变量转换为数组
#         role_array=(${!role})

#         # 判断当前IP是否存在于数组中
#         if contains "$LAN_IP_ADDRESS" role_array[@]; then
#             case $role in
#                 masters)
#                     systemctl restart dolphinscheduler-master
#                     _log "启动 Dolphinscheduler 服务 dolphinscheduler-master ."
#                     ;;
#                 workers)
#                     systemctl restart dolphinscheduler-worker
#                     _log "启动 Dolphinscheduler 服务 dolphinscheduler-worker ."
#                     ;;
#                 apiServers)
#                     systemctl restart dolphinscheduler-api
#                     _log "启动 Dolphinscheduler 服务 dolphinscheduler-api ."
#                     ;;
#                 alertServer)
#                     systemctl restart dolphinscheduler-alert
#                     _log "启动 Dolphinscheduler 服务 dolphinscheduler-alert ."
#                     ;;
#             esac
#         fi
#     done
# fi

## 默认账号 (admin/dolphinscheduler123)
# 登录账号 admin 密码修改
# echo -n "123456" | md5sum
# mysql -e "select * from dolphinscheduler.t_ds_user;"
# mysql -e "update dolphinscheduler.t_ds_user set user_password='e10adc3949ba59abbe56e057f20f883e' where user_name='admin';"
# mysql -e "select * from dolphinscheduler.t_ds_user;"


## 备份
# mysqldump --databases dolphinscheduler > dolphinscheduler.sql