#!/bin/bash

# 获取当前路径
basedir=$(dirname "$(realpath "${BASH_SOURCE[0]}")")

# 读取配置文件
if [ -f "${basedir}/function.sh" ]; then
    . ${basedir}/function.sh
fi

# 读取环境变量
if [ -f "$MANGO_ENV" ]; then
   . $MANGO_ENV
fi

# 安装包名称
PACKAGE=mongodb-linux-x86_64-rhel93-${MONGODB_VER}.tgz
INS_DIR=${PACKAGE%.tgz}


# 检测安装状态
MNOGODB_HOME=${SOFT_INSTALL_DIR}/${INS_DIR}
if [ -d "${MNOGODB_HOME}" ]; then
    _info "本机已经安装 ${INS_DIR} ."
    exit 0
fi

# 配置 MNOGODB 环境变量
update_env_marked MNOGODB_ENV "$(cat <<EOF
MNOGODB_HOME=${SOFT_INSTALL_DIR}/${INS_DIR}
pathmunge \$MNOGODB_HOME/bin
EOF
)"

# 准备软件安装包
if [ -f "${PACKAGE_DIR}/${PACKAGE}" ]; then
    _info "本机已存在软件包 ${PACKAGE} ."
else
    retry_until_success curl -sL -o ${PACKAGE_DIR}/${PACKAGE} ${SERVER_URL}/${PACKAGE}
    _log "下载 >>> ${PACKAGE} ."
fi

# 解压服务端
mkdir -p ${MNOGODB_HOME}
tar xf ${PACKAGE_DIR}/${PACKAGE} --strip-components=1 -C ${MNOGODB_HOME}
_log "解压 >>> ${PACKAGE} ."

# 下载客户端
PACKAGE=mongosh-${MONGOSH_VER}-linux-x64.tgz
if [ -f "${PACKAGE_DIR}/${PACKAGE}" ]; then
    _info "本机已存在软件包 ${PACKAGE} ."
else
    retry_until_success curl -sL -o ${PACKAGE_DIR}/${PACKAGE} ${SERVER_URL}/${PACKAGE}
    _log "下载 >>> ${PACKAGE} ."
fi

# 解压客户端
tar xf ${PACKAGE_DIR}/${PACKAGE} --strip-components=1 -C ${MNOGODB_HOME}
_log "解压 >>> ${PACKAGE} ."


# 创建服务运行用户
if ! id mongod &>/dev/null; then
    groupadd -g 893 -o -r mongod
    useradd -N -g mongod -o -r -M -s /sbin/nologin -c "MongoDB Server" -u 896 mongod
    _log "创建服务运行用户 ."
fi

# 创建数据/配置目录
mkdir -p ${MNOGODB_HOME}/{conf,data,log}
chown -Rf mongod:mongod ${MNOGODB_HOME}/{conf,data,log}

# 创建配置文件
cat > ${MNOGODB_HOME}/conf/mongod.conf <<EOF
systemLog:
  destination: file
  logAppend: true
  path: ${MNOGODB_HOME}/log/mongod.log

storage:
  dbPath: ${MNOGODB_HOME}/data
  wiredTiger:
    engineConfig:
      cacheSizeGB: 2

processManagement:
  timeZoneInfo: /usr/share/zoneinfo
  pidFilePath: /run/mongodb/mongod.pid

net:
  port: 27017
  bindIp: 0.0.0.0
  unixDomainSocket:
    enabled: true
    pathPrefix: /run/mongodb

security:
  authorization: disabled
EOF
_log "创建 MongoDB 配置 mongod.conf ."


cat <<-EOF   > /usr/lib/systemd/system/mongodb.service
[Unit]
Description=MongoDB Database Server
Documentation=https://docs.mongodb.org/manual
After=network-online.target
Wants=network-online.target

[Service]
User=mongod
Group=mongod
Environment="OPTIONS=-f ${MNOGODB_HOME}/conf/mongod.conf"
Environment="MONGODB_CONFIG_OVERRIDE_NOFORK=1"
Environment="GLIBC_TUNABLES=glibc.pthread.rseq=0"
ExecStart=${MNOGODB_HOME}/bin/mongod \$OPTIONS
RuntimeDirectory=mongodb
LimitFSIZE=infinity
LimitCPU=infinity
LimitAS=infinity
LimitNOFILE=64000
LimitNPROC=64000
LimitMEMLOCK=infinity
TasksMax=infinity
TasksAccounting=false

PIDFile=/run/mongodb/mongod.pid
ExecStartPre=/usr/bin/mkdir -p /run/mongodb
ExecStartPre=/usr/bin/chown mongod:mongod /run/mongodb
ExecStartPre=/usr/bin/chmod 0755 /run/mongodb
PermissionsStartOnly=true
Restart=always

[Install]
WantedBy=multi-user.target
EOF

_log "配置系统服务 ."
systemctl daemon-reload

add_service_to_rc_local mongodb
_log "添加服务启动命令到 /etc/rc.d/rc.local ."

systemctl restart mongodb
_log "启动 MongoDB 服务 ."


# 配置认证
if ! grep -q "authorization: enabled" ${MNOGODB_HOME}/conf/mongod.conf; then
    # 等待服务准备就绪
    until nc -vw 1 ${LAN_IP_ADDRESS} -z 27017 &>/dev/null; do
        _info "等待 MongoDB 服务准备就绪 ${LAN_IP_ADDRESS}  ."
        sleep 2
    done

    # 创建 root 管理员用户
    MSSAGE1=$(${MNOGODB_HOME}/bin/mongosh --eval "db.getSiblingDB('admin').createUser({ user: 'root', pwd: '$MONGODB_PASS', roles: [{ role: 'root', db: 'admin' }] })")
    _log "创建 mongod 管理用户 ${MSSAGE1}."

    # 创建 datacollect 数据库(测试写入)
    MSSAGE2=$(${MNOGODB_HOME}/bin/mongosh --eval "db = db.getSiblingDB('datacollect'); db.createCollection('dummy'); db.dummy.insertOne({ name: 'initialData' })")
    _log "创建 datacollect 库写入测试数据 ${MSSAGE2}."

    # 创建 admin 用户并授予读写权限
    MSSAGE3=$(${MNOGODB_HOME}/bin/mongosh --eval "db.getSiblingDB('datacollect').createUser({ user: 'admin', pwd: '$MONGODB_PASS', roles: [{ role: 'readWrite', db: 'datacollect' }] })")
    _log "创建 datacollect 库连接用户及读写权限 ${MSSAGE3}."

    # 启用认证(配置修改后好像是立即生效的)
    sed -i "s@authorization: disabled@authorization: enabled@" ${MNOGODB_HOME}/conf/mongod.conf
    _log "启用 mongodb 连接认证 ."

    # 登录测试
    MSSAGE4=$(${MNOGODB_HOME}/bin/mongosh --eval -u admin -p $MONGODB_PASS --authenticationDatabase datacollect --eval "db.getSiblingDB('datacollect').dummy.find()")
    _log "登录测试查询 ${MSSAGE4}."

    # 自动清理
    mkdir -p /script
    cat <<-ERF    > /script/mongodb.sh
		#!/bin/bash

		# 获取 30 天前日期
		deletetime=\$(date -d "30 day ago" +%F)

		# 连接 mongodb
		mongodb='${MNOGODB_HOME}/bin/mongosh 127.0.0.1:27017'

		\$mongodb <<EOF
		use datacollect
		db.auth('admin', '${MONGODB_PASS}')
		db.rawData.remove({time:{\\\$lt:ISODate("\${deletetime}").toISOString()}})
		db.exceptionData.remove({time:{\\\$lt:ISODate("\${deletetime}").toISOString()}})
		exit;
		EOF
		ERF

    chmod +x /script/mongodb.sh
    grep -q mongodb.sh < <(crontab -l 2>/dev/null) || (crontab -l 2>/dev/null; echo "30 2 * * * /script/mongodb.sh") | crontab -
    _log "配置自动清理策略 ."
fi

systemctl restart mongodb
_log "重启 MongoDB 服务 ."