#!/bin/bash

# 获取当前路径
basedir=$(dirname "$(realpath "${BASH_SOURCE[0]}")")

# 读取配置文件
if [ -f "${basedir}/function.sh" ]; then
    . ${basedir}/function.sh
fi

# 读取环境变量
if [ -f "$MANGO_ENV" ]; then
   . $MANGO_ENV
fi

# 检测/安装 JDK 环境
if [ ! -d ${SOFT_INSTALL_DIR}/zulu* ]; then
    ${basedir}/zulu-jdk.sh
fi

# 检测安装状态
if rpm -q nginx &>/dev/null; then
    _info "本机已经安装 $(rpm -qa nginx) ."
    exit 0
fi

# 安装包名称
PACKAGE=nginx-${NGINX_VER}-ngx.el9.x86_64.rpm

# 准备软件安装包
if [ -f "${PACKAGE_DIR}/${PACKAGE}" ]; then
    _info "本机已存在软件包 ${PACKAGE} ."
else
    retry_until_success curl -sL -o ${PACKAGE_DIR}/${PACKAGE} ${SERVER_URL}/${PACKAGE}
    _log "下载 >>> ${PACKAGE} ."
fi

# 安装 nginx
fetch_and_install_rpm ${PACKAGE_DIR}/nginx-${NGINX_VER}-ngx.el9.x86_64.rpm

# 添加服务器（参数检查+防重复）
add_server() {
    local upstream="$1"
    local servers="$2"
    local params="${3:-weight=2 max_fails=2 fail_timeout=30s}"
    local conf="$4"
    local IFS=','
    
    # 检查是否已存在
    for server in $servers; do
        if ! grep -q "^upstream $upstream" "$conf"; then
            echo "Upstream block not exist: $upstream"
            return 1
        fi
        
        # 构造完整server行
        local target_line="server $server $params;"
        
        # 不存在时追加到块末尾
        if ! sed -n "/upstream $upstream {/,/}/p" "$conf" | grep -q "$target_line"; then
            sed -i "/upstream $upstream {/a\    $target_line" "$conf"
        fi
    done
}

# 删除服务器
del_server() {
    local upstream="$1"
    local servers="$2"
    local conf="$3"
    local IFS=','

    # 删除匹配带参数的 server 行
    for server in $servers; do
        sed -i "/upstream $upstream {/,/}/ {
            /^[[:space:]]*server $server[[:space:];]/d
        }" "$conf"
    done
}

# 删除微服务代理配置
rm -f /etc/nginx/conf.d/{mght.conf,mgweb.conf,tsdb.conf}

# 配置 nacos
add_server nacos_server "$(lookup_ips_with_port 8848 ${NACOS_NODE})" '' /etc/nginx/conf.d/nacos.conf
del_server nacos_server 127.0.0.1:6848 /etc/nginx/conf.d/nacos.conf

# nacos grpc
add_server nacos_grpc "$(lookup_ips_with_port 9848 ${NACOS_NODE})" ' ' /etc/nginx/stream.d/nacos-grpc.conf
del_server nacos_grpc 127.0.0.1:9847 /etc/nginx/stream.d/nacos-grpc.conf


# 配置 dapi 代理
#  echo admin:$(openssl passwd -apr1 "passwd")
cat <<'EOF'   > /etc/nginx/conf.d/dcapi.conf 
upstream dcapi_server {
    server 127.0.0.1:7088;
    server 127.0.0.1:7089;
}

server {
    listen       18080;
    server_name  localhost;

    error_log    /var/log/nginx/dcapi.error.log;
    access_log   /var/log/nginx/dcapi.access.log;

    location / {
        rewrite  ^/dcapi/(.*) /$1 break;
        proxy_redirect off;
        proxy_set_header Host $host;
        proxy_set_header X-Real-IP $remote_addr;
        proxy_set_header X-Forwarded-For $proxy_add_x_forwarded_for;
        proxy_pass http://dcapi_server;
    }

    location /logs {
        auth_basic           "Please input password";
        auth_basic_user_file /etc/nginx/auth.db;
        autoindex            on;
        autoindex_exact_size off;
        autoindex_localtime  on;
        alias                /opt/logs;
    }

    # redirect server error pages to the static page /50x.html
    error_page   500 502 503 504  /50x.html;
    location = /50x.html {
        root   /usr/share/nginx/html;
    }
}
EOF
_log "添加 dapi 代理配置 ."

# https://developer.mozilla.org/zh-CN/docs/Web/HTTP/Guides/MIME_types/Common_types
sed -i 's@txt;@txt log;@' /etc/nginx/mime.types
echo 'admin:$apr1$0dO5Lk3W$0rwaLXkFP8ARpt5YMMeMe1' > /etc/nginx/auth.db
chown nginx:nginx /etc/nginx/auth.db
_log "添加日志密码认证 ."

add_service_to_rc_local nginx
_log "添加服务启动命令到 /etc/rc.d/rc.local ."

systemctl enable nginx &>/dev/null
systemctl restart nginx
_log "启动 nginx 服务."

## 安裝字體
# 安装 fc-list
if ! which fc-list &>/dev/null; then
    dnf install -y fontconfig &>/dev/null
    _log "安装 fontconfig ."
fi

# 安装 ttmkfdir
if ! which ttmkfdir &>/dev/null; then
    dnf install -y ttmkfdir &>/dev/null
    _log "安装 ttmkfdir ."
fi

# 安装包名称
PACKAGE=chinese.tar.gz

# 准备软件安装包
if [ -f "${PACKAGE_DIR}/${PACKAGE}" ]; then
    _info "本机已存在软件包 ${PACKAGE} ."
else
    retry_until_success curl -sL -o ${PACKAGE_DIR}/${PACKAGE} ${SERVER_URL}/${PACKAGE}
    _log "下载 >>> ${PACKAGE} ."
fi

tar xf ${PACKAGE_DIR}/${PACKAGE} -C /usr/share/fonts
_log "解压字体包 >>> ${PACKAGE}"

# 生成字体索引
ttmkfdir -d /usr/share/fonts/chinese -o /usr/share/fonts/chinese/fonts.scale
sed -i '/<dir>\/usr\/share\/fonts<\/dir>/a\        <dir>/usr/share/fonts/chinese</dir>' /etc/fonts/fonts.conf
green_echo ">>> 生成字体索引 ."
fc-cache && fc-list | grep sim