#!/bin/bash

# 获取当前路径
basedir=$(dirname "$(realpath "${BASH_SOURCE[0]}")")

# 读取配置文件
if [ -f "${basedir}/function.sh" ]; then
    . ${basedir}/function.sh
fi

# 读取环境变量
if [ -f "$MANGO_ENV" ]; then
   . $MANGO_ENV
fi

# 检查服务是否存在于 /etc/rc.d/rc.local(检查服务状态时用)
check_rc_entry() {
  local prefix="$1"
  local file="${2:-/etc/rc.d/rc.local}"

  awk -v p="$prefix" '
    BEGIN { found=0 }                # 初始化标记变量
    !/^#/ && $3 == p { found=1 }     # 匹配非注释行且第三列符合条件
    END { exit !found }              # 根据结果返回退出码
  ' "$file"
  
  return $?  # 透传 awk 的退出码（0=找到，1=未找到）
}

# 系统负载
function _uptime() {
    awk '{print $1, $2, $3}' /proc/loadavg
}

# 检测进程 limit/运行时间
function process_limit () {
    local pids="$1"

    for pid in $pids; do
        if [ -f "/proc/$pid/limits" ]; then
            limits=$(awk '/open file/{print $4"/"$5}' /proc/$pid/limits)
            s_time=$(ps -eo pid,lstart | grep -w $pid | awk '{$1="";sub(" ", "");print}')
            time=$(date -d "$s_time" +'%F %T')
            echo "PID: $(green_echo $pid) Limit: $(green_echo $limits) StartTime: $(green_echo $time)"
        fi
    done
}

# 输入进程名输出进程id
function process_is_running () {
    local pattern="$1"

    ps -ef | grep "$pattern" \
           | grep -vE '(grep |bash -l)' \
           | awk '{print $2;a++}END{if (a>0) {exit 0} else {exit 1}}'
}

# 判断进程是否运行,输出对应状态
function ps_status () {
    local application="$1"
    local pattern="$2"
    local systemd="$3"
    local ret

    if [ -n "$systemd" ]; then
        IS_ENABLE=$(systemctl is-active $systemd)
        [ "$IS_ENABLE" == 'active' ] && IS_ENABLE=$(green_echo $IS_ENABLE) || IS_ENABLE=$(red_echo $IS_ENABLE)
    fi

    [ -z "$application" ] && return 0

    _pids=( $(process_is_running "$pattern") )
    if [ ${#_pids[@]} -ne 0 ]; then
        status="$(green_echo RUNNING)"
        ret=0
    else
        status="$(red_echo EXIT)"
        ret=1
    fi

    local SYS_INFO
    [  -n "$systemd" ] && SYS_INFO="Systemd: $IS_ENABLE"

    PID="${_pids[@]}"
    echo "[$(blue_echo $LAN_IP_ADDRESS)] $application: $status"
    echo "[$(blue_echo $LAN_IP_ADDRESS)] $(process_limit $PID) $SYS_INFO"
    return $ret
}

function ps_status_java () {
    local application=$1
    local pattern=$2
    local systemd=$3

    IS_ENABLE=$(systemctl is-active $systemd)
    [ "$IS_ENABLE" == 'active' ] && IS_ENABLE=$(green_echo $IS_ENABLE) || IS_ENABLE=$(red_echo $IS_ENABLE)

    if ps -ef | grep -v grep | grep -q "$pattern"; then
        PID=$(process_is_running "$pattern")
        echo "[$(blue_echo $LAN_IP_ADDRESS)] $application: $(green_echo RUNNING)"
        echo "[$(blue_echo $LAN_IP_ADDRESS)] $(process_limit $PID) Systemd: $IS_ENABLE"
        return 0
    else
        echo "[$(blue_echo $LAN_IP_ADDRESS)] $application: $(red_echo EXIT) Systemd: $IS_ENABLE"
        return 2
    fi
}

function ps_status_zookeeper () {
    title "💻 ($HOSTNAME) $(_uptime)"
    ps_status_java 'zookeeper' 'org.apache.zookeeper.server.quorum.QuorumPeerMain' 'zookeeper'
}

function ps_status_kafka () {
    title "💻 ($HOSTNAME) $(_uptime)"
    ps_status_java 'kafka' 'kafka.Kafka' 'kafka'
}

function ps_status_hadoop () {
    #check_rc_entry
    # listIniKeysValues hadoop environment
    if [ "${#HADOOP[@]}" -ge 5 ]; then
        NameNode=(${HADOOP[@]:0:2})
        DataNode=(${HADOOP[@]:2})

        if contains "$LAN_IP_ADDRESS" NameNode[@]; then
            title "💻 ($HOSTNAME) $(_uptime)"
            ps_status_java 'hadoop.hdfs.server.namenode.NameNode' 'org.apache.hadoop.hdfs.server.namenode.NameNode' 'hdp-namenode'
            ps_status_java 'hadoop.hdfs.tools.DFSZKFailoverController' 'org.apache.hadoop.hdfs.tools.DFSZKFailoverController' 'hdp-zkfc'
            ps_status_java 'hadoop.yarn.server.resourcemanager.ResourceManager' 'org.apache.hadoop.yarn.server.resourcemanager.ResourceManager' 'hdp-resourcemanager'

            if [ -f /usr/lib/systemd/system/hdp-journalnode.service ]; then
                ps_status_java 'hadoop.hdfs.qjournal.server.JournalNode' 'org.apache.hadoop.hdfs.qjournal.server.JournalNode' 'hdp-journalnode'
            fi
        fi

        if contains "$LAN_IP_ADDRESS" DataNode[@]; then
            title "💻 ($HOSTNAME) $(_uptime)"
            ps_status_java 'hadoop.hdfs.server.datanode.DataNode' 'org.apache.hadoop.hdfs.server.datanode.DataNode' 'hdp-datanode'
            ps_status_java 'hadoop.yarn.server.nodemanager.NodeManager' 'org.apache.hadoop.yarn.server.nodemanager.NodeManager' 'hdp-nodemanager'
            
            if [ -f /usr/lib/systemd/system/hdp-journalnode.service ]; then
                ps_status_java 'hadoop.hdfs.qjournal.server.JournalNode' 'org.apache.hadoop.hdfs.qjournal.server.JournalNode' 'hdp-journalnode'
            fi
        fi
    else
        NameNode=(${HADOOP[@]:0:1})
        DataNode=(${HADOOP[@]:1})

        if contains "$LAN_IP_ADDRESS" NameNode[@]; then
            title "💻 ($HOSTNAME) $(_uptime)"
            ps_status_java 'hadoop.hdfs.server.namenode.NameNode' 'org.apache.hadoop.hdfs.server.namenode.NameNode' 'hdp-namenode'
            ps_status_java 'hadoop.hdfs.server.namenode.SecondaryNameNode' 'org.apache.hadoop.hdfs.server.namenode.SecondaryNameNode' 'hdp-secondarynamenode'
            ps_status_java 'hadoop.yarn.server.resourcemanager.ResourceManager' 'org.apache.hadoop.yarn.server.resourcemanager.ResourceManager' 'hdp-resourcemanager'
        fi

        if contains "$LAN_IP_ADDRESS" DataNode[@]; then
            title "💻 ($HOSTNAME) $(_uptime)"
            ps_status_java 'hadoop.hdfs.server.datanode.DataNode' 'org.apache.hadoop.hdfs.server.datanode.DataNode' 'hdp-datanode'
            ps_status_java 'hadoop.yarn.server.nodemanager.NodeManager' 'org.apache.hadoop.yarn.server.nodemanager.NodeManager' 'hdp-nodemanager'
        fi
    fi
}

function ps_status_hbase () {
    # listIniKeysValues hbase environment
    if [ "${#HBASE[@]}" -ge 5 ]; then
        HBASE_MASTER=(${HBASE[@]:0:2})
        HBASE_SLAVE=(${HBASE[@]:2})
    else
        HBASE_MASTER=(${HBASE[@]:0:1})
        HBASE_SLAVE=(${HBASE[@]:1})
    fi

    if contains "$LAN_IP_ADDRESS" HBASE_MASTER[@]; then
        title "💻 ($HOSTNAME) $(_uptime)"
        ps_status_java 'hbase.master.HMaster' 'org.apache.hadoop.hbase.master.HMaster' 'hbase-master'
    fi

    if contains "$LAN_IP_ADDRESS" HBASE_SLAVE[@]; then
        title "💻 ($HOSTNAME) $(_uptime)"
        ps_status_java 'hbase.regionserver.HRegionServer' 'org.apache.hadoop.hbase.regionserver.HRegionServer' 'hbase-regionserver'
    fi
}

function ps_status_opentsdb () {
    # listIniKeysValues opentsdb environment
    if contains "$LAN_IP_ADDRESS" OPENTSDB[@]; then
        title "💻 ($HOSTNAME) $(_uptime)"
        ps_status_java 'opentsdb' 'net.opentsdb.tools.TSDMain' 'opentsdb'
        [ -f /tmp/.tsdb/tsdb-gnuplot/mygnuplot.sh ] || \
            install -D -m 744 -o opentsdb -g opentsdb /usr/share/opentsdb/bin/mygnuplot.sh /tmp/.tsdb/tsdb-gnuplot/mygnuplot.sh
    fi
}

function ps_status_storm () {
    # listIniKeysValues storm environment
    if [ "${#STORM[@]}" -ge 5 ]; then
        STORM_NIMBUS=(${STORM[@]:0:2})
        STORM_WORKER=(${STORM[@]:2})
    else
        STORM_NIMBU=(${STORM[@]:0:1})
        STORM_WORKER=(${STORM[@]:1})
    fi

    if contains "$LAN_IP_ADDRESS" STORM_NIMBU[@]; then
        title "💻 ($HOSTNAME) $(_uptime)"
        ps_status_java 'storm.ui.core' 'org.apache.storm.ui.core' 'storm-ui'
        ps_status_java 'storm.daemon.nimbus' 'org.apache.storm.daemon.nimbus' 'storm-nimbus'
    fi

    if contains "$LAN_IP_ADDRESS" STORM_WORKER[@]; then
        title "💻 ($HOSTNAME) $(_uptime)"
        ps_status_java 'storm.daemon.supervisor.Supervisor' 'org.apache.storm.daemon.supervisor.Supervisor' 'storm-spuervisor'
    fi
}


# 查询 mysql
function ps_status_mysql () {
    if [ "$LAN_IP_ADDRESS" == "${MYSQL[0]}" ]; then
        title "💻 ($HOSTNAME) $(_uptime)"
        ps_status 'mysql master' mysqld 'mysqld'
        if [ $? -eq 0 ]; then
            mysql -te 'show master status;'
        fi
    else
        title "💻 ($HOSTNAME) $(_uptime)"
        ps_status 'mysql slave' mysqld 'mysqld'
        if [ $? -eq 0 ]; then
            mysql -te 'show slave status\G;' | egrep "Slave_IO_Running:|Slave_SQL_Running:"
        fi
    fi
}


# 查询 fastdfs
function ps_status_fastdfs () {
    title "💻 ($HOSTNAME) $(_uptime)"
    ps_status 'fastdfs tracker' '/usr/bin/fdfs_trackerd' 'fdfs_trackerd'
    ps_status 'fastdfs storage' '/usr/bin/fdfs_storaged' 'fdfs_storaged'
    ps_status 'fastdfs nginx' 'nginx' 'nginx'
    if [ -n "$FASTDFS_VIP" ]; then
        ps_status 'keepalived' '/usr/sbin/keepalived' 'keepalived'
    fi

    egrep 'ip_addr|tracker|Storage' < <(fdfs_monitor /etc/fdfs/client.conf -h $LAN_IP_ADDRESS 2>/dev/null)
}

# 查询 mongodb
function ps_status_mongodb () {
    title "💻 ($HOSTNAME) $(_uptime)"
    ps_status 'mongodb' '/bin/mongod' 'mongodb'
}

# 查询 hazelcast
function ps_status_hazelcast () {
    title "💻 ($HOSTNAME) $(_uptime)"
    ps_status 'hazelcast' 'com.hazelcast.core.server.StartServer' 'hazelcast'
}

# 查询 emqtt
function ps_status_emqx () {
    title "💻 ($HOSTNAME) $(_uptime)"
    ps_status 'emqtt: run_erl' 'bin/run_erl' emqx
    ps_status 'emqtt: multi_time_warp' 'emqx -spp true'
    ps_status 'emqtt: erl_child_setup' erl_child_setup
    ps_status 'emqtt: memsup' bin/memsup
    ps_status 'emqtt: cpu_sup' bin/cpu_sup
    ps_status 'emqtt: inet_gethost' bin/inet_gethost
}

# 查询 tomcat
function ps_status_fanruan () {
    title "💻 ($HOSTNAME) $(_uptime)"
    ps_status_java 'Tomcat: apache.catalina.startup.Bootstrap' "org.apache.catalina.startup.Bootstrap" "tomcat-fanruan"
}

# 查询 Nginx 前端/后端/微服务代理
function ps_status_wfw () {
    title "💻 ($HOSTNAME) $(_uptime)"
    ps_status 'nginx 前端/后端/微服务代理' nginx 'nginx'
    if [ -n "$VIP" ]; then
        ps_status 'keepalived' '/usr/sbin/keepalived' 'keepalived'
    fi
}

# 查询 redis
function ps_status_redis () {
    title "💻 ($HOSTNAME) $(_uptime)"
    ps_status 'redis server' '/usr/bin/redis-server' 'redis'
}

# 查询 solr
function ps_status_solr () {
    title "💻 ($HOSTNAME) $(_uptime)"
    ps_status 'solr' '/server/solr' 'solr'
}

# 查询 flink
function ps_status_flink () {
    title "💻 ($HOSTNAME) $(_uptime)"
    # 角色分配
    # 节点数量大于5个, 则前3个为 jobmanager, 其他全部为 taskmanager
    if (( ${#FLINK[@]} > 5 )); then
        jobmanager=${FLINK[@]:0:3}
        taskmanager=${FLINK[@]:3}
        deploy_type=cluster
    # 节点数量等于5个, 则前2个为 jobmanager, 其他全部为 taskmanager
    elif (( ${#FLINK[@]} == 5 )); then
        jobmanager=${FLINK[@]:0:2}
        taskmanager=${FLINK[@]:2}
        deploy_type=cluster
    else
    # 节点数量少于5个, 则第1个为 jobmanager, 其他全部为 taskmanager
        jobmanager=${FLINK[0]}
        taskmanager=${FLINK[@]:1}
        deploy_type=standalone
    fi

    # 配置服务
    for role in jobmanager historyserver taskmanager; do
        # 将变量转换为数组
        role_array=(${!role})

        #echo "$role  --->  ${role_array[@]}"

        # 判断当前IP是否存在于数组中
        if contains "$LAN_IP_ADDRESS" role_array[@]; then
            case $role in
                jobmanager)
                    JOBMANAGER=($jobmanager)
                    if [ "$deploy_type" == 'cluster' ]; then
                        ps_status_java 'flink.runtime.zookeeper.FlinkZooKeeperQuorumPeer' 'org.apache.flink.runtime.zookeeper.FlinkZooKeeperQuorumPeer' 'flink-zookeeper'
                    fi
                    ps_status_java 'flink.runtime.entrypoint.StandaloneSessionClusterEntrypoint' 'org.apache.flink.runtime.entrypoint.StandaloneSessionClusterEntrypoint' 'flink-jobmanager'
                    ;;
                historyserver)
                    ps_status_java 'flink.runtime.webmonitor.history.HistoryServer' 'org.apache.flink.runtime.webmonitor.history.HistoryServer' 'flink-historyserver'
                    ;;
                taskmanager)
                    ps_status_java 'flink.runtime.taskexecutor.TaskManagerRunner' 'org.apache.flink.runtime.taskexecutor.TaskManagerRunner' 'flink-taskmanager'
                    ;;
            esac
        fi
    done
}

# 查询 seatunnel
function ps_status_seatunnel () {
    title "💻 ($HOSTNAME) $(_uptime)"
    [ "$LAN_IP_ADDRESS" == "${SEATUNNEL[0]}" ] && \
    ps_status_java 'seatunnel.app.SeatunnelApplication' 'org.apache.seatunnel.app.SeatunnelApplication' 'seatunnel-web'
    ps_status_java 'seatunnel.core.starter.seatunnel.SeaTunnelServer' 'org.apache.seatunnel.core.starter.seatunnel.SeaTunnelServer' 'seatunnel'
}

# 查询 doris
function ps_status_doris () {
    # 第一个节点执行
    if [ "$LAN_IP_ADDRESS" == "${DORIS[0]}" ]; then
        title "📦 Doris 集群状态"
        green_echo ">>> frontends: "
        awk -F': +' '
            BEGIN {
                BLUE = "\033[34m"
                YELLOW = "\033[33m"
                CYAN = "\033[36m"
                GREEN = "\033[32m"
                RESET = "\033[0m"
            }
            NR%4==1 { ip=$2 }
            NR%4==2 { host=$2 }
            NR%4==3 { role=$2 }
            NR%4==0 {
                alive=$2
                printf "\t%sIP:%s%s  %sHostName:%s%s  %sIsMaster:%s%s  %sAlive:%s%s\n",
                            BLUE,   ip,    RESET,
                            YELLOW, host,  RESET,
                            CYAN,   role,  RESET,
                            GREEN,  alive, RESET
            }' <<<$(egrep 'IsMaster|IP|HostName|Alive' < <(mysql -h127.0.0.1 -P9030 -e "SHOW PROC '/frontends'\G"))

        green_echo ">>> backends: "
        awk -F': +' '
            BEGIN {
                BLUE = "\033[34m"
                YELLOW = "\033[33m"
                CYAN = "\033[36m"
                GREEN = "\033[32m"
                RESET = "\033[0m"
            }
            NR%4==1 { bid=$2 }
            NR%4==2 { ip=$2 }
            NR%4==3 { host=$2 }
            NR%4==0 {
                alive=$2
                printf "\t%sIP:%s%s  %sHostName:%s%s  %sBackendId:%s%s  %sAlive:%s%s\n",
                            BLUE,   ip,    RESET,
                            YELLOW, host,  RESET,
                            CYAN,   bid,   RESET,
                            GREEN,  alive, RESET
            }' <<<$(egrep 'BackendId|IP|HostName|Alive' < <(mysql -h127.0.0.1 -P9030 -e "SHOW PROC '/backends'\G"))
    fi

    title "💻 ($HOSTNAME) $(_uptime)"
    # 角色分配
    # 节点数量大于、等于5个, 则前3个为 FE, 其他全部为 BE
    if (( ${#DORIS[@]} >= 5 )); then
        fe_node=${DORIS[@]:0:3}
        be_node=${DORIS[@]:3}
    else
    # 节点数量少于5个, 则第1个为 FE, 其他全部为 BE
        fe_node=${DORIS[0]}
        be_node=${DORIS[@]:1}
    fi

    # 配置服务
    for role in fe_node be_node; do
        # 将变量转换为数组
        role_array=(${!role})

        #echo "$role  --->  ${role_array[@]}"

        # 判断当前IP是否存在于数组中
        if contains "$LAN_IP_ADDRESS" role_array[@]; then
            case $role in
                fe_node)
                    ps_status_java 'doris.PaloFe' 'org.apache.doris.PaloFe' 'doris-fe'
                    ;;
                be_node)
                    ps_status_java 'doris_be' 'lib/doris_be' 'doris-be'
                    ;;
            esac
        fi
    done
}

# 查询 nacos
function ps_status_nacos () {
    title "💻 ($HOSTNAME) $(_uptime)"
    ps_status_java 'nacos.nacos' 'nacos.nacos' 'nacos'
    [ "$LAN_IP_ADDRESS" == "${NACOS[1]}" ] && ps_status 'mysqld' mysqld 'mysqld'
}

# 查询 minio
function ps_status_minio () {
    title "💻 ($HOSTNAME) $(_uptime)"
    ps_status 'minio server' 'bin/minio' 'minio'
    ps_status 'ht server'    'bin/node'  'ht'
}

# 查询 tdengine
function ps_status_tdengine () {
    if [ "$LAN_IP_ADDRESS" == "${TDENGINE[0]}" ]; then
        green_echo ">>> mnodes 节点信息 ."
        nc -zvw 1 127.0.0.1 6030 &>/dev/null && taos -s 'show mnodes\G'
        green_echo ">>> dnodes 节点信息 ."
        nc -zvw 1 127.0.0.1 6030 &>/dev/null && taos -s 'show dnodes\G'
    fi

    title "💻 ($HOSTNAME) $(_uptime)"
    ps_status 'tdengine' 'bin/taosd' 'taosd'
    ps_status 'tdengine' 'bin/udfd'
    ps_status 'tdengine' 'bin/taosadapter' 'taosadapter'
}

# 查询 dolphinscheduler
function ps_status_dolphinscheduler () {
    title "💻 ($HOSTNAME) $(_uptime)"

    # 角色分配
    if (( ${#DOLPHINSCHEDULER[@]} >= 5 )); then
        masters=${DOLPHINSCHEDULER[@]:0:2}
        apiServers=${DOLPHINSCHEDULER[@]:0:2}
        alertServer=${DOLPHINSCHEDULER[-1]}
        workers=${DOLPHINSCHEDULER[@]:2}
        workers_list=$(lookup_ips_with_port default ${workers})
    else
        masters=${DOLPHINSCHEDULER[0]}
        apiServers=${DOLPHINSCHEDULER[0]}
        alertServer=${DOLPHINSCHEDULER[-1]}
        workers=${DOLPHINSCHEDULER[@]:1}
        workers_list=$(lookup_ips_with_port default ${workers})
    fi

    # 当前IP角色匹配
    for role in masters workers apiServers alertServer; do
        # 将变量转换为数组
        role_array=(${!role})

        #echo "$role  --->  ${role_array[@]}"

        # 判断当前IP是否存在于数组中
        if contains "$LAN_IP_ADDRESS" role_array[@]; then
            case $role in
                masters)
                    ps_status_java 'master.MasterServer' 'org.apache.dolphinscheduler.server.master.MasterServer' 'dolphinscheduler-master'
                    ;;
                workers)
                    ps_status_java 'worker.WorkerServer' 'org.apache.dolphinscheduler.server.worker.WorkerServer' 'dolphinscheduler-worker'
                    ;;
                apiServers)
                    ps_status_java 'api.ApiApplicationServer' 'org.apache.dolphinscheduler.api.ApiApplicationServer' 'dolphinscheduler-api'
                    ;;
                alertServer)
                    ps_status_java 'alert.AlertServer' 'org.apache.dolphinscheduler.alert.AlertServer' 'dolphinscheduler-alert'
                    ;;
            esac
        fi
    done
}

# 查看系统服务状态
function ps_status_system () {
    title "💻 ($HOSTNAME) $(_uptime)"

    local service_list=$(awk '/systemctl start/{print $NF}' /etc/rc.d/rc.local)
    # 定义输出格式（第一列固定40字符宽度）
    local format="%-40s %-12s %-12s\n"
    
    echo '------------------------------------------------------------------------------------------'
    printf "\033[32m${format}\033[0m" "${HOSTNAME}" "$(hostname -i)" "$(date +'%F %T')"
    # 格式化标题中文占用宽度*2，英文占用宽度*1
    printf "%-44s %-14s %-16s\n" "服务名称" "状态" "启动时间"
    echo '------------------------------------------------------------------------------------------'
    
    for service in $service_list; do
        status=$(systemctl is-active $service 2>/dev/null)
        source_start_time=$(systemctl show $service --property ActiveEnterTimestamp --value 2>/dev/null)
        [ -z "$source_start_time" ] && start_time="N/A" || start_time=$(date -d "$source_start_time" +"%Y-%m-%d %H:%M:%S")
        # 带颜色格式化输出（注意ANSI颜色码长度不影响对齐）
        if [ "$status" == "active" ]; then
            printf "\033[32m${format}\033[0m" "$service" "$status" "$start_time"
        else
            printf "\033[31m${format}\033[0m" "$service" "$status" "$start_time"
        fi
    done
}

# 查询节点资源信息
function ps_status_node () {
    cpuname=$(grep 'name' /proc/cpuinfo | awk -F ': ' '{print $2;exit}')
    physical=$(grep 'physical id' /proc/cpuinfo | sort -u | wc -l)
    processor=$(grep 'processor' /proc/cpuinfo | wc -l)
    cpucores=$(grep 'cpu cores' /proc/cpuinfo | awk '{print $NF;exit}')
    siblings=$(grep 'siblings' /proc/cpuinfo | awk '{print $NF;exit}')

    title "💻 ($HOSTNAME) $(_uptime)"
    green_echo "$LAN_IP_ADDRESS 节点资源信息 ."
    echo "（CPU型号）cpu name                   : $cpuname"
    echo "（物理CPU个数）physical id is         : $physical"
    echo "（逻辑CPU个数）processor is           : $processor"
    echo "（CPU内核数）cpu cores is             : $cpucores"
    echo "（单个物理CPU的逻辑CPU数）siblings is : $siblings"
    echo " 系统负载                             : $(_uptime)"

    echo ""
    echo "（MEM）内存信息"
    echo "      $(free -m | awk '{print; exit}')"
    echo "      $(free -ht --si | awk 'NR==2 {print; exit}')"
    echo "      $(free -ht --si | awk 'NR==3 {print; exit}')"
    echo "      $(free -ht --si | awk 'NR==4 {print; exit}')"

    echo ""
    disk_info=($(df -hT | grep '^/' | sort -k5rh | awk '{print $0 ;exit}'))
    echo "（Disk）最大分区挂载点                : ${disk_info[6]}"
    echo "        硬盘分区                      : ${disk_info[0]}"
    echo "        分区类型                      : ${disk_info[1]}"
    echo "        分区容量                      : ${disk_info[2]}"
    echo "        分区已用                      : ${disk_info[3]}"
    echo "        分区可用                      : ${disk_info[4]}"
    echo "        分区使用率                    : ${disk_info[5]}"

    if [ "${disk_info[6]}" != '/' ]; then
        disk_info=($(df -hT / | awk '/dev/{print $0 ;exit}'))
        echo "    根分区挂载点                      : ${disk_info[6]}"
        echo "        硬盘分区                      : ${disk_info[0]}"
        echo "        分区类型                      : ${disk_info[1]}"
        echo "        分区容量                      : ${disk_info[2]}"
        echo "        分区已用                      : ${disk_info[3]}"
        echo "        分区可用                      : ${disk_info[4]}"
        echo "        分区使用率                    : ${disk_info[5]}"
    fi
}

# 查询节点资源信息
function  ps_status_info () {
    bash ${basedir}/info.sh
}

function _server_cmd () {
    action=$1
    application=$2

    # 服务名前缀
    prefix=''
    case $application in
        hadoop)
            prefix="hdp-"
            ;;
        tdengine)
            prefix="taos"
            ;;
        seatunnel)
            prefix="seatunnel"
            ;;
        fastdfs)
            prefix="nginx|fdfs_"
            ;;
        wfw)
            prefix="nginx|keepalived"
            ;;
        nacos)
            prefix="nacos|mysqld"
            ;;
        minio)
            prefix="minio|ht"
            ;;
        $application)
            prefix="${application}-"
            ;;
    esac

    # 获取需要执行的服务
    system_list=$(awk -v p=$prefix '!/#/&&$3~"^"p{printf "%s ", $3} END {print ""}' /etc/rc.d/rc.local)

    if [ -n "$system_list" ]; then
        title "💻 ($HOSTNAME) $(_uptime)"
        green_echo ">>> 当前节点 $application 服务: $system_list ."
        for service in $system_list; do
            systemctl ${action} $service
            if [ $? -eq 0 ]; then
                status="$(green_echo successfully)"
            else
                status="$(red_echo failed)"
            fi
            echo "[$(blue_echo $LAN_IP_ADDRESS)] $application: ${action} $service $status"
        done
    fi
}

# 服务启动关闭/重启
function _service () {
    action=$1
    application=$2

    # 启动/关闭/重启 服务
    case $application in
        hadoop|hbase|flink|seatunnel|doris|dolphinscheduler|taosd|tdengine|fastdfs|nacos|minio|wfw)
            _server_cmd ${action} $application
            ;;
        zookeeper|kafka|opentsdb|mysql|mongodb|emqx|redis|nginx|keepalived|solr|fanruan)
            # 服务名与配置区域不一致
            case $application in
                mysql)
                    service="mysqld"
                    ;;
                fanruan)
                    service="tomcat-fanruan"
                    ;;
                *)
                    service="$application"
                    ;;
            esac

            title "💻 ($HOSTNAME) $(_uptime)"
            green_echo ">>> 当前节点 $application 服务: $system_list ."
            systemctl ${action} $service
            if [ $? -eq 0 ]; then
                status="$(green_echo successfully)"
            else
                status="$(red_echo failed)"
            fi
            echo "[$(blue_echo $LAN_IP_ADDRESS)] $application: ${action} $service $status"
            ;;
    esac

}

function _help () {
    title "使用说明"
    echo -e "\t $0 {hadoop|zookeeper|hbase|opentsdb|kafka|storm|preprocess|mysql|mongodb|fastdfs|hazelcast|emqtt|fanruan|nginx|redis|solr|tdengine|dolphinscheduler|nacos|keepalived|minio|ht|flink|seatunnel|doris|fastdfs}"
}

action=$1
application=$2

case $action in
    stop|start|restart)
        _service ${action} $application
        ;;
    status)
        ps_status_${application}
        ;;
esac