#!/bin/bash
set -e

# 全局配置
APPS=(
    "beszel:/usr/local/beszel:beszel-hub.service"
    "beszel-agent:/usr/local/beszel-agent:beszel-agent.service"
)

SERVICE_USER="beszel"
SCRIPT_DIR="$(cd "$(dirname "$0")" && pwd)"
SYSTEMD_DIR="/etc/systemd/system"

# 显示帮助
usage() {
    echo "用法: $0 [install|uninstall] [all|beszel|beszel-agent]"
    echo "示例:"
    echo "  $0 install all          # 安装所有组件"
    echo "  $0 uninstall beszel     # 卸载 beszel"
    exit 1
}

# 检查 root 权限
check_root() {
    [[ "$(id -u)" == "0" ]] || { echo "错误: 需要 root 权限"; exit 1; }
}

# 创建服务用户
create_service_user() {
    if ! id "$SERVICE_USER" &>/dev/null; then
        useradd -M -s /bin/false "$SERVICE_USER" || { echo "创建用户失败"; exit 1; }
    fi
}

# 安装应用
install_app() {
    local app=$1
    local install_path=$2
    local service_file=$3
    local src_dir="$SCRIPT_DIR/$app"

    echo "安装 $app 到 $install_path..."
    [[ -d "$src_dir" ]] || { echo "源码目录不存在: $src_dir"; exit 1; }

    # 清理旧版本
    mkdir -p "$install_path"
    [[ -d "$install_path" ]] && { rm -rf "$install_path" || exit 1; }

    # 复制文件
    cp -r "$src_dir" "$install_path" || exit 1

    # 设置权限（仅beszel）
    if [[ "$app" == "beszel" ]]; then
        create_service_user
        chown -R "$SERVICE_USER:$SERVICE_USER" "$install_path"
    fi

    # 部署服务
    local service_src="$install_path/$service_file"
    if [[ -f "$service_src" ]]; then
        cp "$service_src" "$SYSTEMD_DIR/" || exit 1
        systemctl daemon-reload
        systemctl enable "$service_file" &>/dev/null
        systemctl start "$service_file" || echo "启动服务失败（可能未就绪）"
    else
        echo "警告: 服务文件 $service_src 缺失"
    fi
}

# 卸载应用
uninstall_app() {
    local app=$1
    local install_path=$2
    local service_file=$3

    echo "卸载 $app..."

    # 停止服务（抑制错误）
    if systemctl is-active --quiet "$service_file" 2>/dev/null; then
        systemctl stop "$service_file" || echo "停止服务失败（可能已停止）"
    fi

    # 禁用服务
    systemctl disable "$service_file" 2>/dev/null || true

    # 删除服务文件
    rm -f "$SYSTEMD_DIR/$service_file" 2>/dev/null || true
    systemctl daemon-reload

    # 删除安装目录
    if [[ -d "$install_path" ]]; then
        rm -rf "$install_path" || { echo "删除目录失败（检查权限）"; exit 1; }
    fi

    # 删除用户（仅beszel）
    if [[ "$app" == "beszel" ]] && id "$SERVICE_USER" &>/dev/null; then
        userdel "$SERVICE_USER" 2>/dev/null || echo "删除用户失败（可能被占用）"
    fi
}

# 主流程
main() {
    check_root
    local action=$1
    local target=$2
    [[ -z "$action" || -z "$target" ]] && usage

    for entry in "${APPS[@]}"; do
        IFS=':' read -r app install_path service_file <<< "$entry"
        if [[ "$target" == "all" || "$target" == "$app" ]]; then
            echo "正在处理: $app"
            "${action}_app" "$app" "$install_path" "$service_file"
        fi
    done

    echo "操作完成!"
}

# 执行入口
main "$@"