#!/bin/bash

# 获取当前路径
basedir=$(dirname "$(realpath "${BASH_SOURCE[0]}")")

# 加载函数
if [ -f "${basedir}/../function.sh" ]; then
    . ${basedir}/../function.sh
fi

# 加载环境变量
. /usr/local/.mango/environment.txt

title "📦️ 初始化 MySQL 主从集群 ."

# 强制执行跳过确认
if [ "$1" != 'force' ]; then
    # 执行前确认
    echo -ne "\033[031;1m[$LAN_IP_ADDRESS] 警告：此操作将初始化 MySQL 主从集群，是否继续？(请输入Yes继续)：\033[0m"
    read -r user_input
else
    user_input='Yes'
fi
if [[ "${user_input}" != "Yes" ]]; then
	_info "用户取消操作，退出。"
	exit 1
else
    echo
fi

# 创建复制/管理用户
if [ "$1" == 'force' ]; then
    green_echo ">>> 创建复制/管理用户 ."
    # 复制用户
    REPL_USER=repl2024
    MYSQL_REPL_PASS=$MYSQL_PASS

    # 默认登录信息
    AUTH="-uroot -p$MYSQL_PASS"

    # 管理用户
    MY_USER=hcroot
    MYSQL_MY_PASS=$MYSQL_PASS

    ## 创建 && 授权复制用户
    USER_EXISTS=$(ssh -p ${SSH_PORT:-22} -T ${MYSQL[0]} <<-EOF
        mysql $AUTH -N -e "SELECT EXISTS(SELECT 1 FROM mysql.user WHERE user='$REPL_USER')" 2>/dev/null
	EOF
    )

    # 判断并创建用户
    if [[ "$USER_EXISTS" -eq 0 ]]; then
        _log "创建复制用户 $REPL_USER ..."
        ssh -p ${SSH_PORT:-22} -T ${MYSQL[0]} <<-EOF
        mysql -e "CREATE USER '$REPL_USER'@'%' IDENTIFIED BY '${MYSQL_REPL_PASS}';" 2>/dev/null
	EOF
    fi

    # 授权
    ssh -p ${SSH_PORT:-22} -T ${MYSQL[0]} <<-EOF
        mysql $AUTH -e "GRANT REPLICATION SLAVE, REPLICATION CLIENT ON *.* TO '$REPL_USER'@'%' IDENTIFIED BY '${MYSQL_REPL_PASS}';" 2>/dev/null
        mysql $AUTH -e "FLUSH PRIVILEGES;" 2>/dev/null
	EOF
    _log "授予用户 $REPL_USER 复制权限 ."


    ## 创建 && 授权管理用户
    USER_EXISTS=$(ssh -p ${SSH_PORT:-22} -T ${MYSQL[0]} <<-EOF
        mysql $AUTH -N -e "SELECT EXISTS(SELECT 1 FROM mysql.user WHERE user='$MY_USER')" 2>/dev/null
	EOF
    )

    # 判断并创建用户
    if [[ "$USER_EXISTS" -eq 0 ]]; then
        _log "正在创建管理用户 $MY_USER..."
        ssh -p ${SSH_PORT:-22} -T ${MYSQL[0]} <<-EOF
        mysql -e "CREATE USER '${MY_USER}'@'%' IDENTIFIED BY '${MYSQL_MY_PASS}';" 2>/dev/null
	EOF
    fi

    # 授权
    ssh -p ${SSH_PORT:-22} -T ${MYSQL[0]} <<-EOF
        mysql $AUTH -e "GRANT ALL PRIVILEGES ON *.* TO '$MY_USER'@'%' IDENTIFIED BY '${MYSQL_MY_PASS}';" 2>/dev/null
        mysql $AUTH -e "FLUSH PRIVILEGES;" 2>/dev/null
	EOF
    _log "授予用户 $MY_USER 管理员权限 ."
fi

# Master 节点是否在线
until ssh -p ${SSH_PORT:-22} -T ${MYSQL[0]} "mysql -e 'SELECT 1'" &>/dev/null; do
    sleep 1
    echo "等待节点 ${MYSQL[0]} 准备就绪."
done

# 判断从节点主从状态
output=$(ssh -p ${SSH_PORT:-22} -T ${MYSQL[1]} "mysql -e 'SHOW SLAVE STATUS\G' 2>/dev/null")

# 提取关键指标
last_error=$(awk '/Last_Error:/ {print $2}' <<<"$output")
io_running=$(awk '/Slave_IO_Running:/ {print $2}' <<<"$output")
sql_running=$(awk '/Slave_SQL_Running:/ {print $2}' <<<"$output")
seconds_behind=$(awk '/Seconds_Behind_Master:/ {print $2}' <<<"$output")

# 判断主从状态
if [ "$io_running" = "Yes" ] && [ "$sql_running" = "Yes" ]; then
    green_echo ">>> 当前主从状态正常无需配置 ."
    _info "✅ 主从状态正常 | 延迟: ${seconds_behind:-0} 秒"
    _info "   Slave_IO_Running: $io_running"
    _info "   Slave_SQL_Running: $sql_running"
    _info "   最后错误: ${last_error:-无}"
else
    green_echo ">>> 配置 MySQL 主从 ."
    #ssh -p ${SSH_PORT:-22} -T ${MYSQL[0]} "mysql -e 'FLUSH TABLES WITH READ LOCK; set global read_only=1;'"

    # 获取全局读锁并维持
    # LOCK_PID=$(ssh -p ${SSH_PORT:-22} -T ${MYSQL[0]} "
    #     nohup mysql -e 'FLUSH TABLES WITH READ LOCK; SELECT SLEEP(86400)' >/dev/null 2>&1 &
    #     echo \$!
    # ")
    # _info "锁定节点 ${MYSQL[0]} 表，PID: $LOCK_PID . "

    # Master 节点启动 ncat 监听端口，往从节点发送数据
    NCAT_CMD_M="ncat --listen --keep-open --send-only --max-conns=1 3307 -c 'xtrabackup --backup --slave-info --stream=xbstream --host=127.0.0.1'"
    NCAT_PID=$(ssh -p ${SSH_PORT:-22} -T ${MYSQL[0]} "${NCAT_CMD_M} &>/tmp/ncat.log & echo \$!")

    # Slave 节点停服务
    ssh -p ${SSH_PORT:-22} -T ${MYSQL[1]} "systemctl stop mysqld"

    # Slave 清理数据
    SLAVE_DATA_DIR=$(ssh -p ${SSH_PORT:-22} -T ${MYSQL[1]} "awk '/^datadir/{print \$NF}' /etc/my.cnf")

    # 强制执行跳过确认
    if [ "$1" != 'force' ]; then
        # 执行删除前确认
        echo -ne "\033[031;1m[$LAN_IP_ADDRESS] 警告：此操作将删除 ${MYSQL[1]} 节点 ${SLAVE_DATA_DIR} 目录下的所有文件，是否继续？(请输入Yes继续)：\033[0m"
        read -r user_input
    else
        user_input='Yes'
    fi

    if [[ "${user_input}" != "Yes" ]]; then
        _info "用户取消操作，退出。"
        exit 1
    else
        ssh -p ${SSH_PORT:-22} -T ${MYSQL[1]} "[ -f ${SLAVE_DATA_DIR}/ibdata1 ] && find ${SLAVE_DATA_DIR} -mindepth 1 -maxdepth 1 -exec rm -rf {} +"
    fi

    # Slave 从 Master 节点获取数据
    NCAT_CMD_S="ncat --recv-only ${MYSQL[0]} 3307 | xbstream -x -C ${SLAVE_DATA_DIR} && xtrabackup --prepare --target-dir=${SLAVE_DATA_DIR}"
    ssh -p ${SSH_PORT:-22} -T ${MYSQL[1]} "nohup sh -c '$NCAT_CMD_S' >/tmp/ncat.log 2>&1 &"

    # 等待从节点同步完成
    DT=$(date +'%F %T')
    MASTER_DATA_DIR=$(ssh -p ${SSH_PORT:-22} -T ${MYSQL[0]} "awk '/^datadir/{print \$NF}' /etc/my.cnf")
    MASTER_SIZE=$(ssh -p ${SSH_PORT:-22} -T ${MYSQL[0]} "awk '{print \$1}' < <(du -sh ${MASTER_DATA_DIR})")
    _info "准备从节点 ${MYSQL[0]} 同步数据 ."
    until ! ssh -p ${SSH_PORT:-22} -T ${MYSQL[1]} "pidof ncat" &>/dev/null; do
        sleep 30
        SLAVE_SIZE=$(ssh -p ${SSH_PORT:-22} -T ${MYSQL[1]} "awk '{print \$1}' < <(du -sh ${SLAVE_DATA_DIR})")
        _info "${MYSQL[1]} 节点同步 ${MYSQL[0]} 数据中，当前进度 [$SLAVE_SIZE/$MASTER_SIZE] ，开始时间 [$DT] ..."
    done
    _info "${MYSQL[1]} 从节点 ${MYSQL[0]} 同步数据完成."

    # Slave 启动服务
    ssh -p ${SSH_PORT:-22} -T ${MYSQL[1]} "chown -Rf mysql:mysql ${SLAVE_DATA_DIR} && systemctl start mysqld"
    _log "启动节点 ${MYSQL[1]} MySQL 服务 ."

    # 等待从节点启动就绪
    until ssh -p ${SSH_PORT:-22} -T ${MYSQL[1]} "mysql -e 'SELECT 1'" &>/dev/null; do
        sleep 1
        echo "等待节点 ${MYSQL[1]} 服务准备就绪 ."
    done
    sleep 3

    # Slave 配置主从 ### ssh -p ${SSH_PORT:-22} -T ${MYSQL[1]} "mysql -e \"CHANGE MASTER TO MASTER_HOST='${MYSQL[0]}', MASTER_USER='repl2024', MASTER_PASSWORD='${MYSQL_PASS}', MASTER_AUTO_POSITION=1; START SLAVE;\""
    LOG_POS=$(ssh -p ${SSH_PORT:-22} -T ${MYSQL[1]} "awk '{print \$2; exit}' ${SLAVE_DATA_DIR}/xtrabackup_binlog_info")
    LOG_FILE=$(ssh -p ${SSH_PORT:-22} -T ${MYSQL[1]} "awk '{print \$1; exit}' ${SLAVE_DATA_DIR}/xtrabackup_binlog_info")
    ssh -p ${SSH_PORT:-22} -T ${MYSQL[1]} "mysql -e 'STOP SLAVE; RESET SLAVE'"
    ssh -p ${SSH_PORT:-22} -T ${MYSQL[1]} "mysql -e \"CHANGE MASTER TO MASTER_HOST='${MYSQL[0]}', MASTER_USER='repl2024', MASTER_PASSWORD='${MYSQL_PASS}', MASTER_LOG_FILE='${LOG_FILE}', MASTER_LOG_POS=${LOG_POS}; START SLAVE;\""
    _log "配置节点 ${MYSQL[1]} 主从关系 ." 


    # 等待主从状态正常
    _info "等待主从状态正常..."
    io_running="No"  sql_running="No"
    until [[ "$io_running" = "Yes" && "$sql_running" = "Yes" ]]; do
        # 获取从库状态
        output=$(ssh -p ${SSH_PORT:-22} -T ${MYSQL[1]} "mysql -e 'SHOW SLAVE STATUS\G' 2>/dev/null")
        
        # 提取关键指标
        last_error=$(awk '/Last_Error:/ {print $2}' <<<"$output")
        io_running=$(awk '/Slave_IO_Running:/ {print $2}' <<<"$output")
        sql_running=$(awk '/Slave_SQL_Running:/ {print $2}' <<<"$output")
        seconds_behind=$(awk '/Seconds_Behind_Master:/ {print $2}' <<<"$output")

        # 打印当前状态
        _info "状态检查: IO 线程状态: $io_running, SQL 线程状态: $sql_running, 延迟: $seconds_behind, last_error: $last_error ."

        # 如果任一线程未运行，等待后继续检查
        if [[ "$io_running" != "Yes" || "$sql_running" != "Yes" ]]; then
            LOG_POS=$(ssh -p ${SSH_PORT:-22} -T ${MYSQL[1]} "awk '{print \$2; exit}' ${SLAVE_DATA_DIR}/xtrabackup_binlog_info")
            LOG_FILE=$(ssh -p ${SSH_PORT:-22} -T ${MYSQL[1]} "awk '{print \$1; exit}' ${SLAVE_DATA_DIR}/xtrabackup_binlog_info")
            ssh -p ${SSH_PORT:-22} -T ${MYSQL[1]} "mysql -e 'STOP SLAVE; RESET SLAVE'"
            sleep 5
            echo "mysql -e \"CHANGE MASTER TO MASTER_HOST='${MYSQL[0]}', MASTER_USER='repl2024', MASTER_PASSWORD='${MYSQL_PASS}', MASTER_LOG_FILE='${LOG_FILE}', MASTER_LOG_POS=${LOG_POS}; START SLAVE;\""
            #ssh -p ${SSH_PORT:-22} -T ${MYSQL[1]} "mysql -e \"CHANGE MASTER TO MASTER_HOST='${MYSQL[0]}', MASTER_USER='repl2024', MASTER_PASSWORD='${MYSQL_PASS}', MASTER_LOG_FILE='${LOG_FILE}', MASTER_LOG_POS=${LOG_POS}; START SLAVE;\""
            _log "配置节点 ${MYSQL[1]} 主从关系 ." 
        fi
    done
    _info "主从状态正常 ."

    # Master 节点解锁表
    # ssh -p ${SSH_PORT:-22} -T ${MYSQL[0]} "kill $LOCK_PID"
    # _info "释放锁节点 ${MYSQL[0]} 锁表，PID: $LOCK_PID ."

    # Master 节点关闭 ncat 监听
    ssh -p ${SSH_PORT:-22} -T ${MYSQL[0]} "kill -9 $NCAT_PID 2>/dev/null || pkill -n cat"

    # 判断从节点主从状态
    output=$(ssh -p ${SSH_PORT:-22} -T ${MYSQL[1]} "mysql -e 'SHOW SLAVE STATUS\G' 2>/dev/null")

    # 提取关键指标
    last_error=$(awk '/Last_Error:/ {print $2}' <<<"$output")
    io_running=$(awk '/Slave_IO_Running:/ {print $2}' <<<"$output")
    sql_running=$(awk '/Slave_SQL_Running:/ {print $2}' <<<"$output")
    seconds_behind=$(awk '/Seconds_Behind_Master:/ {print $2}' <<<"$output")

    # 判断主从状态
    if [ "$io_running" = "Yes" ] && [ "$sql_running" = "Yes" ]; then
        _info "✅ 主从状态正常 | 延迟: ${seconds_behind:-0} 秒"
        _info "   Slave_IO_Running: $io_running"
        _info "   Slave_SQL_Running: $sql_running"
        _info "   最后错误: ${last_error:-无}"
    else
        _warn "❌ 主从状态异常 | 延迟: ${seconds_behind:-0} 秒"
        _warn "   Slave_IO_Running: $io_running"
        _warn "   Slave_SQL_Running: $sql_running"
        _warn "   最后错误: ${last_error:-无}"
        exit 1
    fi
fi


## reset master 后从节点复制异常; 从头开始重放 binlog 日志
## 跳过 GTID 冲突（高风险，仅用于紧急恢复）
# -- 在从库重置 GTID 历史（确保数据与主库一致）
## RESET MASTER;
## SET @@GLOBAL.gtid_purged = 'eab0f7e1-1f2a-11f0-bbf5-007056992009:1';
## STOP SLAVE;
## CHANGE MASTER TO MASTER_AUTO_POSITION=1;
## START SLAVE;
## SHOW SLAVE STATUS\G;