#!/bin/bash

# 获取当前路径
basedir=$(dirname "$(realpath "${BASH_SOURCE[0]}")")

# 读取配置文件
if [ -f "${basedir}/function.sh" ]; then
    . ${basedir}/function.sh
fi

# 加载区域变量
if ! environment full; then
    _err "环境变量加载异常，请检查配置文件 ."
fi

# 检测 HOSTS,SERVERS 数组个数
if [ "${#HOSTS[@]}" != "${#SERVERS[@]}" ]; then
    _err "HOSTS与SERVERS 数组个数不匹配 "HOSTS：${#HOSTS[@]}" SERVERS："${#SERVERS[@]}"."
fi

# 软件路径
dowload_server_url="$SERVER_URL"

# CPU 是否支持 avx2 指令集
if grep -q avx2 /proc/cpuinfo; then
    avx2=
else
    avx2='-noavx2'
fi

# 软件列表
declare -A packages=(
    ["jdk-${JDK_VER}-linux-x64.tar.gz"]="$ZOOKEEPER_NODE $KAFKA_NODE $HADOOP_NODE $NACOS_NODE $NACOS_NODE $WFW_NODE $CAIJI_NODE"
    ["zulu${ZULU_JDK_VER}-linux_x64.tar.gz"]="$WFW_NODE $CAIJI_NODE"
    ["zookeeper-${ZOOKEEPER_VER}.tar.gz"]="$ZOOKEEPER_NODE"
    ["kafka_${KAFKA_VER}.tgz"]="$KAFKA_NODE"
    ["hadoop-${HADOOP_VER}.tar.gz"]="$HADOOP_NODE"
    ["flink-${FLINK_VER}-bin-scala_2.12.tgz"]="$FLINK_NODE"
    ["flink-opertor-1.4-SNAPSHOT.jar"]="$FLINK_NODE"
    ["TDengine-server-${TDENGINE_VER}-Linux-x64.tar.gz"]="$TDENGINE_NODE"
    ["TDengine-client-${TDENGINE_VER}-Linux-x64.tar.gz"]="$WFW_NODE $CAIJI_NODE"
    ["hbase-${HBASE_VER}-bin.tar.gz"]="$HBASE_NODE"
    ["opentsdb-${OPENTSDB_VER}.tgz"]="$OPENTSDB_NODE"
    ["apache-seatunnel-${SEATUNNEL_VER}-bin.tar.gz"]="$SEATUNNEL_NODE"
    ["apache-seatunnel-${SEATUNNEL_VER}-connectors.tgz"]="$SEATUNNEL_NODE"
    ["apache-seatunnel-${SEATUNNEL_VER}-customized.tgz"]="$SEATUNNEL_NODE"
    ["apache-seatunnel-web-${SEATUNNEL_WEB_VER}-bin.tar.gz"]="$SEATUNNEL_NODE"
    ["apache-seatunnel-web-${SEATUNNEL_WEB_VER}-datasource.tgz"]="$SEATUNNEL_NODE"
    ["apache-doris-fe-${DORIS_VER}-bin-x86_64.tar.xz"]="$DORIS_NODE"
    ["apache-doris-be-${DORIS_VER}-bin-x86_64${avx2}.tar.xz"]="$DORIS_NODE"
    ["apache-doris-dependencies-${DORIS_VER}-bin-x86_64.tar.xz"]="$DORIS_NODE"
    ["mysql-client-${MYSQL_VER}.tar.gz"]="$DORIS_NODE"
    ["mysql-${MYSQL_VER}-linux-glibc2.12-x86_64.tar.gz"]="$MYSQL_NODE ${NACOS[1]}"
    ["fastdfs-${FDFS_VER}.tgz"]="$FASTDFS_NODE"
    ["percona-xtrabackup-2.4.27.tgz"]="$MYSQL_NODE ${NACOS[1]}"
    ["apache-dolphinscheduler-${DOLPHINSCHEDULER_VER}-bin.tar.gz"]="$DOLPHINSCHEDULER_NODE"
    ["apache-dolphinscheduler-${DOLPHINSCHEDULER_VER}-libs.tgz"]="$DOLPHINSCHEDULER_NODE"
    ["nacos-server-${NACOS_VER}.tar.gz"]="$NACOS_NODE"
    ["emqx-${EMQX_VER}-el9-amd64.tar.gz"]="$EMQX_NODE"
    ["mongodb-linux-x86_64-rhel93-${MONGODB_VER}.tgz"]="$MONGODB_NODE"
    ["mongosh-${MONGOSH_VER}-linux-x64.tgz"]="$MONGODB_NODE"
    ["minio-${MINIO_VER}.tgz"]="$MINIO_NODE"
    ["node-v${NODEJS_VER}-linux-x64.tar.gz"]="$MINIO_NODE"
    ["ht-3d-editor.tgz"]="$MINIO_NODE"
    ["solr-${SOLR_VER}.tgz"]="$SOLR_NODE"
    ["solr-core.tgz"]="$SOLR_NODE"
    ["tomcat-linux.tar.gz"]="$FANRUAN_NODE"
    ["nginx-${NGINX_VER}-ngx.el9.x86_64.rpm"]="$WFW_NODE $CAIJI_NODE"
    ["ffmpeg"]="$WFW_NODE $CAIJI_NODE"
    ["chinese.tar.gz"]="$FANRUAN_NODE $WFW_NODE $CAIJI_NODE"
    ["beszel-0.10.2.tgz"]="${SERVERS[@]}"
)

# mysql https://downloads.mysql.com/archives/community/
# mongodb https://www.mongodb.com/try/download/community-kubernetes-operator
# emqx https://www.emqx.com/zh/downloads-and-install/broker?os=RHEL

# 检测服务器是否在线
function check_ping() {
    title "📦️ [1/7] ping 检测 ."
    for node in ${SERVERS[@]}; do
        if ping -i 0.2 -c 3 -W 1 $node &>/dev/null; then
            _log ">>> $node ."
        else
            _err ">>> $node ."
        fi
    done
}

# 安装基础环境
function install_base_environment() {
    # 配置安装源，如果是离线安装
    if [ "$INS_TYPE" == 'off-line' ]; then
        http_code=$(curl -I -X GET -m 10 -so /dev/null -s -w %{http_code} http://${dowload_server_url}/local.repo)
        if [ "$http_code" == 200 ]; then
            [ -d "/etc/yum.repos.d.bak" ] || cp -r /etc/yum.repos.d /etc/yum.repos.d.bak 
            rm -f /etc/yum.repos.d/*.repo
            curl http://${dowload_server_url}/local.repo -so /etc/yum.repos.d/local.repo
        else
            _err "软件安装源异常 ：http://${dowload_server_url}/local.repo"
            exit 1
        fi
    else
        # 配置软件源加速
        sed -e 's|^mirrorlist=|#mirrorlist=|g' \
            -e "s|^#baseurl=http://dl.rockylinux.org/\$contentdir|baseurl=$BASE_REPO_URL|g" \
            -i $(find /etc/yum.repos.d -name *.repo)
        _log "配置 dnf 软件源 ."

        # 安装 epel 源
        dnf install -y epel-release &>/dev/null

        # 配置 epel 源加速
        sed -e 's|^metalink=|#metalink=|g' \
            -e 's|^#baseurl=|baseurl=|g' \
            -e "s|https://download.fedoraproject.org/pub|$EPEL_REPO_URL|g" \
            -e "s|https://download.example/pub|$EPEL_REPO_URL|g" \
            -e 's|http://mirrors|https://mirrors|g' \
            -i /etc/yum.repos.d/epel{,-testing}.repo
        _log "配置 epel 软件源 ."
    fi

    title '📦️ [2/7] 安装 tar wget .'
    for pkg in tar wget; do
        if ! rpm -q $pkg --quiet; then
            _log "安装 $pkg ."
            dnf install -y $pkg &>/dev/null
        else
            true && _info "服务器已经安装 $pkg ."
        fi
    done

    # 配置 hosts 解析
    sed -i '3,$d' /etc/hosts
    echo -e "\n# mango" >> /etc/hosts
    let SER_LEN=${#SERVERS[@]}-1
    for ((i=0;i<=$SER_LEN;i++)); do
        echo "${SERVERS[i]}  ${HOSTS[i]}" >> /etc/hosts
    done
}

# 密钥登陆
function setup_ssh_key() {
    title "📦️ [3/7] 配置 ssh 秘钥登录 ."
    SERVER_LIST="${SERVERS[@]}"
    deploy_ssh_keys "$SERVER_LIST" $SSH_USER $SSH_PASS $SSH_PORT

    green_echo ">>> 优化 ssh 连接速度，重启 sshd 服务 ."
    for node in $SERVER_LIST; do
        ssh -p $SSH_PORT -T ${SSH_USER}@${node} '''
          sed -i "s/#UseDNS yes/UseDNS no/" /etc/ssh/sshd_config
          sed -i "s/GSSAPIAuthentication .*/GSSAPIAuthentication no/" /etc/ssh/sshd_config
          systemctl restart sshd
        '''
        _log ">>> $node ."
    done
}


function download_software() {
    title '📦️ [4/7] 下载软件安装包 .'
    mkdir -p "${PACKAGE_DIR}" && cd "${PACKAGE_DIR}"
    _log "创建软件下载路径 >>> ${PACKAGE_DIR} ."

    local pkg=1
    pkg_count=${#packages[@]}
    for package in "${!packages[@]}"; do
        # 下载软件
        _info "[$pkg/$pkg_count] $package 开始下载 ."
        retry_until_success wget -qc "$dowload_server_url/$package"
        _log "[$pkg/$pkg_count] $package 下载完成 ."

        let pkg++
    done
}


function distribute_software() {
    title "📦️ [5/7] 同步软件安装包 ."
    local pkg=1
    pkg_count=${#packages[@]}
    for package in "${!packages[@]}"; do
        remote_ips=$(uniq < <(sort -n < <(sed 's/ /\n/g' <<<"${packages[$package]}")))
        node_count=$(wc -l <<<"$remote_ips")
        green_echo ">>> [$pkg/$pkg_count] $package ."
        local count=1
        for node in $remote_ips; do
            # 如果当前主机存在于部署主机中则跳过
            #[ "$LAN_IP_ADDRESS" == "$node" ] && continue

            # 获取远程节点 IP
            REMOTE_IP=$(lookup_ip_addresses $node)

            # 推送软件到节点
            ssh -p $SSH_PORT -T $node "mkdir -p $PACKAGE_DIR" &>/dev/null
            if scp -P $SSH_PORT -q ${PACKAGE_DIR}/${package} ${SSH_USER}@${REMOTE_IP:-$node}:${PACKAGE_DIR} &>/dev/null; then
                _log "[$count/$node_count] $package --> ${SSH_USER}@${REMOTE_IP:-$node}:${PACKAGE_DIR} ."
            else
                _err "[$count/$node_count] $package --> ${SSH_USER}@${REMOTE_IP:-$node}:${PACKAGE_DIR} ."
            fi
            let count++
        done
        let pkg++
    done
}


function distribute_scripts() {
    title "📦️ [6/7] 同步安装脚本 ."
    local count=1
    node_count=${#SERVERS[@]}
    for node in ${SERVERS[@]}; do
        ssh -p $SSH_PORT -T $node "mkdir -p $PACKAGE_DIR/script" &>/dev/null
        if scp -P $SSH_PORT -qr ${basedir}/* ${SSH_USER}@${node}:${PACKAGE_DIR}/script &>/dev/null; then
            _log "[$count/$node_count] ${basedir}/* --> ${SSH_USER}@${node}:${PACKAGE_DIR}/script ."
        else
            _err "[$count/$node_count] ${basedir}/* --> ${SSH_USER}@${node}:${PACKAGE_DIR}/script ."
        fi
        let count++
    done
}

function key_login() {
    title "📦️ [7/7] 配置 ssh 秘钥登录."
    if [ ! -f /usr/local/.keys ]; then
        # 配置免密登录 大数据集群
        _info "配置大数据集群密钥登录 ."
        BIG_DATA_KEY="$(get_ssh_pubkey ${HADOOP[@]:0:3})"
        KNOWN_HOSTS="$(get_ssh_known_hosts ${HADOOP[@]:0:3})"
        for NODE in ${HADOOP_NODE}; do
            _info "[${NODE}] ---> [${HADOOP_NODE}] ."
            ssh -p $SSH_PORT -T $NODE "[ -d ~/.ssh ] || mkdir -p ~/.ssh; echo \"${BIG_DATA_KEY}\" >> ~/.ssh/authorized_keys; echo \"${KNOWN_HOSTS}\" >> ~/.ssh/known_hosts" &>/dev/null
        done

        # 配置免密登录 Nacos 集群
        _info "配置 Nacos 集群密钥登录 ."
        NACOS_KEY="$(get_ssh_pubkey ${NACOS_NODE})"
        KNOWN_HOSTS="$(get_ssh_known_hosts ${NACOS_NODE})"
        for NODE in ${NACOS_NODE}; do
            _info "[${NODE}] ---> [${NACOS_NODE}] ."
            ssh -p $SSH_PORT -T $NODE "[ -d ~/.ssh ] || mkdir -p ~/.ssh; echo \"${NACOS_KEY}\" >> ~/.ssh/authorized_keys; echo \"${KNOWN_HOSTS}\" >> ~/.ssh/known_hosts" &>/dev/null
        done

        # 配置免密登录 Flink 集群
        _info "配置 Flink 集群密钥登录."
        FLINK_KEY="$(get_ssh_pubkey ${FLINK[@]:0:3})"
        KNOWN_HOSTS="$(get_ssh_known_hosts ${FLINK[@]:0:3})"
        for NODE in ${FLINK_NODE}; do
            _info "[${NODE}] ---> [${FLINK_NODE}] ."
            ssh -p $SSH_PORT -T $NODE "[ -d ~/.ssh ] || mkdir -p ~/.ssh; echo \"${FLINK_KEY}\" >> ~/.ssh/authorized_keys; echo \"${KNOWN_HOSTS}\" >> ~/.ssh/known_hosts" &>/dev/null
        done

        # 配置免密登录 MySQL 集群
        _info "配置 MySQL 集群密钥登录."
        MYSQL_KEY="$(get_ssh_pubkey ${MYSQL_NODE})"
        KNOWN_HOSTS="$(get_ssh_known_hosts ${MYSQL_NODE})"
        for NODE in ${MYSQL_NODE}; do
            _info "[${NODE}] ---> [${MYSQL_NODE}] ."
            ssh -p $SSH_PORT -T $NODE "[ -d ~/.ssh ] || mkdir -p ~/.ssh; echo \"${MYSQL_KEY}\" >> ~/.ssh/authorized_keys; echo \"${KNOWN_HOSTS}\" >> ~/.ssh/known_hosts" &>/dev/null
        done

        # 配置免密登录 前端集群
        _info "配置微服务服务器密钥登录."
        WFW_KEY="$(get_ssh_pubkey ${CAIJI_NODE} ${LM_NODE} ${WFW_NODE})"
        KNOWN_HOSTS="$(get_ssh_known_hosts ${CAIJI_NODE} ${LM_NODE} ${WFW_NODE})"
        for NODE in ${CAIJI_NODE} ${LM_NODE} ${WFW_NODE}; do
            _info "[${NODE}] ---> [${CAIJI_NODE} ${LM_NODE} ${WFW_NODE}] ."
            ssh -p $SSH_PORT -T $NODE "[ -d ~/.ssh ] || mkdir -p ~/.ssh; echo \"${WFW_KEY}\" >> ~/.ssh/authorized_keys; echo \"${KNOWN_HOSTS}\" >> ~/.ssh/known_hosts" &>/dev/null
        done

        touch /usr/local/.keys
    else
        _info "密钥登录配置已存在，跳过配置 ."
    fi
}


function install_info () {
    let SER_LEN=${#SERVERS[@]}-1

    # 打印配置信息
    bash ${basedir}/info.sh

    read -p '确认以上信息，开始安装请输入[Y]：' ARG
    [ "$ARG" != 'Y' ] && { echo -e '\n\t取消...\n'; exit 1; }
}

function install_cmd (){
    printf "\n\033[034;1m%-38s %s %-38s\033[0m\n" "" "所有节点执行" | tr ' ' "-"
    printf "\033[032;1m%-4s %s\033[0m\n" "" "bash $PACKAGE_DIR/script/install.sh"
    printf "\033[034;1m%-90s\033[0m\n" | tr ' ' "-"
}

ARG=${1:-start}

case $ARG in
    sync)
        check_ping
        install_info
        install_base_environment
        setup_ssh_key
        distribute_software
        distribute_scripts
        install_cmd
        ;;
    download)
        check_ping
        install_base_environment
        distribute_software
        distribute_scripts
        ;;
    start)
        check_ping
        install_info
        install_base_environment
        setup_ssh_key
        download_software
        distribute_software
        distribute_scripts
        key_login
        install_cmd
        ;;
    test)
        check_ping
        install_base_environment
        setup_ssh_key
        distribute_scripts
        key_login
        install_cmd
        ;;
esac