#!/bin/bash

# 获取当前路径
basedir=$(dirname "$(realpath "${BASH_SOURCE[0]}")")

# 读取配置文件
if [ -f "${basedir}/function.sh" ]; then
    . ${basedir}/function.sh
fi

# 读取环境变量
if [ -f "$MANGO_ENV" ]; then
   . $MANGO_ENV
else
    # 加载区域变量
    if ! environment full; then
        _err "环境变量加载异常，请检查配置文件 ."
    fi
fi

# 样式配置
declare -A STYLES=(
    ["reset"]="\033[0m"
    ["title"]="\033[1;36m"       # 青色加粗
    ["header"]="\033[1;34m"      # 蓝色加粗  
    ["service"]="\033[1;32m"     # 绿色加粗
    ["label"]="\033[38;5;214m"   # 橙色
    ["value"]="\033[0;37m"       # 灰色
    ["ok"]="\033[1;32m"          # 绿色
    ["error"]="\033[1;31m"       # 红色
)

# 生成分隔线（支持Unicode字符）
draw_separator() {
    local width=${1:-90}
    local pattern=${2:-'='}
    printf "${STYLES[header]}%*s${STYLES[reset]}\n" "$width" | tr ' ' "$pattern"
}

# 带样式的打印函数
print_section() {
    [ "$1" ] && printf "${STYLES[title]}%s${STYLES[reset]}\n" "$1"
    draw_separator 90 '-'
}

print_service() {
    printf "\n${STYLES[service]} > %s${STYLES[reset]}\n" "$1"
}

print_field() {
    printf "${STYLES[label]}%-32s${STYLES[reset]} ${STYLES[value]}%s${STYLES[reset]}\n" "$1:" "$2"
}


han_num() {
    local text="$@"
    local str_length=$(wc -m <<<$text)
    local chinese_length=$(wc -l < <(grep -oP "\p{Han}" <<<$text))
    local text_length=$(( chinese_length + 12 ))
    echo $text_length
}

# 服务信息生成器
generate_service_block() {
    local service_name="$1"
    shift
    local -a fields=("$@")
    
    print_service "$service_name"
    for field in "${fields[@]}"; do
        IFS='|' read -r label value <<< "$field"
        text_length=$(han_num ${label})
        printf "    ${STYLES[label]}%-${text_length}s${STYLES[reset]} ${STYLES[value]}%s${STYLES[reset]}\n" "${label}:" "$value"
    done
}

# 生成完整报告
generate_report() {
    # 主机信息
    print_section
    print_section "主机名 IP"
    printf "  ${STYLES[label]}%-12s${STYLES[reset]} ${STYLES[value]}%s${STYLES[reset]}\n" \
        "总节点数:" "${#SERVERS[@]}"

    for ((i=0; i<=${#SERVERS[@]}-1; i++)); do
        printf "\t${STYLES[label]}%-15s${STYLES[reset]}${STYLES[value]}%-40s${STYLES[reset]}\n" "${SERVERS[i]}" "${HOSTS[i]}"
    done
    echo ""

    # 软件安装信息
    draw_separator 90
    printf "${STYLES[title]} %-34s 软件安装信息 ${STYLES[reset]}\n"
    draw_separator 90
    
    generate_service_block "Zookeeper"          "集群节点|$ZOOKEEPER_NODE"
    generate_service_block "Kafka"              "集群节点|$KAFKA_NODE"
    generate_service_block "Hadoop"             "NameNode|$([ "${#HADOOP[@]}" -ge 5 ] && echo ${HADOOP[@]:0:2} || echo ${HADOOP[0]})" \
                                                "DataNode|$([ "${#HADOOP[@]}" -ge 5 ] && echo ${HADOOP[@]:2} || echo ${HADOOP[@]:1})"
    generate_service_block "Flink"              "JobManager|$([ "${#FLINK[@]}" -ge 5 ] && echo ${FLINK[@]:0:3} || echo ${FLINK[0]})" \
                                                "TaskManager|$([ "${#FLINK[@]}" -ge 5 ] && echo ${FLINK[@]:3} || echo ${FLINK[@]:1})"
    generate_service_block "Seatunnel"          "集群节点|$SEATUNNEL_NODE"
    generate_service_block "Doris"              "doris-fe|$([ "${#DORIS[@]}" -ge 5 ] && echo ${DORIS[@]:0:3} || echo ${DORIS[0]})" \
                                                "doris-be|$([ "${#DORIS[@]}" -ge 5 ] && echo ${DORIS[@]:3} || echo ${DORIS[@]:1})"
    generate_service_block "Dolphinscheduler"   "Master|$([ "${#DOLPHINSCHEDULER[@]}" -ge 5 ] && echo ${DOLPHINSCHEDULER[@]:0:2} || echo ${DOLPHINSCHEDULER[0]})" \
                                                "ApiServers|$([ "${#DOLPHINSCHEDULER[@]}" -ge 5 ] && echo ${DOLPHINSCHEDULER[@]:0:2} || echo ${DOLPHINSCHEDULER[0]})" \
                                                "Workers|$([ "${#DOLPHINSCHEDULER[@]}" -ge 5 ] && echo ${DOLPHINSCHEDULER[@]:2} || echo ${DOLPHINSCHEDULER[@]:1})" \
                                                "AlertServer|${DOLPHINSCHEDULER[-1]}"
    generate_service_block "Tdengine"           "mnode|$TDENGINE_NODE" \
                                                "dnode|$TDENGINE_NODE"
    generate_service_block "Nacos"              "集群节点|$NACOS_NODE"
    generate_service_block "MySQL"              "集群节点|$MYSQL_NODE"
    generate_service_block "Fastdfs"            "集群节点|$FASTDFS_NODE"
    generate_service_block "Redis"              "节点|$REDIS_NODE"
    generate_service_block "Minio"              "节点|$MINIO_NODE"
    generate_service_block "Solr"               "节点|$SOLR_NODE"
    generate_service_block "帆软"               "节点|$FANRUAN_NODE"
    generate_service_block "MongoDB"            "节点|$MONGODB_NODE"
    generate_service_block "Emqx"               "节点|$EMQX_NODE"
    generate_service_block "数据采集"            "节点|$CAIJI_NODE"
    generate_service_block "智能分析"            "节点|$LM_NODE"
    generate_service_block "微服务"              "节点|$WFW_NODE"
    [ "$VIP" ] && generate_service_block "HA"              "IP|$VIP"

    # 底部信息
    draw_separator 90
    printf "${STYLES[header]}%-74s ${STYLES[value]}%s${STYLES[reset]}\n" "生成时间:" "$(date '+%F %T')"
    draw_separator 90
}

generate_report