#!/bin/bash

export PATH=/usr/local/xtrabackup/bin:$PATH

# 显示使用帮助
usage () {
    echo "Usage: ${0} [OPTION...]"
    echo "  -H, --help          This option displays a help screen and exits."
    echo "  -n, --compress-threads=#"
    echo "                      This option specifies the number of worker threads that"
    echo "                      will be used for parallel compression. It is passed"
    echo "                      directly to the xtrabackup child process."
    echo "  -e, --extra-lsndir=name"
    echo "                      This option specifies the directory in which to save an"
    echo "                      extra copy of the \"xtrabackup_checkpoints\" file. The"
    echo "                      option accepts a string argument. It is passed directly"
    echo "                      to xtrabackup's --extra-lsndir option. See the xtrabackup"
    echo "                      documentation for details."
    echo "  -d, --incremental-basedir=name"
    echo "                      This option specifies the directory containing the full"
    echo "                      backup that is the base dataset for the incremental"
    echo "                      backup.  The option accepts a string argument."
    echo "  -r, --parallel=#    On backup, this option specifies the number of threads"
    echo "                      the xtrabackup child process should use to back up files"
    echo "                      concurrently.  The option accepts an integer argument. It"
    echo "                      is passed directly to xtrabackup's --parallel option. See"
    echo "                      the xtrabackup documentation for details."
    echo "  -s, --stream=name   This option specifies the format in which to do the"
    echo "                      streamed backup.  The option accepts a string argument."
    echo "                      The backup will be done to STDOUT in the specified"
    echo "                      format. Currently, the only supported formats are tar and"
    echo "                      xbstream. This option is passed directly to xtrabackup's"
    echo "                      --stream option."
    echo "  -u, --user=name     This option specifies the MySQL username used when"
    echo "                      connecting to the server, if that's not the current user."
    echo "                      The option accepts a string argument. See mysql --help"
    echo "                      for details."
    echo "  -h, --host=name     This option specifies the host to use when connecting to"
    echo "                      the database server with TCP/IP.  The option accepts a"
    echo "                      string argument. See mysql --help for details."
    echo "  -P, --port=#        This option specifies the port to use when connecting to"
    echo "                      the database server with TCP/IP.  The option accepts a"
    echo "                      string argument. See mysql --help for details."
    echo "  -p, --password=name This option specifies the password to use when connecting"
    echo "                      to the database. It accepts a string argument.  See mysql"
    echo "                      --help for details."
    echo "  -S, --socket=name   This option specifies the socket to use when connecting"
    echo "                      to the local database server with a UNIX domain socket."
    echo "                      The option accepts a string argument. See mysql --help"
    echo "                      for details."
    echo "  -b, --backup-dir    The local backup directory for temp use"
    echo "  -R, --remote-server The remote server with SSH where you want to put the backup"
    echo "                      file into."
    echo "  -D, --remote-dir    The backup directory on remote server"
    echo "  -v, --version       Output version information and exit."
}

# 读取命令后参数
OPTS=$(getopt -o Hvh:P:u:p:S:n:e:d:r:m:R:b:D: --long help,version,host:,port:,user:,password:,socket:,compress-threads:,extra-lsndir:,incremental-basedir:,parallel:,stream:,remote-server:,backup-dir:,remote-dir: -n 'parse-options' -- "$@")
if [ $? != 0 ]; then
    exit 1 
fi

eval set -- "$OPTS"

# 参数默认值设定
HELP=0
VERSION=0
SSH_PORT=22
MYSQL_HOST='127.0.0.1'
MYSQL_PORT=3306
MYSQL_USER='root'
MYSQL_PASS='123abc@DEF'
MYSQL_SOCK=''
INCREMENTAL=0
COMPRESS_THREADS=8
EXTRA_LSNDIR=''
INCREMENTAL_BASEDIR=''
PARALLEL=4
STREAM='xbstream'
REMOTE_SERVER=''
BACKUP_DIR='/home/backup'
REMOTE_DIR=''

# 参数赋值
while true; do
    case "$1" in
        -H | --help                ) HELP=1;                 break   ;; # 显示帮助信息，无需解析更多参数直接退出
        -v | --version             ) VERSION=1;              break   ;; # 显示版本信息，无需解析更多参数直接退出
        -h | --host                ) MYSQL_HOST=$2;          shift 2 ;; # 备份的主机
        -P | --port                ) MYSQL_PORT=$2;          shift 2 ;; # 服务端口
        -u | --user                ) MYSQL_USER=$2;          shift 2 ;; # 登录用户
        -p | --password            ) MYSQL_PASS=$2;          shift 2 ;; # 登录密码
        -S | --socket              ) MYSQL_SOCK=$2;          shift 2 ;; # 嵌套字文件位置
        -n | --compress-threads    ) COMPRESS_THREADS=$2;    shift 2 ;; # 压缩线程数，默认开启压缩
        -e | --extra-lsndir        ) EXTRA_LSNDIR=$2;        shift 2 ;; # 检查点信息保存的位置，存在则覆盖
        -d | --incremental-basedir ) INCREMENTAL_BASEDIR=$2; shift 2 ;; # 全量备份检查点位置
        -r | --parallel            ) PARALLEL=$2;            shift 2 ;; # 并发子进程数量
        -m | --stream              ) STREAM=$2;              shift 2 ;; # 数据流格式，默认 xbstream
        -R | --remote-server       ) REMOTE_SERVER=$2;       shift 2 ;; # 远程服务器信息，比如 root@10.0.0.1
        -b | --backup-dir          ) BACKUP_DIR=$2;          shift 2 ;; # 本地工作目录，默认 /home/backup
        -D | --remote-dir          ) REMOTE_DIR=$2;          shift 2 ;; # 远程备份路径，比如 /home/backup

        -- ) shift; break ;;
        * ) break ;;
    esac
done

# 显示版本
if [[ $VERSION -eq 1 ]]; then
	    echo "MySQL Adaptive Backup v1.0.0"
	    exit 0
fi

# 显示帮助
if [[ $HELP -eq 1 ]]; then
    usage
    exit 0
fi

# 对参数进行判断，如果没有提供则报错并退出
if ! [ -n "${EXTRA_LSNDIR}" ]; then
    echo "Please specify the action you want to run with -e or -extra_lsndir"
    exit 1;
fi

if ! [ -n "${REMOTE_SERVER}" ]; then
    echo "Please specify the action you want to run with -R or --remote-server"
    exit 1
fi

if ! [ -n "${BACKUP_DIR}" ]; then
	  echo "Please specify the action you want to run with -b or --backup-dir"
	  exit 1
fi

if ! [ -n "${REMOTE_DIR}" ]; then
	  echo "Please specify the action you want to run with -D or --remote-dir"
	  exit 1
fi

# 开始备份
dt=$(date +"%Y-%m-%d %H:%M:%S") # 当前时间，写日志需要使用
h=$(date +"%k")                 # 当前的小时数，24小时制
d=$(date +"%-d")                # 当前日期的天，可用用于更复杂的备份策略
m=$(date +"%-m")                # 当前日期的月，可以用于更复杂的备份策略
w=$(date +"%u")                 # 当前日期的周，可以用于更复杂的备份策略(1-7)
t=$(date +"%Y_%m_%d_%H_%M")     # 备份文件名上的时间戳

echo "[${dt}] ----------------------------------------" >> /var/tmp/backup.log

# 生成备份命令 --no-lock
CMD="innobackupex --compress"
CMD="${CMD} --host=${MYSQL_HOST}"
CMD="${CMD} --port=${MYSQL_PORT}"
CMD="${CMD} --user=${MYSQL_USER}"
CMD="${CMD} --password=${MYSQL_PASS}"
CMD="${CMD} --compress-threads=${COMPRESS_THREADS}"
CMD="${CMD} --parallel=${PARALLEL}"
CMD="${CMD} --stream=${STREAM}"

# 如果是周末 或 没有全量备份
[ -d "${EXTRA_LSNDIR}" ] || mkdir -p "${EXTRA_LSNDIR}"

# 创建远程备份路径
ssh -o StrictHostKeyChecking=no -p ${SSH_PORT} ${REMOTE_SERVER} "mkdir -p ${REMOTE_DIR}"

if [[ ${w} -eq 7 ]] || [[ ! -f "${EXTRA_LSNDIR}/full/xtrabackup_checkpoints" ]]; then
    # 全量备份
    echo "[${dt}] Make full backup" >> /var/tmp/backup.log
    CMD="${CMD} --extra-lsndir=${EXTRA_LSNDIR}/full"
    CMD="${CMD} ${BACKUP_DIR}"
    CMD="${CMD} | ssh -o StrictHostKeyChecking=no -p ${SSH_PORT} ${REMOTE_SERVER}"
    CMD="${CMD} 'gzip - > ${REMOTE_DIR}/full_${t}.xbstream.gz'"

    # 清理远程服务器备份只保留 4 周
    ssh -o StrictHostKeyChecking=no -p ${SSH_PORT} ${REMOTE_SERVER} "find ${REMOTE_DIR} -mtime +27 -name "*.xbstream.gz" -exec rm -f {} \;"
else
    # 增量备份
    echo "[${dt}] Make incremental backup" >> /var/tmp/backup.log
    CMD="${CMD} --extra-lsndir=${EXTRA_LSNDIR}/inc"
    CMD="${CMD} --incremental"
    CMD="${CMD} --incremental-basedir=${EXTRA_LSNDIR}/full"
    CMD="${CMD} ${BACKUP_DIR}"
    CMD="${CMD} | ssh -o StrictHostKeyChecking=no -p ${SSH_PORT} ${REMOTE_SERVER}"
    CMD="${CMD} 'gzip - > ${REMOTE_DIR}/inc_${t}.xbstream.gz'"
fi

# 输出备份目录到日志
echo "[${dt}] ${CMD}" >> /var/tmp/backup.log

# 执行备份操作
eval $CMD

# 检查备份结果
echo "[${dt}] exit $?" >> /var/tmp/backup.log