#!/bin/bash

# bacup doris
# ./backup_to_minio_with_cleanup.sh -o --no-tablespaces
# 30 4 * * * /script/backup_to_minio_with_cleanup.sh -o '--no-tablespaces --no-create-info' &> /tmp/backup_to_minio_with_cleanup.log

# MySQL 全库备份
# ./backup_to_minio_with_cleanup.sh -d '--all-databases'



# 默认配置（如果命令行参数未指定，则使用默认值）
MYSQL_HOST="${MYSQL_HOST:-127.0.0.1}"
MYSQL_PORT="${MYSQL_PORT:-9030}"
MYSQL_USER="${MYSQL_USER:-root}"
MYSQL_PASSWORD="${MYSQL_PASSWORD:-doris123}"
DATABASE_NAME="${DATABASE_NAME:-blueberry_data}"
MINIO_ALIAS="${MINIO_ALIAS:-minio}"
MINIO_BUCKET="${MINIO_BUCKET:-backup}"
MYSQLDUMP_OPTIONS="${MYSQLDUMP_OPTIONS:---routines --triggers --events --single-transaction --set-gtid-purged=OFF --force}"

# 备份文件的前缀
BACKUP_PREFIX="lm-mysql"

# 保留最近多少天的备份
RETENTION_DAYS=7

# 是否压缩备份文件
COMPRESS_BACKUP=true

# 压缩工具，支持 gzip、zstd、xz 等
COMPRESSION_TOOL="gzip"

# 获取当前日期
CURRENT_DATE=$(date +%F-%H-%M)
CURRENT_DATE=$(date +%F)

# 脚本信息
SCRIPT_NAME="backup_to_minio_with_cleanup.sh"
SCRIPT_VERSION="1.0.0"
SCRIPT_DESCRIPTION="MySQL database backup script with MinIO storage support."

# 定义颜色
RED=$(tput setaf 1)
GREEN=$(tput setaf 2)
RESET=$(tput sgr0)

# 日志函数
log_success() {
    local TIMESTAMP=$(date +"%Y/%m/%d %H:%M:%S")
    echo "${GREEN}[${TIMESTAMP}] SUCCESS: $1${RESET}" | tee -a ${LOG_FILE}
}

log_error() {
    local TIMESTAMP=$(date +"%Y/%m/%d %H:%M:%S")
    echo "${RED}[${TIMESTAMP}] ERROR: $1${RESET}" | tee -a ${LOG_FILE}
}

log_info() {
    local TIMESTAMP=$(date +"%Y/%m/%d %H:%M:%S")
    echo "[${TIMESTAMP}] INFO: $1" | tee -a ${LOG_FILE}
}

# 帮助信息
show_help() {
    echo "Usage: ${SCRIPT_NAME} [options]"
    echo "Description: ${SCRIPT_DESCRIPTION}"
    echo "Options:"
    echo "  -h MySQL host (default: ${MYSQL_HOST})"
    echo "  -P MySQL port (default: ${MYSQL_PORT})"
    echo "  -u MySQL user (default: ${MYSQL_USER})"
    echo "  -p MySQL password (default: ${MYSQL_PASSWORD})"
    echo "  -d Database name(s) (default: ${DATABASE_NAME})"
    echo "  -b MinIO bucket (default: ${MINIO_BUCKET})"
    echo "  -a MinIO alias (default: ${MINIO_ALIAS})"
    echo "  -o mysqldump options (default: ${MYSQLDUMP_OPTIONS})"
    echo "  -v Show version"
    echo "  -? Show help"
    exit 0
}

# 版本信息
show_version() {
    echo "${SCRIPT_NAME} version ${SCRIPT_VERSION}"
    exit 0
}

# 解析命令行参数
while getopts "h:P:u:p:d:b:a:o:v?" opt; do
    case $opt in
        h) MYSQL_HOST="$OPTARG" ;;
        P) MYSQL_PORT="$OPTARG" ;;
        u) MYSQL_USER="$OPTARG" ;;
        p) MYSQL_PASSWORD="$OPTARG" ;;
        d) DATABASE_NAME="$OPTARG" ;;
        b) MINIO_BUCKET="$OPTARG" ;;
        a) MINIO_ALIAS="$OPTARG" ;;
        o) MYSQLDUMP_OPTIONS="$OPTARG" ;;
        v) show_version ;;
        ?) show_help ;;
        *) echo "Invalid option: -$OPTARG" >&2
           exit 1 ;;
    esac
done

# 定义日志文件
LOG_FILE="/var/log/backup_to_minio.log"

# 检查 MySQL 连接
if ! mysql -h ${MYSQL_HOST} -P ${MYSQL_PORT} -u ${MYSQL_USER} -p${MYSQL_PASSWORD} -e "SELECT 1;" > /dev/null 2>&1; then
    log_error "Failed to connect to MySQL server. Please check your MySQL configuration."
    exit 1
fi

# 检查 Minio 配置
if ! mc alias ls ${MINIO_ALIAS} > /dev/null 2>&1; then
    log_error "Please check your MinIO configuration."
    exit 1
fi

# 检查 MinIO 配置
if ! mc ls ${MINIO_ALIAS}/${MINIO_BUCKET} > /dev/null 2>&1; then
    log_error "Failed to access MinIO bucket. Please check your MinIO configuration."
    exit 1
fi

# 检查 DATABASE_NAME 是否为空
if [[ -z "${DATABASE_NAME}" ]]; then
    log_error "DATABASE_NAME is empty. Please specify the database(s) to backup."
    exit 1
fi

# 备份函数
backup_database() {
    local INPUT_DB_NAME=$1  # 函数参数命名为 INPUT_DB_NAME

    # 判断是否全库备份
    if [[ "${INPUT_DB_NAME}" == "all" || "${INPUT_DB_NAME}" == "ALL" ]]; then
        DB_NAME="--all-databases"  # 设置为 --all-databases 以简化后续命令
        local BACKUP_FILE="${BACKUP_PREFIX}-all-${CURRENT_DATE}.sql"
    else
        DB_NAME="${INPUT_DB_NAME}"  # 使用实际的数据库名称
        local BACKUP_FILE="${BACKUP_PREFIX}-${DB_NAME}-${CURRENT_DATE}.sql"
    fi
    log_info "Backing up database ${DB_NAME} with options: ${MYSQLDUMP_OPTIONS}..."

    # 如果需要压缩，将压缩工具添加到 MYSQLDUMP_OPTIONS
    if [ "$COMPRESS_BACKUP" = true ]; then
        COMPRESSION_OPTIONS=" | ${COMPRESSION_TOOL}"
        BACKUP_FILE="${BACKUP_FILE}.gz"  # 使用压缩后的文件名
    else
        COMPRESSION_OPTIONS=""  # 不需要压缩
    fi

    # 使用 eval 执行命令
    eval mysqldump -h ${MYSQL_HOST} -P ${MYSQL_PORT} -u ${MYSQL_USER} -p${MYSQL_PASSWORD} ${MYSQLDUMP_OPTIONS} ${DB_NAME} 2>/dev/null ${COMPRESSION_OPTIONS} | mc pipe ${MINIO_ALIAS}/${MINIO_BUCKET}/${BACKUP_FILE}

    # 检查备份是否成功
    if [ $? -eq 0 ]; then
        log_success "Backup of ${INPUT_DB_NAME} completed successfully."

        # 获取备份文件大小
        local FILE_SIZE=$(mc stat --json ${MINIO_ALIAS}/${MINIO_BUCKET}/${BACKUP_FILE} | sed -n 's/.*"size":\([0-9]*\).*/\1/p')

        # 检查文件大小是否大于 1MB
	if [[ ${FILE_SIZE} -lt $((1048576 * 1)) ]]; then  # 1MB = 1048576 bytes
            log_error "Backup file ${BACKUP_FILE} is smaller than 1MB (${FILE_SIZE} bytes). Deleting the backup file..."
            mc rm ${MINIO_ALIAS}/${MINIO_BUCKET}/${BACKUP_FILE} &>/dev/null
            exit 1
        else
            log_success "Backup file ${BACKUP_FILE} size is ${FILE_SIZE} bytes (greater than 1MB)."
        fi
    else
        log_error "Backup of ${INPUT_DB_NAME} failed."
        exit 1
    fi
}

# 将 DATABASE_NAME 按逗号或空格分隔为数组
IFS=', ' read -r -a DATABASES <<< "${DATABASE_NAME}"

# 逐个备份数据库
for DB in "${DATABASES[@]}"; do
    backup_database "${DB}"
done

# 定期清理旧的备份文件
log_info "Cleaning up backups older than ${RETENTION_DAYS} days..."
mc find ${MINIO_ALIAS}/${MINIO_BUCKET} --name "${BACKUP_PREFIX}*" --older-than "${RETENTION_DAYS}d" --exec "mc rm {}"

# 检查清理是否成功
if [ $? -eq 0 ]; then
    log_success "Cleanup completed successfully."
else
    log_error "Cleanup failed."
    exit 1
fi
