#!/bin/bash
#
# bash ./get_source.sh "$(echo $(ps -eo pid,args | grep -v grep | egrep '[0-9] /.*bin/java|[0-9] java' | awk '{print $1}'))"
#

# 中文字符计数函数
count_chinese_chars() {
    echo -n "$1" | grep -oP '[\p{Han}]' | wc -l
}

# 将内存值从 KB 转换为其他单位
convert_units() {
    local value=$1
    local unit=$2

    case $unit in
        MB)
            echo "$((value / 1024))"
            ;;
        GB)
            echo "$((value / 1024 / 1024))"
            ;;
        KB)
            echo "$value"
            ;;
        *)
            echo "无效单位: $unit"
            return 1
            ;;
    esac
}

# 参数检查
if [ $# -lt 1 ]; then
    echo "用法: $0 <PID列表> [单位（MB/KB/GB）]"
    exit 1
fi

pids_input=$1
unit=${2:-MB}  # 默认单位MB

# 处理PID列表（支持逗号/空格分隔）
IFS=',| ' read -ra pids <<< "$pids_input"

# 过滤有效PID
valid_pids=()
for pid in "${pids[@]}"; do
    if [ -d "/proc/$pid" ]; then
        valid_pids+=("$pid")
    else
        echo "警告: 忽略无效PID $pid"
    fi
done
[ ${#valid_pids[@]} -eq 0 ] && { echo "错误: 无有效PID"; exit 1; }

# 获取系统级信息（所有进程共享）
MemTotal=$(awk '/MemTotal/ {print $2}' /proc/meminfo)
MemFree=$(awk '/MemFree/ {print $2}' /proc/meminfo)
SwapTotal=$(awk '/SwapTotal/ {print $2}' /proc/meminfo)
SwapFree=$(awk '/SwapFree/ {print $2}' /proc/meminfo)
load=$(awk '{printf "%s %s %s", $1, $2, $3}' /proc/loadavg)
cpu_cores=$(nproc)

# 转换系统内存单位
MemTotal_converted=$(convert_units "$MemTotal" "$unit")
MemFree_converted=$(convert_units "$MemFree" "$unit")
SwapTotal_converted=$(convert_units "$SwapTotal" "$unit")
SwapFree_converted=$(convert_units "$SwapFree" "$unit")

# 表格列定义
columns=(
    "进程 PID"
    "物理内存总量"
    "可用物理内存"
    "虚拟内存总量"
    "可用虚拟内存"
    "线程数量"
    "最大常驻内存 VmHWM"
    "当前常驻内存 VmRSS"
    "使用交换空间 VmSwap"
    "CPU 使用率 PS"
    "CPU 使用率 TOP"
    "CPU 核心数"
    "系统负载 1m, 5m, 15m"
)

# 动态计算列宽
max_width=24
declare -a column_widths=()
for col in "${columns[@]}"; do
    han_chars=$(count_chinese_chars "$col")
    #if ((han_chars > 4)); then
    #    width=$((max_width + han_chars/2 +1))
    #elif ((han_chars >=3)); then
    #    width=$((max_width + han_chars/2))
    #else
    #    width=$max_width
    #fi

    # 减 2 是因为 printf 前后空格(1个汉字等于2占位符)
    width=$((max_width + han_chars - 2 ))

    column_widths+=("$width")
done

# 生成分隔线
separator="+"
for i in "${columns[@]}"; do
    separator+=$(printf "%${max_width}s" | tr ' ' '-')"+"
done

# 打印表头
echo "$separator"
printf "|"
for i in "${!columns[@]}"; do
    printf " %-*s |" "${column_widths[i]}" "${columns[i]}"
    #echo "${column_widths[i]}" "${columns[i]}"
done
printf "\n%s\n" "$separator"

# 遍历每个PID生成数据行
for pid in "${valid_pids[@]}"; do
    # 获取进程级信息
    Threads=$(awk '/Threads/ {print $2}' /proc/"$pid"/status 2>/dev/null || echo 0)
    VmHWM=$(awk '/VmHWM/ {print $2}' /proc/"$pid"/status 2>/dev/null || echo 0)
    VmRSS=$(awk '/VmRSS/ {print $2}' /proc/"$pid"/status 2>/dev/null || echo 0)
    VmSwap=$(awk '/VmSwap/ {print $2}' /proc/"$pid"/status 2>/dev/null || echo 0)
    cpu_usage_ps=$(ps -p "$pid" -o %cpu --no-headers 2>/dev/null | xargs || echo "0.0")
    cpu_usage_top=$(top -b -n 1 -p "$pid" | awk 'NR>7 {print $9}')

    # 转换进程内存单位
    VmHWM_converted=$(convert_units "${VmHWM:-0}" "$unit")
    VmRSS_converted=$(convert_units "${VmRSS:-0}" "$unit")
    VmSwap_converted=$(convert_units "${VmSwap:-0}" "$unit")

    # 构建数据行
    data_fields=(
        "$pid"
        "$MemTotal_converted $unit"
        "$MemFree_converted $unit"
        "$SwapTotal_converted $unit"
        "$SwapFree_converted $unit"
        "${Threads:-0}"
        "${VmHWM_converted} $unit"
        "${VmRSS_converted} $unit"
        "${VmSwap_converted} $unit"
        "${cpu_usage_ps}%"
        "${cpu_usage_top}%"
        "$cpu_cores C"
        "$load"
    )

    # 打印数据行
    printf "|"
    for i in "${!data_fields[@]}"; do
	# 减 2 是因为 printf 前后空格(1个汉字等于2占位符)
        printf " %-*s |" "$((max_width - 2))" "${data_fields[i]}"
    done
    printf "\n%s\n" "$separator"
done
