#!/bin/bash

. /etc/profile &>/dev/null
export TERM=xterm

# 检查环境变量
if [[ "$SERVICE_MODE" == "systemctl" ]]; then
    echo "systemctl 执行"

    # 重定向标准错误到 /dev/null (后台执行日志异常)
    exec 2>/dev/null
else
    echo "命令行执行"
fi

# 微服务路径
base_dir="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )" && cd ${base_dir}

# 本机运行的微服务模块
list='uaa gateway base equipment job portal message patrol loong alarm-task inventory calc-root voice data-analysis knowledge activiti prod-schedule bim diagnosis energy-consumption backstage etf visual3d video ai backstage-web web web-bim platform-h5'

# 指定缓存配置, 中央配置服务器
nacos_addr=127.0.0.1:8848
local_ip=$(nmcli | awk '/inet4/{print $2}' | egrep '10.34|172.31|192.168' | awk -F '/' '{print $1}' | head -n 1)

java_bin="/home/hadoop/zulu17.52.17-ca-jdk17.0.12/bin/java"
java_opts="--spring.cloud.inetutils.preferred-networks=${local_ip%.*} --spring.cloud.nacos.config.server-addr=${nacos_addr} --spring.cloud.nacos.config.namespace=M30 --spring.profiles.active=prod --logging.level.com.alibaba.cloud.nacos.client=ERROR"

# 下载地址
download_server=https://p.yw.hc-yun.com

# color
red_echo ()      { [ "$HASTTY" == 0 ] && echo "$@" || echo -e "\033[031;1m$@\033[0m"; }
green_echo ()    { [ "$HASTTY" == 0 ] && echo "$@" || echo -e "\033[032;1m$@\033[0m"; }
yellow_echo ()   { [ "$HASTTY" == 0 ] && echo "$@" || echo -e "\033[033;1m$@\033[0m"; }
blue_echo ()     { [ "$HASTTY" == 0 ] && echo "$@" || echo -e "\033[034;1m$@\033[0m"; }
purple_echo ()   { [ "$HASTTY" == 0 ] && echo "$@" || echo -e "\033[035;1m$@\033[0m"; }
bred_echo ()     { [ "$HASTTY" == 0 ] && echo "$@" || echo -e "\033[041;1m$@\033[0m"; }
bgreen_echo ()   { [ "$HASTTY" == 0 ] && echo "$@" || echo -e "\033[042;1m$@\033[0m"; }
byellow_echo ()  { [ "$HASTTY" == 0 ] && echo "$@" || echo -e "\033[043;1m$@\033[0m"; }
bblue_echo ()    { [ "$HASTTY" == 0 ] && echo "$@" || echo -e "\033[044;1m$@\033[0m"; }
bpurple_echo ()  { [ "$HASTTY" == 0 ] && echo "$@" || echo -e "\033[045;1m$@\033[0m"; }
bgreen_echo ()   { [ "$HASTTY" == 0 ] && echo "$@" || echo -e "\033[042;34;1m$@\033[0m"; }

# 循环执行直到成功
function repeat() { while true; do  $@ && return;  sleep 3; done; }

# 检查 nacos
function check_nacos() {
    http_code=$(curl -I -X GET -m 10 -o /dev/null -s -w %{http_code} http://${nacos_addr}/nacos/)
    if [ "$http_code" == '200' ]; then
        return 0
    else
        return 1
    fi
}

# 获取进程 Pid
function _get_pid() {
    local name=$1
    #ps -eo pid,args | grep "[0-9] $java_bin" | grep " ${name}-" | awk '{print $1}'
    ps -eo pid,args | grep "[0-9] $java_bin" | grep " ${name}-" | awk '{printf "%s ", $1} END {print ""}'
}

# 等待时间
function sleep_time () {
    local max_time=$1
    tput sc
    for ((i=1; i<=$max_time; i ++)); do
        tput rc
        tput ed
        echo -n " Time:$i/$max_time"
        sleep 1
    done
    echo ""
}

# 服务启动后延时
function wait_time() {
    local module=$1
    case ${module} in
      # 如是 web 服务
      web|web-bim|backstage-web|platform-h5)
        sleep 1
        ;;
      # 如是 uaa gateway base
      uaa|gateway|base)
        sleep_time 40
        ;;
      # 其他模块
      *)
        sleep_time 30
        ;;
    esac
}

# 清理备份
function backup_package() {
    local max_num=7
    local app_dir=$1

    # 创建备份目录
    [ -d "${app_dir}/backup" ] || mkdir -p ${app_dir}/backup

    # 判断是否存在指定类型文件. 如果存在移动到备份目录(只搜索当前目录 -maxdepth 1)
    if ls ${app_dir} | egrep -q '*.zip|*.war|*.jar'; then
        find ${app_dir} -maxdepth 1 \( -name "*.jar" -o -name "*.war" -o -name "*.zip" \) -exec mv {} "${app_dir}/backup" \;
    fi

    # 获取备份目录有多少个旧版本
    curr_num=$(ls ${app_dir}/backup | egrep '*.zip|*.war|*.jar' | wc -l)
    if [ ${curr_num} -gt ${max_num} ]; then
        # 获取备份目录需删除几个
        let num=${curr_num}-${max_num}

        # 备份目录删除旧版本
        ls ${app_dir}/backup -rt | egrep '*.zip|*.war|*.jar' | head -n ${num} | xargs -i rm -f ${app_dir}/backup/{}
    fi
}


# 下载微服务包
function download() {
    local name=$1
    local version=$2

    # 检测版本号是否有效
    http_status=$(curl -o /dev/null -s -w "%{http_code}" ${download_server}/version/${version}.txt)
    if [ $http_status -ne 200 ]; then
        red_echo "地址 [${download_server}/version/${version}.txt] 不可用 请检测输入的版本号是否存在."
        exit 1
    fi

    # 不存在版本号文件
    [ -f ./version ] || touch ./version

    # 下载服务器上版本号文件
    [ -d /var/log/update ] || mkdir /var/log/update
    curl -so /var/log/update/${version}.txt ${download_server}/version/${version}.txt

    # 本地版本号与服务器版本号差异
    diff ${base_dir}/version /var/log/update/${version}.txt -y | egrep '\||>' > /var/log/update/current.diff

    # 获取需要升级的模块
    awk '{print $NF}' /var/log/update/current.diff > /var/log/update/current.txt

    # 如果是下载版本所有包
    if [ ${name} = 'all' ]; then
        # 查找需要升级的服务并下载
        for module in $list; do
            dir_name=${module}

            # 获取需要升级的模块包名称
            package=$(awk -F = "/^${module//-/_}=/{print \$NF}" /var/log/update/${version}.txt)
            if [ -z "${package}" ]; then
                red_echo "没有查找到服务 ${module} . 请确认服务是否存在 ."
                continue
            fi

            # 读取需要更新的版本号文件
            source /var/log/update/current.txt

            # 如果包存在于备份目录(历史版本)
            if [ -f "${base_dir}/${module}/backup/${package}" ]; then
                green_echo "历史备份目录存在. 已经移动到: [${base_dir}/${module}/${package}]"
                /usr/bin/mv ${base_dir}/${module}/backup/${package} ${base_dir}/${module}/${package}
                continue
            fi

            # 判断本地是否存在对应版本包
            if [ -f "${base_dir}/${module}/${package}" ]; then
                green_echo "文件已下载: [${base_dir}/${module}/${package}]"
            else
                # 移动包到备份目录/删除备份包
                backup_package ${base_dir}/${module}

                # 下载对应包
                green_echo "download ${package}"
                curl -Lo ${base_dir}/${module}/${package} ${download_server}/mango/${module}/${package}
            fi
        done

        # 更新当前版本号文件 && 读取新版本
        /bin/cp /var/log/update/${version}.txt ${base_dir}/version
        source ${base_dir}/version

        # xjar-agent-hibernate
        if [ ! -f "${base_dir}/${xjar_agent_hibernate}" ]; then
            green_echo "download ${xjar_agent_hibernate}"
            curl -Lo ${base_dir}/${xjar_agent_hibernate} ${download_server}/mango/xjar-agent-hibernate/${xjar_agent_hibernate} 
        fi

        # xjar
        if [ ! -f "${base_dir}/xjar" ]; then
            green_echo "download xjar"
            curl -Lo ${base_dir}/xjar ${download_server}/mango/xjar
            chmod +x ${base_dir}/xjar
        fi
    else
        name_list=$(sed 's/^/|/;s/$/|/' <<<${list// /|})
        if [[ "$name_list" =~ "|$name|" ]]; then
            line=$(awk -F = "/^${name//-/_}=/{print}" /var/log/update/${version}.txt)
            module=${line%=*}  package=${line#*=}

            # 读取需要更新的版本号变量
            source /var/log/update/${version}.txt

            # 如果包存在于备份目录(历史版本)
            if [ -f "${base_dir}/${name}/backup/${package}" ]; then
                green_echo "历史备份目录存在，已移动到: [${base_dir}/${name}/${package}]"
                /usr/bin/mv ${base_dir}/${name}/backup/${package} ${base_dir}/${name}/${package}

            # 判断本地是否存在对应版本包
            elif [ -f "${base_dir}/${name}/${package}" ]; then
                green_echo "文件已下载: [${base_dir}/${name}/${package}]"
            else
                # 移动包到备份目录/删除备份包
                backup_package ${base_dir}/${name}

                # 下载对应包
                green_echo "download ${package}"
                curl -o ${base_dir}/${name}/${package} ${download_server}/mango/${name}/${package}
            fi

            # 检查当前包信息是否存在于新的版本号文件中
            if grep -q "^${name//-/_}=" ${base_dir}/version; then
                # 如果存在则替换版本号
                sed -i "s#^${module}=.*#${module}=${package}#" ${base_dir}/version
            else
                # 如果不存在则追加到末尾
                echo "${module}=${package}" >> ${base_dir}/version
            fi
        else
            red_echo "服务 [${name}] 不存在于 [list] 变量中."
            exit 1
        fi
    fi
    echo $version > ${base_dir}/.version
}


# 启动服务
function start_module() {
    local name=$1
    local type=$2

    source ${base_dir}/version

    local package=${name//-/_}
    local package=${!package}

    # 如果是启动所有模块
    if [ "${name}" == 'all' ]; then
        green_echo "启动所有服务 Start Time: [$(date '+%F %T')]"
        for module in ${list}; do
            local package=${module//-/_}
            local package=${!package}

            # 判断服务是否在运行中
            if ps -eo args | grep -w "^$java_bin " | grep -q " ${module}-"; then
                if ps -eo args | grep -w "^$java_bin " | grep -q "${package}"; then
                    red_echo "服务 [${module}] 已经在运行中 . 启动服务命令未执行 . pid: [$(_get_pid ${module})]"
                    continue
                else
                    # 如果启动版本异常，再次启动触发判断重启
                    start_module ${module}
                fi
            else
                # 按照 list 顺序启动模块
                start_module ${module}

                # 如果当前服务不是最后一个延时等待
                if [ "${list##* }" != "${module}"  ]; then
                    wait_time ${module}
                fi
            fi
        done
        green_echo "所有服务启动完成. End Time: [$(date '+%F %T')]"
        exit 0
    fi

    # 如果是启动指定的模块 检测模块名称是否存在 list
    name_list=$(sed 's/^/|/;s/$/|/' <<<${list// /|})
    if [[ ! "$name_list" =~ "|$name|" ]]; then
        red_echo "服务 [${name}] 不存在于 [list] 变量中."
        exit 1
    fi

    # 判断服务是否在运行中
    if ps -eo args | grep -w "^$java_bin " | grep -q " ${name}-"; then
        if ps -eo args | grep -w "^$java_bin " | grep -q "${package}"; then
            red_echo "服务 [${name}] 已经在运行中 . 启动服务命令未执行 . pid: [$(_get_pid ${name})]"
            exit 2
        else
            old_pkg=$(ps -eo args | grep -w "^$java_bin " | grep -o "${name}-[^ ]*\.jar")
            red_echo "服务 [${name}] 已经在运行中 . 运行版本与实际不符[${old_pkg}]，已经重启 . pid: [$(_get_pid ${name})]"
            stop_module ${name}
        fi
    fi

    # 模块目录如果不存在
    if [ ! -d "${base_dir}/${name}" ]; then
        red_echo "本地不存在 [${name}] 服务目录 ."
        exit 2
    fi

    # 进入模块所在目录
    cd ${base_dir}/${name}

    # 判断文件是否存在
    if [ ! -f "${package}" ]; then
        red_echo "服务 [$name] 无法启动 . 本地不存在包 [${package}] ."
        exit 3
    fi

    java_agent="-javaagent:../${xjar_agent_hibernate}"
    java_opens="--add-opens java.base/jdk.internal.loader=ALL-UNNAMED --add-opens java.base/java.lang=ALL-UNNAMED --add-opens java.base/java.net=ALL-UNNAMED"

    case ${name} in
      uaa|gateway)
        nohup ../xjar $java_bin $java_agent $java_opens -Xms1024m -Xmx1024m -jar ${package} $java_opts &> /dev/null &
        sleep 1
        green_echo "服务 [${name}] 启动,  版本: [${package}],  pid: [$(_get_pid ${name})]"
        ;;
      web-bim|platform-h5|web|backstage-web)
        if [[ "$name" == "web" ]] || [[ "$name" == "backstage-web" ]]; then
            workdir=dist
        elif [[ "$name" == "web-bim" ]]; then
            workdir=bim-web
        else
            workdir=${name}
        fi
        [ -f "./${workdir}/version" ] && local_version=$(cat ./${workdir}/version)
        if [ "${package}" == "${local_version}" ]; then
            red_echo "已经是最新版本：[${package}] . 启动服务命令未执行 ."
        else
            # 判断文件是否存在
            if [ ! -f "${package}" ]; then
                red_echo "服务 [$name] 处理异常 . 本地不存在包 [${package}] ."
                exit 3
            fi

            # 备份旧版本
            if [ -d "./${workdir}" ]; then
                [ -d ./backup ] || mkdir ./backup
                bak_name=${package%.*}-$(date +%Y%m%d%H%M%S)
                /bin/mv ./${workdir} ./backup/${bak_name}
            fi

            # 解压包
            unzip -q ${package}

            # 恢复默认权限
            if [ "${name}" == 'platform-h5' ]; then
                #find -type d | xargs chmod 755
                #find -type f | xargs chmod 644
                chmod 644 -R ./
                find ./ -type d -print | xargs chmod 755
            fi

            # 判断解压后的路径是否存在
            if [ ! -d "./${workdir}" ]; then
                red_echo "${package} 解压异常, 在当前目录没有产生 ${workdir} 文件夹,请检查压缩包是否正常."
                /bin/mv ./backup/${bak_name} ./${workdir}
                green_echo "操作已经回滚."
                exit 4
            fi

            # 将当前运行版本号写入文件
            echo ${package} > ./${workdir}/version
            [ -f './htconfig.js' ] && /bin/cp htconfig.js dist/custom/configs/htconfig.js
            [ -f './FD4Y7s6koxI9.js' ] && /bin/cp FD4Y7s6koxI9.js dist/client/license/FD4Y7s6koxI9.js

            # 启动 nginx
            if $(which nginx) -t &>/dev/null; then
                systemctl restart nginx
                green_echo "服务 [${name}] 启动,  版本: [${package}],"
            fi
        fi
        ;;
      dc-api|dc-mqtt)
        [ "$name" == 'dc-api' ] && port1=7088 port2=7089
        [ "$name" == 'dc-mqtt' ] && port1=7091 port2=7092
        nohup ../xjar $java_bin $java_agent $java_opens -Xms1500m -Xmx1500m -jar ${package} --server.port=${port1} --spring.cloud.nacos.config.enabled=false --spring.cloud.nacos.discovery.enabled=false --spring.profiles.active=prod &> /dev/null &
        nohup ../xjar $java_bin $java_agent $java_opens -Xms1500m -Xmx1500m -jar ${package} --server.port=${port2} --spring.cloud.nacos.config.enabled=false --spring.cloud.nacos.discovery.enabled=false --spring.profiles.active=prod &> /dev/null &
        sleep 1
        green_echo "服务 [${name}] 启动,  版本: [${package}],  pid: [$(_get_pid ${name})]"
        ;;
      process)
        /home/hadoop/flink-1.17.1/bin/flink run -d -p 1 -c com.hc.preprocess.app.ProcessAPP ${package} application.properties
        #echo 'yarn application -list'
        #sleep_time 10
        /home/hadoop/flink-1.17.1/bin/flink list
        ;;
      calc-task)
        /home/hadoop/flink-1.17.1/bin/flink run -d -p 1 -c com.hc.task.app.CalcTaskAPP ${package} task.properties
        #echo 'yarn application -list'
        sleep_time 7
        #yarn application -list
        /home/hadoop/flink-1.17.1/bin/flink list
        ;;
      ${name})
        nohup ../xjar $java_bin $java_agent $java_opens -Xms768m -Xmx768m -jar ${package} $java_opts &> /dev/null &
        sleep 1
        green_echo "服务 [${name}] 启动,  版本: [${package}],  pid: [$(_get_pid ${name})]"
        ;;
    esac
}


function stop_module() {
    local name=$1

    source ${base_dir}/version

    local package=${name//-/_}
    local package=${!package}

    # 如果是关闭所有模块
    if [ "${name}" = 'all' ]; then
        green_echo "关闭所有服务 开始时间: [$(date '+%F %T')]"

        # 反转列表
        stop_list=$(awk '{ for (i=NF; i>1; i--) printf("%s ",$i); print $1; }' <<<$list)

        # 循环列表并关闭
        for module in ${stop_list}; do
            stop_module ${module}
            sleep 2
        done
        green_echo "关闭所有服务完成 结束时间: [$(date '+%F %T')]"
        exit 0
    fi

    # 如果是启动指定的模块 检测模块名称是否存在 list
    name_list=$(sed 's/^/|/;s/$/|/' <<<${list// /|})
    if [[ ! "$name_list" =~ "|$name|" ]]; then
        red_echo "服务 [${name}] 不存在于 [list] 变量中."
        exit 1
    fi

    # 过滤 java 进程
    ps -eo pid,args | grep "[0-9] $java_bin " > /tmp/.mgstop

    # 关闭服务
    case ${name} in
        web-bim|platform-h5|web|backstage-web)
            sleep 0.1
            ;;
        ${name})
            # 其他服务
            if grep -q " ${name}-" /tmp/.mgstop; then
                kill -9 $(awk "/ ${name}-/{print \$1}" /tmp/.mgstop)
                green_echo "服务 [${name}] 已经关闭 ."
            else
                red_echo "服务 [${name}] 没有在运行 ."
            fi
            ;;
    esac
}

function update() {
    local name=$1
    local version=$2

    # 下载包
    download ${name} ${version}

    # 如果是更新所有服务
    if [ ${name} = 'all' ]; then
        # update_list=$(cat /var/log/update/current.txt | awk -F '=' '{print $1}' | sed 's#_#-#')
        for module in ${list}; do
            if grep -q "=${module}-" /var/log/update/current.txt; then
                # 关闭服务
                stop_module ${module}

                # 启动服务
                start_module ${module} auto
                wait_time ${module}
            fi
        done
    else
        stop_module ${name}
        start_module ${name}
    fi
}

# 帮助信息
function usage() {
    cat <<-EOF
    Usage :
        $0 {start|stop}        {uaa|gateway|all|...}
        $0 {download|update}   {uaa|gateway|all|...}   version

        # 更新到指定版本(下载模块→停止模块→启动模块)
        $0 update all 32A

        # 下载模块
        $0 download all 32A

        # 停止模块(停止uaa)
        $0 stop uaa

        # 启动模块
        $0 start uaa
EOF
}


# 服务运行状态
function status () {
    source ${base_dir}/version

    # 过滤 java 进程
    ps -eo pid,args | grep "[0-9] $java_bin " > /tmp/.mgstatus

    for module in $list; do
        local package=${module//-/_}
        local package=${!package}

        if [[ "$module" =~ ^(web-bim|platform-h5|web|backstage-web)$ ]]; then
            if [ -n "$(/usr/sbin/pidof nginx)" ]; then
                ngx_status='nginx is running...'
            else
                ngx_status='nginx is not running...'
            fi

            if [[ "$module" == "web" ]] || [[ "$module" == "backstage-web" ]]; then
                workdir=dist
            elif [[ "$module" == "web-bim" ]]; then
                workdir=bim-web
            else
                workdir=${module}
            fi

            [ -f "${module}/${workdir}/version" ] && local_version=$(cat ${module}/${workdir}/version)

            if [ "${local_version}" == "${package}" ]; then
                green_echo "${module}: ${package} ok  ${nginx_status}"
            else
                red_echo "${module}: ${package} no  ${nginx_status}"
            fi
        else
            if grep -q ${package} /tmp/.mgstatus; then
                green_echo "${module}: ${package}  ok"
            else
                red_echo "${module}: ${package}  no"
            fi
        fi
    done
}

function status_table() {
    # 读取版本号信息
    source ${base_dir}/version

    # 健康状态检测
    server_ip=${local_ip:-127.0.0.1}

    # 过滤 java 进程
    ps -eo pid,args | grep "[0-9] $java_bin " > /tmp/.mgstatus

    awk 'BEGIN{
         print "+--------------------+-----------------------------------+---------------+----------+----------------------+----------------------+---------------+"
         printf "|%-18s|%-31s|%-11s|%-8s|%-18s|%-18s|%-11s|\n","'"        模块"'","'"          微服务包"'","'"    启动状态"'","'" 进程 PID"'","'"      监听地址"'","'"      启动时间 "'","'"    API 状态   "'";
         print "+--------------------+-----------------------------------+---------------+----------+----------------------+----------------------+---------------+"
    }'

    echo > /tmp/info
    for module in ${list}; do
        local package=${module//-/_}
        local package=${!package}

        local status="     $(red_echo ×)      "
        local pid= time= listen_port=
        local curl_status="      $(red_echo ×)      "
        if [[ "$module" =~ ^(web-bim|platform-h5|web|backstage-web)$ ]]; then
            if [[ "$module" == "web" ]] || [[ "$module" == "backstage-web" ]]; then
                workdir=dist
            elif [[ "$module" == "web-bim" ]]; then
                workdir=bim-web
            else
                workdir=${module}
            fi 

            # 获取模块下 version 版本信息与 mango/version 是否一致
            [ -f "${module}/${workdir}/version" ] && local_version=$(cat ${module}/${workdir}/version)
            if [ "${local_version}" == "${package}" ]; then
                status="     $(green_echo √)      "
            fi

            pid=$(awk 'NR==1{print $1}' < <(ps -C nginx --no-header))
            if [ -n "${pid}" ]; then
                time=$(date -d "$(ps -eo pid,lstart | grep -w "${pid}" | awk '{$1="";sub(" ", "");print}')" +'%F %T')
		listen_port=${server_ip}:$(ss -ntulp | grep -w "${pid}" | awk 'NR==1 && /LISTEN/{gsub(/\*|::/, "", $5); print $5}' | awk -F: '{print $2}')
            fi

            curl_status="    UNLL     "

            awk 'BEGIN{
                printf "|%-20s|%-35s|%-15s|%-10s|%-22s|%-22s|%-9s|\n","'" ${module}"'","'" ${package}"'","'" ${status}  "'","'" ${pid}"'","'" "${listen_port}"  "'","'" ${time}  "'","'" ${curl_status} "'";
                print "+--------------------+-----------------------------------+---------------+----------+----------------------+----------------------+---------------+";
            }'
        else
            if grep -q "${package}" /tmp/.mgstatus; then
                status="     $(green_echo √)      "
                pid_list=$(awk "/ ${package}/{print \$1}"  /tmp/.mgstatus)
                for pid in $pid_list; do
		    listen_port=${server_ip}:$(ss -ntulp | grep -w "${pid}" | awk '/LISTEN/{gsub(/\*|::/, "", $5); print $5}' | awk -F: '{print $2}')
                    time=$(date -d "$(ps -eo pid,lstart | grep -w "${pid}" | awk '{$1="";sub(" ", "");print}')" +'%F %T')

                    if [[ ! "$module" =~ ^(calc-root|calc-task|backstage)$ ]]; then
                        server_port=${listen_port/0.0.0.0/${server_ip}}
                        if [ -n "${server_port}" ]; then
                            curl -sS -u admin:admin http://${server_port}/management/health | grep -qPo '"UP"'
                            if [ $? -eq 0 ]; then
                                curl_status="      $(green_echo √)      "
                            else
                                curl_status="      $(red_echo ×)      "
                            fi
                        else
                            curl_status="    UNLL     "
                        fi
                    else
                        curl_status="    UNLL     "
                    fi

                    awk 'BEGIN{
                        printf "|%-20s|%-35s|%-15s|%-10s|%-22s|%-22s|%-9s|\n","'" ${module}"'","'" ${package}"'","'" ${status}  "'","'" ${pid}"'","'" ${listen_port}  "'","'" ${time}  "'","'" ${curl_status} "'";
                        print "+--------------------+-----------------------------------+---------------+----------+----------------------+----------------------+---------------+";
                    }'

                    # tmp
                    server_port=${listen_port/0.0.0.0/${server_ip}}
                    awk 'BEGIN{
                        printf "%-20s %-4s \n","'" ${module}  "'","'" curl -sS -u admin:admin 'http://${server_port}/management/health' "'";
                    }' >> /tmp/info
                done
            else
                # 如果服务没有运行
                awk 'BEGIN{
                    printf "|%-20s|%-35s|%-15s|%-10s|%-22s|%-22s|%-9s|\n","'" ${module}"'","'" ${package}"'","'" ${status}  "'","'" ${pid}"'","'" ${listen_port}  "'","'" ${time}  "'","'" ${curl_status} "'";
                    print "+--------------------+-----------------------------------+---------------+----------+----------------------+----------------------+---------------+";
                }'

                # tmp
                server_port=${listen_port/0.0.0.0/${server_ip}}
                awk 'BEGIN{
                    printf "%-20s %-4s \n","'" ${module}  "'","'" curl -sS -u admin:admin 'http://${server_port}/management/health' "'";
                }' >> /tmp/info
            fi
        fi
    done
}

# 服务名称
name=${2%*/}

# 动作(停止/启动/下载)
action=$1

# 下载时指定微服务版本号
version=$3

case $action in
    start)
        if [ -n "${version}" ]; then
            check_lock ${name}
            start_module ${name} system
        else
            start_module ${name}
        fi
        ;;
    stop)
        stop_module ${name}
        ;;
    restart)
        stop_module ${name}
        sleep 2
        start_module ${name}
        ;;
    download)
        download ${name} ${version}
        ;;
    update)
        update ${name} ${version}
        ;;
    status)
        if [ -n "${name}" ]; then
            status_table
        else
            status
        fi
        ;;
    *)
        usage
        ;;
esac
